      subroutine cancuts(p1,ps1,p2,ps2,p3,ps3,p4,ps4,p5,ps5,p6,ps6,
     &                                                    p7,ps7,cross)
c----------------------------------------------------------------------c
c Imposes the canonical cuts on the process 
c
c  e+(p1) + e-(p2) --> f1u(p3) + f1d~(p4) + f2d(p5) + f2u~(p6) + (gamma(p7))
c
c
c Input: icrit - process class,
c        irad = 0/1/2 for Born/soft/hard,
c        ctlb,ctlq,ctgb,ctgl,ctgq,ecutg,ecutl,ecutq,mqq - the canonical cuts,
c        p1,ps1,p2,ps2,p3,ps3,p4,ps4,p5,ps5,p6,ps6,p7,ps7 - four momenta,
c
c Output: cross=1.d0/0.d0 if the canonical cuts are satisfied/unsatisfied
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 13 April, 2000; modified on 19 October, 2001
c----------------------------------------------------------------------c

      use drivec   ! Defines parameters of distributions
      use cutvalues

      implicit none

      real(kind(1.d0)) :: cross,srr
      real(kind(1.d0)), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7
      real(kind(1.d0)), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7

      cross=1.d0

      if (icrit == 1) then

       if(irad < 2) then
        if (abs(ps5(1)) > ctlb) then
         cross=0.d0
        else if (dot_product(p5(1:),p3(1:))/ps5(0)/ps3(0) > ctlq) then
         cross=0.d0
        else if (dot_product(p5(1:),p4(1:))/ps5(0)/ps4(0) > ctlq) then
         cross=0.d0
        else if (p5(0) < ecutl) then
         cross=0.d0
        else if (p3(0) < ecutq) then
         cross=0.d0
        else if (p4(0) < ecutq) then
         cross=0.d0
        else if (srr(p3+p4,p3+p4) < mqq2) then
         cross=0.d0
        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0
        else if (p5(0) < ecutl) then
         cross=0.d0
        else if (p3(0) < ecutq) then
         cross=0.d0
        else if (p4(0) < ecutq) then
         cross=0.d0
        else if (srr(p3+p4,p3+p4) < mqq2) then
         cross=0.d0
        else if (abs(ps5(1)) > ctlb) then
         cross=0.d0
        else if (dot_product(p5(1:),p3(1:))/ps5(0)/ps3(0) > 
     &                                                    ctlq) then
         cross=0.d0
        else if (dot_product(p5(1:),p4(1:))/ps5(0)/ps4(0) > 
     &                                                    ctlq) then
         cross=0.d0
        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0
        else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > 
     &                                                    ctgl) then
         cross=0.d0
        else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > 
     &                                                    ctgq) then
         cross=0.d0
        else if (dot_product(p7(1:),p4(1:))/ps7(0)/ps4(0) > 
     &                                                    ctgq) then
         cross=0.d0
        end if
       end if ! irad

      else if (icrit == 2) then

       if(irad < 2) then
        if (p3(0) < ecutq) then
         cross=0.d0
        else if (p4(0) < ecutq) then
         cross=0.d0
        else if (p5(0) < ecutq) then
         cross=0.d0
        else if (p6(0) < ecutq) then
         cross=0.d0
        else if (srr(p3+p4,p3+p4) < mqq2) then
         cross=0.d0
        else if (srr(p3+p5,p3+p5) < mqq2) then
         cross=0.d0
        else if (srr(p3+p6,p3+p6) < mqq2) then
         cross=0.d0
        else if (srr(p4+p5,p4+p5) < mqq2) then
         cross=0.d0
        else if (srr(p4+p6,p4+p6) < mqq2) then
         cross=0.d0
        else if (srr(p5+p6,p5+p6) < mqq2) then
         cross=0.d0
        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0
        else if (p3(0) < ecutq) then
         cross=0.d0
        else if (p4(0) < ecutq) then
         cross=0.d0
        else if (p5(0) < ecutq) then
         cross=0.d0
        else if (p6(0) < ecutq) then
         cross=0.d0
        else if (srr(p3+p4,p3+p4) < mqq2) then
         cross=0.d0

        else if (srr(p3+p5,p3+p5) < mqq2) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mqq2) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mqq2) then
         cross=0.d0

        else if (srr(p4+p6,p4+p6) < mqq2) then
         cross=0.d0

        else if (srr(p5+p6,p5+p6) < mqq2) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > 
     &                                                    ctgq) then
         cross=0.d0

        else if (dot_product(p7(1:),p4(1:))/ps7(0)/ps4(0) > 
     &                                                    ctgq) then
         cross=0.d0

        else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > 
     &                                                    ctgq) then
         cross=0.d0

        else if (dot_product(p7(1:),p6(1:))/ps7(0)/ps6(0) > 
     &                                                    ctgq) then
         cross=0.d0

        end if
       end if ! irad

      else if (icrit == 3) then

       if(irad < 2) then
        if (p4(0) < ecutl) then
         cross=0.d0

        else if (p5(0) < ecutl) then
         cross=0.d0

        else if (abs(ps4(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps5(1)) > ctlb) then
         cross=0.d0

        else if (dot_product(p4(1:),p5(1:))/ps4(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mll2) then
         cross=0.d0

        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0

        else if (p4(0) < ecutl) then
         cross=0.d0

        else if (p5(0) < ecutl) then
         cross=0.d0

        else if (abs(ps4(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps5(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        else if (dot_product(p4(1:),p5(1:))/ps4(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (dot_product(p7(1:),p4(1:))/ps7(0)/ps4(0) > 
     &                                                    ctgl) then
         cross=0.d0

        else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > 
     &                                                    ctgl) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mll2) then
         cross=0.d0

        end if
       end if ! irad

      else if (icrit == 4) then

       if(irad < 2) then
        if (abs(ps4(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps5(1)) > ctlb) then
         cross=0.d0

        else if (dot_product(p4(1:),p3(1:))/ps4(0)/ps3(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p5(1:),p3(1:))/ps5(0)/ps3(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p4(1:),p6(1:))/ps4(0)/ps6(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p5(1:),p6(1:))/ps5(0)/ps6(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p4(1:),p5(1:))/ps4(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (p4(0) < ecutl) then
         cross=0.d0

        else if (p5(0) < ecutl) then
         cross=0.d0

        else if (p3(0) < ecutq) then
         cross=0.d0

        else if (p6(0) < ecutq) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mqq2) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mll2) then
         cross=0.d0

        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0

        else if (p4(0) < ecutl) then
         cross=0.d0

        else if (p5(0) < ecutl) then
         cross=0.d0

        else if (p3(0) < ecutq) then
         cross=0.d0

        else if (p6(0) < ecutq) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mqq2) then
         cross=0.d0

        else if (abs(ps4(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps5(1)) > ctlb) then
         cross=0.d0

        else if (dot_product(p4(1:),p3(1:))/ps4(0)/ps3(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p5(1:),p3(1:))/ps5(0)/ps3(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p4(1:),p6(1:))/ps4(0)/ps6(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p5(1:),p6(1:))/ps5(0)/ps6(0) > ctlq) then
         cross=0.d0

        else if (dot_product(p4(1:),p5(1:))/ps4(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        else if (dot_product(p7(1:),p4(1:))/ps7(0)/ps4(0) > 
     &                                                    ctgl) then
         cross=0.d0

        else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > 
     &                                                    ctgl) then
         cross=0.d0

        else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > 
     &                                                    ctgq) then
         cross=0.d0

        else if (dot_product(p7(1:),p6(1:))/ps7(0)/ps6(0) > 
     &                                                    ctgq) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mll2) then
         cross=0.d0

        end if
       end if ! irad

      else if (icrit == 5) then

       if(irad < 2) then
        if (p3(0) < ecutq) then
         cross=0.d0

        else if (p6(0) < ecutq) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mqq2) then
         cross=0.d0

        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0

        else if (p3(0) < ecutq) then
         cross=0.d0

        else if (p6(0) < ecutq) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mqq2) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > 
     &                                                    ctgq) then
         cross=0.d0

        else if (dot_product(p7(1:),p6(1:))/ps7(0)/ps6(0) > 
     &                                                    ctgq) then
         cross=0.d0

        end if
       end if ! irad

      else if (icrit == 6) then

       if(irad < 2) then
        if (abs(ps3(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps4(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps5(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps6(1)) > ctlb) then
         cross=0.d0

        else if (p3(0) < ecutl) then
         cross=0.d0

        else if (p4(0) < ecutl) then
         cross=0.d0

        else if (p5(0) < ecutl) then
         cross=0.d0

        else if (p6(0) < ecutl) then
         cross=0.d0

        else if (dot_product(p3(1:),p4(1:))/ps3(0)/ps4(0) > ctll) then
         cross=0.d0

        else if (dot_product(p3(1:),p5(1:))/ps3(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (dot_product(p3(1:),p6(1:))/ps3(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (dot_product(p4(1:),p5(1:))/ps4(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (dot_product(p4(1:),p6(1:))/ps4(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (dot_product(p5(1:),p6(1:))/ps5(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (srr(p3+p4,p3+p4) < mll2) then
         cross=0.d0

        else if (srr(p3+p5,p3+p5) < mll2) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mll2) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mll2) then
         cross=0.d0

        else if (srr(p4+p6,p4+p6) < mll2) then
         cross=0.d0

        else if (srr(p5+p6,p5+p6) < mll2) then
         cross=0.d0

        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0

        else if (p3(0) < ecutl) then
         cross=0.d0

        else if (p4(0) < ecutl) then
         cross=0.d0

        else if (p5(0) < ecutl) then
         cross=0.d0

        else if (p6(0) < ecutl) then
         cross=0.d0

        else if (abs(ps3(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps4(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps5(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps6(1)) > ctlb) then
         cross=0.d0

        else if (dot_product(p3(1:),p4(1:))/ps3(0)/ps4(0) > ctll) then
         cross=0.d0

        else if (dot_product(p3(1:),p5(1:))/ps3(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (dot_product(p3(1:),p6(1:))/ps3(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (dot_product(p4(1:),p5(1:))/ps4(0)/ps5(0) > ctll) then
         cross=0.d0

        else if (dot_product(p4(1:),p6(1:))/ps4(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (dot_product(p5(1:),p6(1:))/ps5(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > ctgl) then
         cross=0.d0

        else if (dot_product(p7(1:),p4(1:))/ps7(0)/ps4(0) > ctgl) then
         cross=0.d0

        else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > ctgl) then
         cross=0.d0

        else if (dot_product(p7(1:),p6(1:))/ps7(0)/ps6(0) >  ctgl) then
         cross=0.d0

        else if (srr(p3+p4,p3+p4) < mll2) then
         cross=0.d0

        else if (srr(p3+p5,p3+p5) < mll2) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mll2) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mll2) then
         cross=0.d0

        else if (srr(p4+p6,p4+p6) < mll2) then
         cross=0.d0

        else if (srr(p5+p6,p5+p6) < mll2) then
         cross=0.d0

        end if
       end if ! irad

      else if (icrit == 7) then

       if(irad < 2) then
        if (abs(ps3(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps6(1)) > ctlb) then
         cross=0.d0

        else if (p3(0) < ecutl) then
         cross=0.d0

        else if (p6(0) < ecutl) then
         cross=0.d0

        else if (dot_product(p3(1:),p6(1:))/ps3(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mll2) then
         cross=0.d0

        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0

        else if (p3(0) < ecutl) then
         cross=0.d0

        else if (p6(0) < ecutl) then
         cross=0.d0

        else if (abs(ps3(1)) > ctlb) then
         cross=0.d0

        else if (abs(ps6(1)) > ctlb) then
         cross=0.d0

        else if (dot_product(p3(1:),p6(1:))/ps3(0)/ps6(0) > ctll) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > 
     &                                                    ctgl) then
         cross=0.d0

        else if (dot_product(p7(1:),p6(1:))/ps7(0)/ps6(0) > 
     &                                                    ctgl) then
         cross=0.d0

        else if (srr(p3+p6,p3+p6) < mll2) then
         cross=0.d0

        end if
       end if ! irad

      else if (icrit == 8) then

       if(irad == 2) then
        if (p7(0) < ecutg) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        end if
       end if ! irad

      else if (icrit == 9) then

       if(irad < 2) then
        if (p4(0) < ecutq) then
         cross=0.d0

        else if (p5(0) < ecutq) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mqq2) then
         cross=0.d0

        end if
       else ! irad=2
        if (p7(0) < ecutg) then
         cross=0.d0

        else if (p4(0) < ecutq) then
         cross=0.d0

        else if (p5(0) < ecutq) then
         cross=0.d0

        else if (srr(p4+p5,p4+p5) < mqq2) then
         cross=0.d0

        else if (abs(ps7(1)) > ctgb) then
         cross=0.d0

        else if (dot_product(p7(1:),p4(1:))/ps7(0)/ps4(0) > 
     &                                                    ctgq) then
         cross=0.d0

        else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > 
     &                                                    ctgq) then
         cross=0.d0

        end if
       end if ! irad

      else

       write(*,*)'CANONICAL CUTS ARE NOT DEFINED FOR THIS PROCESS'

      end if ! icrit

      end 


