      subroutine cseeww(ecm,mw,gamw,mz,gamz,ew,sw2,convc,csect)
c----------------------------------------------------------------------c
c The unpolarized cross section of 
c
c                        e+ e- --> W+ W-
c 
c Input parameters:
c        ecm - the cms energy,
c        mw - W boson mass
c        gamw - W boson width,
c        mz - Z boson mass,
c        gamz - Z boson width,
c        ew - electric charge at 2*mw,
c        sw2 - electroweak mixing angle,
c        convc - conversion constant to pb.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 11 September, 1998
c----------------------------------------------------------------------c
      implicit none
      integer, parameter :: dp=kind(1.d0)

      real(dp) ecm,mw,gamw,mz,gamz,ew,sw2,convc,csect

      real(dp) pi,mw2,mz2,coef0a,es,beta,s10m,s10p,t10m,sprim,
     &        g1,g2,g3,sigmal,sigmar


      pi=4.d0*atan(1.d0)
      mw2=mw**2
      mz2=mz**2
      coef0a=convc*(ew**2/4.d0/pi)**2/4.d0*2.d0*pi


      es=ecm**2

      BETA=DSQRT(1D0-4D0*MW2/ES)
      S10M=-(1D0-1D0/(1D0-MZ2/ES))/ES-0.5D0/SW2/ES/(1D0-MZ2/ES)
      S10P=-(1D0-1D0/(1D0-MZ2/ES))/ES
      T10M=-0.5D0/SW2/MW2
      SPRIM=ES/MW2

      G1=SPRIM/3D0*(SPRIM**3+16D0*SPRIM**2-68D0*SPRIM-48D0)
      G2=(SPRIM**2+20D0*SPRIM-48D0+48D0*(1D0-2D0/SPRIM)/BETA
     &    *DLOG((1D0+BETA)/(1D0-BETA)))/3D0
      G3=-2D0/3D0*(SPRIM**3+18D0*SPRIM**2-28D0*SPRIM-24D0
     &    -48D0*(2D0+1D0/SPRIM)/BETA*DLOG((1D0+BETA)/(1D0-BETA)))

      SIGMAR=S10P**2*G1
      SIGMAL=S10M**2*G1+T10M**2*G2+S10M*T10M*G3


      csect=coef0a*mw2**2/4.d0*BETA/ES*(SIGMAR+SIGMAL)

      end
