       function ddilog(x)                            
C  ******************************************************************
C  *                                                                *
C  *                 program for calculating                        *
C  *      the real part of the dilogarithm for real arguments       *
C  *                                                                *
C  *      functions: ddilog(x)                                      *
C  *                 rli2(x)                                        *
C  *                 clausen2(phi)                                  *
C  *                 cl2(phi)                                       *
C  *                                                                *
C  *           F. Jegerlehner, Paul Scherrer Institute              *
C  *                                                                *
C  *                     Version: 25-OCT-1990                       *
C  *                                                                *
C  ******************************************************************
c Modified by KK on 26 June, 2001
      implicit none
      real(kind(1.d0)) :: x,r
      real(kind(1.d0)) :: ddilog,rli2,sparg,omx,clo
c      common /polylog1/pi6,null,one,two,half,qua
c      save   /polylog1/
      real(kind(1.d0)), parameter :: pi6=1.644934066848226d0
      real(kind(1.d0)), parameter :: null=0.d0,one=1.d0,two=2.d0,
     &                               half=.5d0,qua=.25d0
c       data null,one,two,half,qua/0.d0,1.d0,2.d0,.5d0,.25d0/
       ddilog=0.d0
       if (x.eq.one) then
         ddilog=pi6
       return
       endif
       r=dabs(x)
       if (r.le.half) then
          ddilog=rli2(x)
          return
       else if (x.le.-two) then
          sparg=one/x 
          ddilog=-rli2(sparg)-half*dlog(-x)**2-pi6
          return
       else if (x.ge.two) then
          sparg=one/x 
          ddilog=-rli2(sparg)-half*dlog(x)**2+two*pi6
          return
       else if (x.lt.-one) then 
          sparg=one/(one-x)
          ddilog=rli2(sparg)-dlog(-x)*dlog(one-x)
     &           +half*dlog(one-x)**2-pi6
          return
       else if (x.lt.-half) then
          omx =one-x 
          sparg=one-one/omx                            
          ddilog=-rli2(sparg)-half*dlog(omx)**2
          return
       else if (x.gt.one) then
          clo=dlog(x) 
          sparg=one-one/x 
          ddilog=rli2(sparg)+half*clo**2
     &          -clo*dlog(x-one)+pi6
          return
       else if (x.gt.half) then 
          sparg=one-x                            
          ddilog=-rli2(sparg)-dlog(x)*dlog(sparg)+pi6
          return
       endif
       end

      function rli2(x)
c Spence function for real arguments of modulus smaller than one
      real (kind(1.d0)) :: x,rli2,b,z,z2
      integer :: ini=0
      dimension b(10)
c      common /polylog2/one,half,qua,b,ini
c      save   /polylog2/
c      data ini/0/,one,half,qua/1.d0,.5d0,.25d0/
      real(kind(1.d0)), parameter :: one=1.d0,half=.5d0,qua=.25d0
      save b,ini

      if (ini.eq.0) goto 2
 1    z=-dlog(one-x)
      z2=z*z
      rli2=z*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*b(10)+b(9))
     &  +b(8))+b(7))+b(6))+b(5))+b(4))+b(3))+b(2))+b(1))
     &  +one-z*qua)
      return
 2       b(1) =  2.7777777777777778d-02
         b(2) = -2.7777777777777778d-04
         b(3) =  4.7241118669690098d-06
         b(4) = -9.1857730746619635d-08
         b(5) =  1.8978869988970999d-09
         b(6) = -4.0647616451442255d-11
         b(7) =  8.9216910204564526d-13
         b(8) = -1.9939295860721076d-14
         b(9) =  4.5189800296199182d-16
         b(10)= -1.0356517612181247d-17
      ini=1
      goto 1
      end

      function clausen2(phi)
c Clausens integral for arbitrary real arguments ( defined as the
c imaginary part of the complex Spence function on the unit circle 
c z=exp(i*phi) )  (2pi-periodic,odd)
      implicit none
      real (kind(1.d0)) :: phi,phiabs,cl2,clausen2
c      common /polylog3/null,one,pi,zpi
c      save   /polylog3/
c      data pi /3.141592653589793d0/,zpi /6.283185307179586d0/    
      real(kind(1.d0)), parameter :: pi=3.141592653589793d0,
     &                       zpi=6.283185307179586d0,
     &                       null=0.d0,one=1.d0
c      data null,one/0.d0,1.d0/
      phi=dmod(phi,zpi)
      if (phi.gt.pi) phi=phi-zpi
      phiabs=dabs(phi)
      if (phi.eq.null) then 
         clausen2=null
      else
         clausen2=phi/phiabs*cl2(phiabs)
      endif
      return
      end
         
      function cl2(phi)        
c Clausens integral for real arguments 0<phi<pi
      implicit none
      real (kind(1.d0)) :: b(15),phi,z,z2,cl2
      integer :: ini=0
c      common /polylog4/one,pi,pi2,b,ini
c      save   /polylog4/
c      data pi /3.141592653589793d0/,pi2 /1.570796326794897d0/    
c      data ini/0/,one/1.d0/
      real(kind(1.d0)), parameter :: pi=3.141592653589793d0,
     &                      pi2=1.570796326794897d0,
     &                      one=1.d0
      save ini,b

      if (ini.eq.0) goto 2
 1    z=phi
      z2=z*z
      if (phi.le.pi2) then 
      if (phi.eq.0d0) then
      cl2=0d0
      return
      endif
      cl2= z*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2* b(10)+ b(9))
     &  + b(8))+ b(7))+ b(6))+ b(5))+ b(4))+ b(3))+ b(2))+ b(1))
     &  +one-dlog(dabs(z)))
      else                                       
      cl2= z*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*(z2*
     & (z2*(z2* b(15)+ b(14))+ b(13))+ b(12))+ b(11))+ b(10))+ b(9))
     &  + b(8))+ b(7))+ b(6))+ b(5))+ b(4))+ b(3))+ b(2))+ b(1))
     &  +one-dlog(dabs(z)))
      endif
      return
 2       b(1) =  1.3888888888888889E-02
         b(2) =  6.9444444444444444E-05
         b(3) =  7.8735197782816830E-07
         b(4) =  1.1482216343327454E-08
         b(5) =  1.8978869988970999E-10
         b(6) =  3.3873013709535213E-12
         b(7) =  6.3726364431831804E-14
         b(8) =  1.2462059912950673E-15
         b(9) =  2.5105444608999546E-17
         b(10)=  5.1782588060906234E-19
         b(11)=  1.0887357368300848E-20
         b(12)=  2.3257441143020872E-22
         b(13)=  5.0351952131473897E-24
         b(14)=  1.1026499294381215E-25
         b(15)=  2.4386585509007344E-27
      ini=1
      goto 1
      end
                                 
