      subroutine eeff1(p1,p2,p3,p4,ps1,ps2,ps3,ps4,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c              e+(p1) + e-(p2) -> f(p3+p4) + f~(p5+p6)
c
c Input: pi(0:3), i=1,...,4 - external particle momenta,
c        psi(0:4), i=1,...,4 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 3 November, 2000
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc

      implicit none
      integer, parameter :: dp=kind(1.d0)

      real(dp) mat2
      real(dp), dimension(0:3) :: p1,p2,p3,p4
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4
      integer i1,i2,i3,i4,i

      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,
     &                              yp1,yp2,yp3,yp4

      complex(dp), dimension(4,n1) :: u1
      complex(dp), dimension(4,n2) :: u2
      complex(dp), dimension(4,2) :: u3
      complex(dp), dimension(4,2) :: u4

      complex(dp), dimension(0:3,n1,n2) :: ea12,ez12
      complex(dp), dimension(2,2,n1,n2) :: xa12,xz12,
     &                                    ya12,yz12

      complex(dp), dimension(2,n1,n2,2) :: amp

c External particle momenta in the spinor representation:

      call spinr(p1,xp1,yp1)
      call spinr(p2,xp2,yp2)
      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)

c Spinors of the external fermions:

      call spinornew(p1,ps1,m1,-1,1,n1,u1)
      call spinornew(p2,ps2,m2,1,0,n2,u2)
      call spinornew(p3,ps3,m3,1,1,2,u3)
      call spinornew(p4,ps4,m3,-1,0,2,u4)

c Building blocks of the diagrams:

c Vector boson couplings sandwiched between spinors:

      call fvf(u1,n1,-p1,m1,ga1m,ga1p,zero,ma2,u2,n2,-p2,m2,
     &                                                 ea12,xa12,ya12)
      call fvf(u1,n1,-p1,m1,gz1m,gz1p,zero,mz2,u2,n2,-p2,m2,
     &                                                 ez12,xz12,yz12)

c Amplitudes:

      call fef(u3,ga3p*xa12+gz3p*xz12,ga3m*ya12+gz3m*yz12,
     &         onec,onec,u4,amp,2,n1*n2,2)


c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,n1
      do i2=1,n2
      do i3=1,2
      do i4=1,2

       mat2=mat2+abs(amp(i3,i1,i2,i4))**2

      end do
      end do
      end do
      end do

      mat2=colfac*mat2/dble(n1*n2)

      end 



