      subroutine eeffffcc1(iproc,iww,p1,p2,p3,p4,p5,p6,
     &                             ps1,ps2,ps3,ps4,ps5,ps6,mat2,mat2ww)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c      e+(p1) + e-(p2) --> f1u(p3) + f1d~(p4) + f2d(p5) + f2u~(p6)
c
c for the charged current processes not containing virtual gluons
c and the matrix element squared in the double resonant approximation 
c mat2ww
c
c Input: iproc -specifies the proces,
c        iww = 1 (the double resonance diagrams only), else (all the 
c              diagrams),
c        pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 18 January, 2000; modified 20 January, 2000; 21 June, 2000;
c                            15 February, 2001; 11 June, 2001;
c                            20 June, 2003
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use inistate

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer iproc,iww
      real(dp) :: mat2,mat2ww,t52,srr,ebeam=0.d0
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      integer i1,i2,i3,i4,i5,i6,i

      complex(dp) :: mat,mww,scc

      complex(dp), dimension(2,2) :: xp3,xp4,xp5,xp6,
     &                              yp3,yp4,yp5,yp6

      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,
     &                                    ya36,yz36
      complex(dp), dimension(0:3,n5,n4) :: ea54,ez54
      complex(dp), dimension(2,2,n5,n4) :: xa54,xz54,ya54,yz54
      complex(dp), dimension(0:3,n3,n4) :: ew34
      complex(dp), dimension(0:3,n5,n6) :: ew56
      complex(dp), dimension(2,2,n3,n4) :: xw34,yw34
      complex(dp), dimension(2,2,n5,n6) :: xw56,yw56

      complex(dp), dimension(0:3,n1,n6) :: ew16
      complex(dp), dimension(2,2,n1,n6) :: xw16,yw16
      complex(dp), dimension(0:3,n5,n2) :: ea52,ez52
      complex(dp), dimension(2,2,n5,n2) :: xa52,ya52,xz52,yz52

      complex(dp), dimension(n5,n4) :: h54

      complex(dp), dimension(n1,n3,n4,n5,n6,n2) :: amp1
      complex(dp), dimension(n5,n6,n3,n4,n1,n2) :: amp2
      complex(dp), dimension(n3,n1,n2,n5,n6,n4) :: amp3
      complex(dp), dimension(n3,n5,n6,n1,n2,n4) :: amp4
      complex(dp), dimension(n5,n1,n2,n3,n4,n6) :: amp5
      complex(dp), dimension(n5,n3,n4,n1,n2,n6) :: amp6
      complex(dp), dimension(n1,n5,n4,n3,n6,n2) :: amp7
      complex(dp), dimension(n1,n3,n6,n5,n4,n2) :: amp8
      complex(dp), dimension(n3,n1,n2,n5,n4,n6) :: amp9
      complex(dp), dimension(n3,n5,n4,n1,n2,n6) :: amp10
      complex(dp), dimension(n5,n1,n2,n3,n6,n4) :: amp11
      complex(dp), dimension(n5,n3,n6,n1,n2,n4) :: amp12
      complex(dp), dimension(n1,n6,n3,n4,n5,n2) :: amp17
      complex(dp), dimension(n5,n3,n4,n1,n6,n2) :: amp18
      complex(dp), dimension(n3,n5,n2,n1,n6,n4) :: amp19
      complex(dp), dimension(n3,n1,n6,n5,n2,n4) :: amp20
      complex(dp), dimension(n1,n5,n2,n3,n4,n6) :: amp21
      complex(dp), dimension(n1,n3,n4,n5,n2,n6) :: amp22

      save ebeam

      if (p1(0).ne.ebeam) then
       call inipart(p1,p2,ps1,ps2)
       ebeam=p1(0)
      end if

c External particle momenta in the spinor representation:

      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)

c Spinors of the external fermions:

      call spinornew(p3,ps3,m3,1,1,n3,u3)
      call spinornew(p4,ps4,m4,-1,0,n4,u4)
      call spinornew(p5,ps5,m5,1,1,n5,u5)
      call spinornew(p6,ps6,m6,-1,0,n6,u6)

c Building blocks of the diagrams:

c Vector boson couplings sandwiched between spinors:

      call fvf(u3,n3,p3,m3,wff,ocom,zero,mw2,u4,n4,p4,m4,
     &                                                 ew34,xw34,yw34)
      call fvf(u5,n5,p5,m5,wff,ocom,zero,mw2,u6,n6,p6,m6,
     &                                                 ew56,xw56,yw56)
      if (iproc > 15) then

      t52=-2.d0*ps2(0)*ps5(0)*(1.d0+ps5(1)
     &    +(m2*(p2(0)-p5(0))/(p2(0)*p5(0)))**2/2.d0)

       call fvf(u1,n1,-p1,m1,wff,ocom,zero,mw2,u6,n6,p6,m6,
     &                                                 ew16,xw16,yw16)
       call fvf(u5,n5,p5,m5,ga5m,ga5p,t52,ma2,u2,n2,-p2,m2,
     &                                                 ea52,xa52,ya52)
       call fvf(u5,n5,p5,m5,gz5m,gz5p,t52,mz2,u2,n2,-p2,m2,
     &                                                 ez52,xz52,yz52)
      end if ! iproc > 15

      if (iww /= 1) then
      if (iproc > 10 .and. iproc < 16) then
       if (ga3m /= ocom) then
        call fvf(u3,n3,p3,m3,ga3m,ga3p,zero,ma2,u6,n6,p6,m6,
     &                                                 ea36,xa36,ya36)
       else
        xa36=ocom
        ya36=ocom
       end if ! ga3m
       call fvf(u3,n3,p3,m3,gz3m,gz3p,zero,mz2,u6,n6,p6,m6,
     &                                                 ez36,xz36,yz36)
       call fvf(u5,n5,p5,m5,ga5m,ga5p,zero,ma2,u4,n4,p4,m4,
     &                                                 ea54,xa54,ya54)
       call fvf(u5,n5,p5,m5,gz5m,gz5p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez54,xz54,yz54)
       if (ihiggs == 1) then
        call fhf(u5,n5,p5,hff(5),zero,mh2,u4,n4,p4,h54)
       end if
      end if
      end if ! iww

c Amplitudes:

      call fefef(u1,-p1,xw34,yw34,wff,ocom,
     &          -xp1+xp3+xp4,-yp1+yp3+yp4,ocom,ocom,
     &           xw56,yw56,wff,ocom,u2,-p2,amp1,n1,n3*n4,n5*n6,n2)

      call eee(p5+p6,ew56,-p3-p4,ew34,p1+p2,
     &                       gww*ea12+zww*ez12,amp2,n5*n6,n3*n4,n1*n2)

      if (iproc > 15) then

      call eee(-p1+p6,ew16,-p3-p4,ew34,p2-p5,
     &                       gww*ea52+zww*ez52,amp17,n1*n6,n3*n4,n5*n2)

      call fefef(u5,p5,xw34,yw34,wff,ocom,
     &           xp5+xp3+xp4,yp5+yp3+yp4,ocom,ocom,
     &           xw16,yw16,wff,ocom,u2,-p2,amp18,n5,n3*n4,n1*n6,n2)

      end if ! iproc > 15


      if (iww /= 1) then

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           xw56,yw56,wff,ocom,u4,p4,amp3,n3,n1*n2,n5*n6,n4)

      call fefef(u3,p3,xw56,yw56,wff,ocom,
     &           xp3+xp5+xp6,yp3+yp5+yp6,cm4,ocom,
     &        ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,u4,p4,
     &                                         amp4,n3,n5*n6,n1*n2,n4)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,ga5m*ya12+gz5m*yz12,
     &           onec,onec,xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,
     &           xw34,yw34,wff,ocom,u6,p6,amp5,n5,n1*n2,n3*n4,n6)

      call fefef(u5,p5,xw34,yw34,wff,ocom,
     &           xp5+xp3+xp4,yp5+yp3+yp4,cm6,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,p6,amp6,n5,n3*n4,n1*n2,n6)

      if (iproc > 10 .and. iproc < 16) then

      call fefef(u1,-p1,ga1p*xa54+gz1p*xz54,
     &                                ga1m*ya54+gz1m*yz54,onec,onec,
     &          -xp1+xp5+xp4,-yp1+yp5+yp4,cm1,ocom,
     &           ga2p*xa36+gz2p*xz36,ga2m*ya36+gz2m*yz36,onec,onec,
     &                                u2,-p2,amp7,n1,n5*n4,n3*n6,n2)

      call fefef(u1,-p1,ga1p*xa36+gz1p*xz36,
     &                                ga1m*ya36+gz1m*yz36,onec,onec,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &           ga2p*xa54+gz2p*xz54,ga2m*ya54+gz2m*yz54,onec,onec,
     &                                u2,-p2,amp8,n1,n3*n6,n5*n4,n2)

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,onec,onec,
     &                                u6,p6,amp9,n3,n1*n2,n5*n4,n6)

      call fefef(u3,p3,ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,
     &           onec,onec,xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,p6,amp10,n3,n5*n4,n1*n2,n6)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,
     &                                ga5m*ya12+gz5m*yz12,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm5,ocom,
     &           ga4p*xa36+gz4p*xz36,ga4m*ya36+gz4m*yz36,onec,onec,
     &                                u4,p4,amp11,n5,n1*n2,n3*n6,n4)


      call fefef(u5,p5,ga5p*xa36+gz5p*xz36,
     &                                ga5m*ya36+gz5m*yz36,onec,onec,
     &           xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &           ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                                u4,p4,amp12,n5,n3*n6,n1*n2,n4)

      end if  ! iproc

      if (iproc > 15) then

      call fefef(u3,p3,ga3p*xa52+gz3p*xz52,ga3m*ya52+gz3m*yz52,
     &           onec,onec,xp3+xp5-xp2,yp3+yp5-yp2,cm3,ocom,
     &           xw16,yw16,wff,ocom,u4,p4,amp19,n3,n5*n2,n1*n6,n4)

      call fefef(u3,p3,xw16,yw16,wff,ocom,
     &           xp3+xp6-xp1,yp3+yp6-yp1,cm4,ocom,
     &           ga4p*xa52+gz4p*xz52,ga4m*ya52+gz4m*yz52,onec,onec,
     &                              u4,p4,amp20,n3,n1*n6,n5*n2,n4)

      call fefef(u1,-p1,ga1p*xa52+gz1p*xz52,
     &                                ga1m*ya52+gz1m*yz52,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,
     &           xw34,yw34,wff,ocom,u6,p6,amp21,n1,n5*n2,n3*n4,n6)

      call fefef(u1,-p1,xw34,yw34,wff,ocom,
     &          -xp6+xp2-xp5,-yp6+yp2-yp5,cm6,ocom,
     &           xz52,yz52,gz6m,gz6p,u6,p6,amp22,n1,n3*n4,n5*n2,n6)

      end if  ! iproc
      end if    ! iww


c The spin averaged squared matrix element:

      mat2=zero
      mat2ww=zero

      do i1=1,n1
      do i2=1,n2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

c A sum of the diagrams:

      if (iproc < 11 .or. iproc > 13) then
      mat=amp1(i1,i3,i4,i5,i6,i2)
     &   +amp2(i5,i6,i3,i4,i1,i2)

      if (iproc > 15) then
      mat=mat
     &   -amp17(i1,i6,i3,i4,i5,i2)
     &   -amp18(i5,i3,i4,i1,i6,i2)
      end if ! iproc

      mww=mat

      if (iww.ne.1) then
        mat=mat                                          
     &     +amp3(i3,i1,i2,i5,i6,i4)
     &     +amp4(i3,i5,i6,i1,i2,i4)
     &     +amp5(i5,i1,i2,i3,i4,i6)
     &     +amp6(i5,i3,i4,i1,i2,i6)

      if (iproc > 13 .and. iproc < 16) then
        mat=mat                                          
     &     -amp7(i1,i5,i4,i3,i6,i2)
     &     -amp8(i1,i3,i6,i5,i4,i2)
     &     -amp9(i3,i1,i2,i5,i4,i6)
     &     -amp10(i3,i5,i4,i1,i2,i6)
     &     -amp11(i5,i1,i2,i3,i6,i4)
     &     -amp12(i5,i3,i6,i1,i2,i4)
      else if (iproc > 15) then
        mat=mat
     &     -amp19(i3,i5,i2,i1,i6,i4)
     &     -amp20(i3,i1,i6,i5,i2,i4)
     &     -amp21(i1,i5,i2,i3,i4,i6)
     &     -amp22(i1,i3,i4,i5,i2,i6)
      end if ! iproc
      end if
      if (iww /= 1) then
       if (iproc > 10 .and. iproc < 16) then
        if (ihiggs == 1) then
         mat=mat
     &      -hzz*h54(i5,i4)*scc(ez12(:,i1,i2),ez36(:,i3,i6))
        end if
       end if
      end if

      mat2=mat2+abs(mat)**2
      mat2ww=mat2ww+abs(mww)**2

      end if  ! iproc

      end do
      end do
      end do
      end do
      end do
      end do

      mat2=comf1*mat2
      mat2ww=comf1*mat2ww

      end 



