      subroutine eeffffcc1a(iproc,iww,p1,p2,p3,p4,p5,p6,
     &                             ps1,ps2,ps3,ps4,ps5,ps6,mat2,mat2ww)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c      e+(p1) + e-(p2) --> f1u(p3) + f1d~(p4) + f2d(p5) + f2u~(p6)
c
c for the charged current processes not containing virtual gluons
c and the matrix element squared in the double resonant approximation mat2ww
c
c Input: iproc -specifies the proces,
c        iww = 1 (the double resonance diagrams only), else (all the 
c              diagrams),
c        pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 18 January, 2000; modified 20 January, 2000; 21 June, 2000; 
c                            1 November, 2001
c----------------------------------------------------------------------c

      use inprms 
      use mathprms 
      use parsm
      use helproc
      use parproc
      use gaugepar
c      use topan

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer iproc,iww
      real(dp) mat2,mat2ww,t52,srr
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      integer i1,i2,i3,i4,i5,i6,i

      complex(dp) :: mat,mww,src,scc

      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,xp5,xp6,
     &                              yp1,yp2,yp3,yp4,yp5,yp6

      complex(dp), dimension(4,n1) :: u1
      complex(dp), dimension(4,n2) :: u2
      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(0:3,n1,n2) :: ea12,ez12
      complex(dp), dimension(2,2,n1,n2) :: xa12,xz12,
     &                                    ya12,yz12
      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,
     &                                    ya36,yz36
      complex(dp), dimension(0:3,n5,n4) :: ea54,ez54
      complex(dp), dimension(2,2,n5,n4) :: xa54,xz54,ya54,yz54
      complex(dp), dimension(0:3,n3,n4) :: ew34
      complex(dp), dimension(0:3,n5,n6) :: ew56
      complex(dp), dimension(2,2,n3,n4) :: xw34,yw34
      complex(dp), dimension(2,2,n5,n6) :: xw56,yw56

      complex(dp), dimension(0:3,n1,n6) :: ew16
      complex(dp), dimension(2,2,n1,n6) :: xw16,yw16
      complex(dp), dimension(0:3,n5,n2) :: ea52,ez52
      complex(dp), dimension(2,2,n5,n2) :: xa52,ya52,xz52,yz52

      complex(dp), dimension(n1,n3,n4,n5,n6,n2) :: amp1,am1
      complex(dp), dimension(n5,n6,n3,n4,n1,n2) :: amp2,am2
      complex(dp), dimension(n3,n1,n2,n5,n6,n4) :: amp3,am3
      complex(dp), dimension(n3,n5,n6,n1,n2,n4) :: amp4,am4
      complex(dp), dimension(n5,n1,n2,n3,n4,n6) :: amp5,am5
      complex(dp), dimension(n5,n3,n4,n1,n2,n6) :: amp6,am6
      complex(dp), dimension(n1,n5,n4,n3,n6,n2) :: amp7,am7
      complex(dp), dimension(n1,n3,n6,n5,n4,n2) :: amp8,am8
      complex(dp), dimension(n3,n1,n2,n5,n4,n6) :: amp9,am9
      complex(dp), dimension(n3,n5,n4,n1,n2,n6) :: amp10,am10
      complex(dp), dimension(n5,n1,n2,n3,n6,n4) :: amp11,am11
      complex(dp), dimension(n5,n3,n6,n1,n2,n4) :: amp12,am12
      complex(dp), dimension(n1,n6,n3,n4,n5,n2) :: amp17,am17
      complex(dp), dimension(n5,n3,n4,n1,n6,n2) :: amp18,am18
      complex(dp), dimension(n3,n5,n2,n1,n6,n4) :: amp19,am19
      complex(dp), dimension(n3,n1,n6,n5,n2,n4) :: amp20,am20
      complex(dp), dimension(n1,n5,n2,n3,n4,n6) :: amp21,am21
      complex(dp), dimension(n1,n3,n4,n5,n2,n6) :: amp22,am22

      complex(dp) :: u134(4,n1*n3*n4),u534(4,n5*n3*n4),u356(4,n3*n5*n6),
     &              u312(4,n3*n1*n2),u512(4,n5*n1*n2),u352(4,n3*n5*n2),
     &              u316(4,n3*n1*n6),u152(4,n1*n5*n2),
     &              es12(n1,n2),eh12(n1,n2),es34(n3,n4),es56(n5,n6),
     &              es16(n1,n6),es52(n5,n2),eh52(n5,n2),es54(n5,n4),
     &              es36(n3,n6)

      complex(dp) :: v134(4,n1*n3*n4),v534(4,n5*n3*n4),v356(4,n3*n5*n6),
     &              v312(4,n3*n1*n2),v512(4,n5*n1*n2),v352(4,n3*n5*n2),
     &              v316(4,n3*n1*n6),v152(4,n1*n5*n2)

c External particle momenta in the spinor representation:

      call spinr(p1,xp1,yp1)
      call spinr(p2,xp2,yp2)
      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)

c Spinors of the external fermions:

      call spinornew(p1,ps1,m1,-1,1,n1,u1)
      call spinornew(p2,ps2,m2,1,0,n2,u2)
ckk>
      call spinornew(p3,ps3,m3,1,1,n3,u3)
c      call spinornw(p3,ps3,xp3,yp3,cm3,1,0,n3,u3)
ckk<
      call spinornew(p4,ps4,m4,-1,0,n4,u4)
      call spinornew(p5,ps5,m5,1,1,n5,u5)
      call spinornew(p6,ps6,m6,-1,0,n6,u6)


c Building blocks of the diagrams:

c Vector boson couplings sandwiched between spinors:

      call fvfa(u1,n1,-p1,ga1p,ga1m,zero,ma2,ksia,u2,n2,-p2,
     &                                                 ea12,xa12,ya12)

      call fvfa(u1,n1,-p1,gz1p,gz1m,zero,mz2,ksiz,u2,n2,-p2,
     &                                                 ez12,xz12,yz12)

      if (ksiz < 1.d16) then
       call fsfa(u1,n1,-p1,-cm1*c3ff,cm1*c3ff,zero,mz2,ksiz,u2,n2,-p2,
     &                                                           es12)
      end if

      if (ihiggs == 1) then
       call fsfa(u1,n1,-p1,cm1*chff,cm1*chff,zero,mh2,one,u2,n2,-p2,
     &                                                           eh12)
      end if
      call fvfa(u3,n3,p3,ocom,wff,zero,mw2,ksiw,u4,n4,p4,
     &                                                 ew34,xw34,yw34)
      call fvfa(u5,n5,p5,ocom,wff,zero,mw2,ksiw,u6,n6,p6,
     &                                                 ew56,xw56,yw56)

      if (ksiw < 1.d16) then
      call fsfa(u3,n3,p3,-cm4*csff,cm3*csff,zero,mw2,ksiw,u4,n4,p4,es34)
      call fsfa(u5,n5,p5,cm6*csff,-cm5*csff,zero,mw2,ksiw,u6,n6,p6,es56)
      end if

      if (iproc > 15) then

      t52=-2.d0*ps2(0)*ps5(0)*(1.d0+ps5(1)
     &    +(m2*(p2(0)-p5(0))/(p2(0)*p5(0)))**2/2.d0)

       call fvfa(u1,n1,-p1,ocom,wff,zero,mw2,ksiw,u6,n6,p6,
     &                                                 ew16,xw16,yw16)
       call fvfa(u5,n5,p5,ga5p,ga5m,t52,ma2,ksia,u2,n2,-p2,
     &                                                 ea52,xa52,ya52)
       call fvfa(u5,n5,p5,gz5p,gz5m,t52,mz2,ksiz,u2,n2,-p2,
     &                                                 ez52,xz52,yz52)
       if (ksiw < 1.d16) then
        call fsfa(u1,n1,-p1,cm6*csff,-cm1*csff,zero,mw2,ksiw,u6,n6,p6,
     &                                                             es16)
       end if
       if (ksiz < 1.d16) then
        call fsfa(u5,n5,p5,-cm2*c3ff,cm2*c3ff,t52,mz2,ksiz,u2,n2,-p2,
     &                                                             es52)
       end if
      if (ihiggs == 1) then
        call fsfa(u5,n5,p5,cm2*chff,cm2*chff,t52,mh2,one,u2,n2,-p2,eh52)
      end if
      end if ! iproc > 15

      if (iww /= 1) then
      if (iproc > 10 .and. iproc < 16) then
       if (ga3m /= ocom) then
        call fvfa(u3,n3,p3,ga3p,ga3m,zero,ma2,ksia,u6,n6,p6,
     &                                                 ea36,xa36,ya36)
       else
        xa36=ocom
        ya36=ocom
       end if ! ga3m
       call fvfa(u3,n3,p3,gz3p,gz3m,zero,mz2,ksiz,u6,n6,p6,
     &                                                 ez36,xz36,yz36)
       call fvfa(u5,n5,p5,ga5p,ga5m,zero,ma2,ksia,u4,n4,p4,
     &                                                 ea54,xa54,ya54)
       call fvfa(u5,n5,p5,gz5p,gz5m,zero,mz2,ksiz,u4,n4,p4,
     &                                                 ez54,xz54,yz54)

      if (ksiz < 1.d16) then
       call fsfa(u3,n3,p3,cm3*c3ff,-cm3*c3ff,zero,mz2,ksiz,u6,n6,p6,
     &                                                           es36)
       call fsfa(u5,n5,p5,-cm5*c3ff,cm5*c3ff,zero,mz2,ksiz,u4,n4,p4,
     &                                                           es54)
      end if
      end if
      end if ! iww

c Amplitudes:

      call uep(u1,xw34,yw34,ocom,wff,
     &        -xp1+xp3+xp4,-yp1+yp3+yp4,ocom,ocom,u134,n1,n3*n4)

      call fef(u134,xw56,yw56,ocom,wff,u2,amp1,n1*n3*n4,n5*n6,n2)

      if (ksiw < 1.d16) then
       call fsf(u134,es56,-cm2*csff,ocom,u2,am1,n1*n3*n4,n5*n6,n2)
       amp1=amp1+am1
       call usp(u1,es34,ocom,-cm1*csff,
     &        -xp1+xp3+xp4,-yp1+yp3+yp4,ocom,ocom,v134,n1,n3*n4)
       call fef(v134,xw56,yw56,ocom,wff,u2,am1,n1*n3*n4,n5*n6,n2)
       amp1=amp1+am1
       call fsf(v134,es56,-cm2*csff,ocom,u2,am1,n1*n3*n4,n5*n6,n2)
       amp1=amp1+am1
      end if

      call eee(p5+p6,ew56,-p3-p4,ew34,p1+p2,
     &                    gww*ea12+zww*ez12,amp2,n5*n6,n3*n4,n1*n2)

      call uep(u5,xw34,yw34,ocom,wff,
     &           xp5+xp3+xp4,yp5+yp3+yp4,cm6,ocom,u534,n5,n3*n4)

      if (ksiw < 1.d16) then
       call usp(u5,es34,ocom,-cm5*csff,
     &           xp5+xp3+xp4,yp5+yp3+yp4,cm6,ocom,v534,n5,n3*n4)
      end if

      if (iproc > 15) then

      call eee(-p1+p6,ew16,-p3-p4,ew34,p2-p5,
     &                   gww*ea52+zww*ez52,amp17,n1*n6,n3*n4,n5*n2)

      call fef(u534,xw16,yw16,ocom,wff,u2,amp18,n5*n3*n4,n1*n6,n2)

      if (ksiw < 1.d16) then
       call fsf(u534,es16,-cm2*csff,ocom,u2,am18,n5*n3*n4,n1*n6,n2)
       amp18=amp18+am18
       call fef(v534,xw16,yw16,ocom,wff,u2,am18,n5*n3*n4,n1*n6,n2)
       amp18=amp18+am18
       call fsf(v534,es16,-cm2*csff,ocom,u2,am18,n5*n3*n4,n1*n6,n2)
       amp18=amp18+am18
      end if
      end if ! iproc > 15


      do i1=1,n1
      do i2=1,n2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

       if (ihiggs == 1) then
        amp2(i5,i6,i3,i4,i1,i2)=amp2(i5,i6,i3,i4,i1,i2)
     &         +hww*eh12(i1,i2)*scc(ew34(:,i3,i4),ew56(:,i5,i6))
       end if

      if (ksiw < 1.d16) then
       amp2(i5,i6,i3,i4,i1,i2)=amp2(i5,i6,i3,i4,i1,i2)
     & +scc(saw*ea12(:,i1,i2)+szw*ez12(:,i1,i2),ew56(:,i5,i6))
     &                                               *es34(i3,i4)
     & +scc(saw*ea12(:,i1,i2)+szw*ez12(:,i1,i2),ew34(:,i3,i4))
     &                                               *es56(i5,i6)
     & +src(p5+p6-p3-p4,ass*ea12(:,i1,i2)+zss*ez12(:,i1,i2))
     &                                   *es34(i3,i4)*es56(i5,i6)
     & +w3s*es12(i1,i2)
     &               *(src(-p1-p2-p3-p4,ew56(:,i5,i6))*es34(i3,i4)
     &                +src(-p1-p2-p5-p6,ew34(:,i3,i4))*es56(i5,i6))

       if (ihiggs == 1) then
        amp2(i5,i6,i3,i4,i1,i2)=amp2(i5,i6,i3,i4,i1,i2)
     & +eh12(i1,i2)*(hss*es34(i3,i4)*es56(i5,i6)
     & +whs*(src(-p1-p2-p3-p4,ew56(:,i5,i6))*es34(i3,i4)
     &       -src(-p1-p2-p5-p6,ew34(:,i3,i4))*es56(i5,i6)))
       end if
      end if

      if (iproc > 15) then

       if (ihiggs == 1) then
        amp17(i1,i6,i3,i4,i5,i2)=amp17(i1,i6,i3,i4,i5,i2)
     &         +hww*eh52(i5,i2)*scc(ew34(:,i3,i4),ew16(:,i1,i6))
       end if
      if (ksiw < 1.d16) then
       amp17(i1,i6,i3,i4,i5,i2)=amp17(i1,i6,i3,i4,i5,i2)
     & +scc(saw*ea52(:,i5,i2)+szw*ez52(:,i5,i2),ew16(:,i1,i6))
     &                                               *es34(i3,i4)
     & +scc(saw*ea52(:,i5,i2)+szw*ez52(:,i5,i2),ew34(:,i3,i4))
     &                                               *es16(i1,i6)
     & +src(-p1+p6-p3-p4,ass*ea52(:,i5,i2)+zss*ez52(:,i5,i2))
     &                                   *es34(i3,i4)*es16(i1,i6)
     & +w3s*es52(i5,i2)*
     &               (src(p5-p2-p3-p4,ew16(:,i1,i6))*es34(i3,i4)
     &               +src(p5-p2+p1-p6,ew34(:,i3,i4))*es16(i1,i6))

       if (ihiggs == 1) then
        amp17(i1,i6,i3,i4,i5,i2)=amp17(i1,i6,i3,i4,i5,i2)
     &         +eh52(i5,i2)*(hss*es34(i3,i4)*es16(i1,i6)
     & +whs*(src(p5-p2-p3-p4,ew16(:,i1,i6))*es34(i3,i4)
     &       -src(p5-p2+p1-p6,ew34(:,i3,i4))*es16(i1,i6)))
       end if
      end if

      end if ! iproc > 15

      end do
      end do
      end do
      end do
      end do
      end do


      if (iww /= 1) then

      call uep(u3,ga3p*xa12+gz3p*xz12,ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,u312,n3,n1*n2)

c      if (iwtb == 1) then
c       call fefa(u312,xw56,yw56,wff,
c     &          -xp5-xp6,-yp5-yp6,f1p,f1m,f2p/mw,f2m/mw,
c     &                                      u4,amp3,n3*n1*n2,n5*n6,n4)
c      else
       call fef(u312,xw56,yw56,ocom,wff,u4,amp3,n3*n1*n2,n5*n6,n4)
c      end if ! iwtb

      if (ksiw < 1.d16) then
       call fsf(u312,es56,-cm4*csff,cm3*csff,u4,am3,n3*n1*n2,n5*n6,n4)
       amp3=amp3+am3
      end if
      if (ksiz < 1.d16) then
       call usp(u3,es12,cm3*c3ff,-cm3*c3ff,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,v312,n3,n1*n2)
       call fef(v312,xw56,yw56,ocom,wff,u4,am3,n3*n1*n2,n5*n6,n4)
       amp3=amp3+am3
       call fsf(v312,es56,-cm4*csff,cm3*csff,u4,am3,n3*n1*n2,n5*n6,n4)
       amp3=amp3+am3
      end if
      if (ihiggs == 1) then
       call usp(u3,eh12,cm3*chff,cm3*chff,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,v312,n3,n1*n2)

       call fef(v312,xw56,yw56,ocom,wff,u4,am3,n3*n1*n2,n5*n6,n4)
       amp3=amp3+am3
      end if

      if (ksiw < 1.d16) then
       call fsf(v312,es56,-cm4*csff,cm3*csff,u4,am3,n3*n1*n2,n5*n6,n4)
       amp3=amp3+am3
      end if
      call uep(u3,xw56,yw56,ocom,wff,
     &           xp3+xp5+xp6,yp3+yp5+yp6,cm4,ocom,u356,n3,n5*n6)

      call fef(u356,ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                                      u4,amp4,n3*n5*n6,n1*n2,n4)

      if (ksiz < 1.d16) then
       call fsf(u356,es12,-cm4*c3ff,cm4*c3ff,u4,am4,n3*n5*n6,n1*n2,n4)
       amp4=amp4+am4
      end if

      if (ihiggs == 1) then
       call fsf(u356,eh12,cm4*chff,cm4*chff,u4,am4,n3*n5*n6,n1*n2,n4)
       amp4=amp4+am4
      end if

      if (ksiw < 1.d16) then
       call usp(u3,es56,-cm4*csff,cm3*csff,
     &           xp3+xp5+xp6,yp3+yp5+yp6,cm4,ocom,v356,n3,n5*n6)
       call fef(v356,ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                                      u4,am4,n3*n5*n6,n1*n2,n4)
       amp4=amp4+am4
       call fsf(v356,es12,-cm4*c3ff,cm4*c3ff,u4,am4,n3*n5*n6,n1*n2,n4)
       amp4=amp4+am4
       if (ihiggs == 1) then
        call fsf(v356,eh12,cm4*chff,cm4*chff,u4,am4,n3*n5*n6,n1*n2,n4)
        amp4=amp4+am4
       end if
      end if

      call uep(u5,ga5p*xa12+gz5p*xz12,ga5m*ya12+gz5m*yz12,onec,onec,
     &           xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,u512,n5,n1*n2)

      call fef(u512,xw34,yw34,ocom,wff,u6,amp5,n5*n1*n2,n3*n4,n6)

      if (ksiw < 1.d16) then
       call fsf(u512,es34,cm6*csff,-cm5*csff,u6,am5,n5*n1*n2,n3*n4,n6)
       amp5=amp5+am5
      end if

      if (ksiz < 1.d16) then
       call usp(u5,es12,-cm5*c3ff,cm5*c3ff,
     &           xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,v512,n5,n1*n2)
       call fef(v512,xw34,yw34,ocom,wff,u6,am5,n5*n1*n2,n3*n4,n6)
       amp5=amp5+am5
       call fsf(v512,es34,cm6*csff,-cm5*csff,u6,am5,n5*n1*n2,n3*n4,n6)
       amp5=amp5+am5
      end if

      if (ihiggs == 1) then
       call usp(u5,eh12,cm5*chff,cm5*chff,
     &           xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,v512,n5,n1*n2)

       call fef(v512,xw34,yw34,ocom,wff,u6,am5,n5*n1*n2,n3*n4,n6)
       amp5=amp5+am5
      end if

      if (ksiw < 1.d16) then
       call fsf(v512,es34,cm6*csff,-cm5*csff,u6,am5,n5*n1*n2,n3*n4,n6)
       amp5=amp5+am5
      end if

      call fef(u534,ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,amp6,n5*n3*n4,n1*n2,n6)

      if (ksiz < 1.d16) then
       call fsf(u534,es12,cm6*c3ff,-cm6*c3ff,u6,am6,n5*n3*n4,n1*n2,n6)
       amp6=amp6+am6
      end if

      if (ihiggs == 1) then
       call fsf(u534,eh12,cm6*chff,cm6*chff,u6,am6,n5*n3*n4,n1*n2,n6)
       amp6=amp6+am6
      end if

      if (ksiw < 1.d16) then
       call fef(v534,ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,am6,n5*n3*n4,n1*n2,n6)
       amp6=amp6+am6
       call fsf(v534,es12,cm6*c3ff,-cm6*c3ff,u6,am6,n5*n3*n4,n1*n2,n6)
       amp6=amp6+am6
      end if

      if (ihiggs == 1) then
       call fsf(u534,eh12,cm6*chff,cm6*chff,u6,am6,n5*n3*n4,n1*n2,n6)
       amp6=amp6+am6
      end if

      if (iproc > 10 .and. iproc < 16) then

      call fefef(u1,-p1,ga1p*xa54+gz1p*xz54,
     &                                ga1m*ya54+gz1m*yz54,one,one,
     &          -xp1+xp5+xp4,-yp1+yp5+yp4,m1,gam1,ocom,
     &           ga2p*xa36+gz2p*xz36,ga2m*ya36+gz2m*yz36,one,one,
     &                                u2,-p2,amp7,n1,n5*n4,n3*n6,n2)

      call fefef(u1,-p1,ga1p*xa36+gz1p*xz36,
     &                                ga1m*ya36+gz1m*yz36,one,one,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,m1,gam1,ocom,
     &           ga2p*xa54+gz2p*xz54,ga2m*ya54+gz2m*yz54,one,one,
     &                                u2,-p2,amp8,n1,n3*n6,n5*n4,n2)

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,one,one,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,m3,gam3,ocom,
     &           ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,one,one,
     &                                u6,p6,amp9,n3,n1*n2,n5*n4,n6)

      call fefef(u3,p3,ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,one,one,
     &           xp3+xp5+xp4,yp3+yp5+yp4,m3,gam3,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,one,one,
     &                                u6,p6,amp10,n3,n5*n4,n1*n2,n6)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,
     &                                ga5m*ya12+gz5m*yz12,one,one,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,m5,gam5,ocom,
     &           ga4p*xa36+gz4p*xz36,ga4m*ya36+gz4m*yz36,one,one,
     &                                u4,p4,amp11,n5,n1*n2,n3*n6,n4)


      call fefef(u5,p5,ga5p*xa36+gz5p*xz36,
     &                                ga5m*ya36+gz5m*yz36,one,one,
     &           xp5+xp3+xp6,yp5+yp3+yp6,m5,gam5,ocom,
     &           ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,one,one,
     &                                u4,p4,amp12,n5,n3*n6,n1*n2,n4)

      end if  ! iproc

      if (iproc > 15) then

      call uep(u3,ga3p*xa52+gz3p*xz52,ga3m*ya52+gz3m*yz52,onec,onec,
     &           xp3+xp5-xp2,yp3+yp5-yp2,cm3,ocom,u352,n3,n5*n2)

      call fef(u352,xw16,yw16,ocom,wff,u4,amp19,n3*n5*n2,n1*n6,n4)

      if (ksiw < 1.d16) then
       call fsf(u352,es16,-cm4*csff,cm3*csff,u4,am19,n3*n5*n2,n1*n6,n4)
       amp19=amp19+am19
      end if

      if (ksiz < 1.d16) then
       call usp(u3,es52,cm3*c3ff,-cm3*c3ff,
     &           xp3+xp5-xp2,yp3+yp5-yp2,cm3,ocom,v352,n3,n5*n2)
       call fef(v352,xw16,yw16,ocom,wff,u4,am19,n3*n5*n2,n1*n6,n4)
       amp19=amp19+am19
       call fsf(v352,es16,-cm4*csff,cm3*csff,u4,am19,n3*n5*n2,n1*n6,n4)
       amp19=amp19+am19
      end if

      if (ihiggs == 1) then
       call usp(u3,eh52,cm3*chff,cm3*chff,
     &           xp3+xp5-xp2,yp3+yp5-yp2,cm3,ocom,v352,n3,n5*n2)

       call fef(v352,xw16,yw16,ocom,wff,u4,am19,n3*n5*n2,n1*n6,n4)
       amp19=amp19+am19
      end if

      if (ksiw < 1.d16) then
       call fsf(v352,es16,-cm4*csff,cm3*csff,u4,am19,n3*n5*n2,n1*n6,n4)
       amp19=amp19+am19
      end if

      call uep(u3,xw16,yw16,ocom,wff,
     &           xp3+xp6-xp1,yp3+yp6-yp1,cm4,ocom,u316,n3,n1*n6)

      call fef(u316,ga4p*xa52+gz4p*xz52,ga4m*ya52+gz4m*yz52,onec,onec,
     &                              u4,amp20,n3*n1*n6,n5*n2,n4)

      if (ksiz < 1.d16) then
       call fsf(u316,es52,-cm4*c3ff,cm4*c3ff,
     &                              u4,am20,n3*n1*n6,n5*n2,n4)
       amp20=amp20+am20
      end if
      if (ksiw < 1.d16) then
       call usp(u3,es16,-cm4*csff,cm3*csff,
     &           xp3+xp6-xp1,yp3+yp6-yp1,cm4,ocom,v316,n3,n1*n6)
       call fef(v316,ga4p*xa52+gz4p*xz52,ga4m*ya52+gz4m*yz52,onec,onec,
     &                              u4,am20,n3*n1*n6,n5*n2,n4)
       amp20=amp20+am20
       call fsf(v316,es52,-cm4*c3ff,cm4*c3ff,
     &                              u4,am20,n3*n1*n6,n5*n2,n4)
       amp20=amp20+am20
       if (ihiggs == 1) then
        call fsf(v316,es52,cm4*chff,cm4*chff,
     &                              u4,am20,n3*n1*n6,n5*n2,n4)
        amp20=amp20+am20
       end if 
      end if

      call uep(u1,ga1p*xa52+gz1p*xz52,ga1m*ya52+gz1m*yz52,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,u152,n1,n5*n2)

      call fef(u152,xw34,yw34,ocom,wff,u6,amp21,n1*n5*n2,n3*n4,n6)

      if (ksiw < 1.d16) then
       call fsf(u152,es34,cm6*csff,-cm1*csff,u6,am21,n1*n5*n2,n3*n4,n6)
       amp21=amp21+am21
      end if
      if (ksiz < 1.d16) then
       call usp(u1,es52,-cm1*c3ff,cm1*c3ff,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,v152,n1,n5*n2)
       call fef(v152,xw34,yw34,ocom,wff,u6,am21,n1*n5*n2,n3*n4,n6)
       amp21=amp21+am21
       call fsf(v152,es34,cm6*csff,-cm1*csff,u6,am21,n1*n5*n2,n3*n4,n6)
       amp21=amp21+am21
      end if

      if (ihiggs == 1) then
       call usp(u1,eh52,cm1*chff,cm1*chff,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,v152,n1,n5*n2)

       call fef(v152,xw34,yw34,ocom,wff,u6,am21,n1*n5*n2,n3*n4,n6)
       amp21=amp21+am21
      end if

      if (ksiw < 1.d16) then
       call fsf(v152,es34,cm6*csff,-cm1*csff,u6,am21,n1*n5*n2,n3*n4,n6)
       amp21=amp21+am21
      end if

      call fef(u134,xz52,yz52,gz6p,gz6m,u6,amp22,n1*n3*n4,n5*n2,n6)

      if (ksiw < 1.d16) then
       call fef(v134,xz52,yz52,gz6p,gz6m,u6,am22,n1*n3*n4,n5*n2,n6)
       amp22=amp22+am22
      end if

      end if  ! iproc
      end if    ! iww


c The spin averaged squared matrix element:

      mat2=zero
      mat2ww=zero

      do i1=1,n1
      do i2=1,n2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

c A sum of the diagrams:

      if (iproc < 11 .or. iproc > 13) then

      mww=amp1(i1,i3,i4,i5,i6,i2)
     &   +amp2(i5,i6,i3,i4,i1,i2)

      if (iproc > 15) then
      mww=mww
     &   -amp17(i1,i6,i3,i4,i5,i2)
     &   -amp18(i5,i3,i4,i1,i6,i2)
      end if ! iproc

      mat=mww

      if (iww.ne.1) then
        mat=mat                                          
     &     +amp3(i3,i1,i2,i5,i6,i4)
     &     +amp4(i3,i5,i6,i1,i2,i4)
     &     +amp5(i5,i1,i2,i3,i4,i6)
     &     +amp6(i5,i3,i4,i1,i2,i6)
      if (iproc > 13 .and. iproc < 16) then
        mat=mat                                          
     &     -amp7(i1,i5,i4,i3,i6,i2)
     &     -amp8(i1,i3,i6,i5,i4,i2)
     &     -amp9(i3,i1,i2,i5,i4,i6)
     &     -amp10(i3,i5,i4,i1,i2,i6)
     &     -amp11(i5,i1,i2,i3,i6,i4)
     &     -amp12(i5,i3,i6,i1,i2,i4)
      else if (iproc > 15) then
        mat=mat
     &     -amp19(i3,i5,i2,i1,i6,i4)
     &     -amp20(i3,i1,i6,i5,i2,i4)
     &     -amp21(i1,i5,i2,i3,i4,i6)
     &     -amp22(i1,i3,i4,i5,i2,i6)
      end if ! iproc
      end if

      mat2=mat2+abs(mat)**2
      mat2ww=mat2ww+abs(mww)**2

      end if  ! iproc

      end do
      end do
      end do
      end do
      end do
      end do

      mat2=comf1*mat2
      mat2ww=comf1*mat2ww

      end 



