      subroutine eeffffcc2(iew,iww,p1,p2,p3,p4,p5,p6,
     &                            ps1,ps2,ps3,ps4,ps5,ps6,mat2,mat2ww)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours for 
c
c      e+(p1) + e-(p2) --> f1u(p3) + f1d~(p4) + f2d(p5) + f2u~(p6)
c
c for the charged current processes not containing virtual gluons
c and the matrix element squared in the double resonant approximation 
c mat2ww
c
c Input: iew = 1 (only electroweak contributions)/ else (ew + QCD)
c        iww = 1 (the double resonance diagrams only), else (all the 
c                 diagrams),
c        pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 18 January, 2000; modified 20 January, 2000; 21 June, 2000;
c                            15 February, 2001; 28 March, 2001;
c                            1 November, 2001; 20 June, 2003
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use inistate

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer iew,iww
      real(dp) :: mat2,mat2ww,srr,ebeam=0.d0
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      integer i1,i2,i3,i4,i5,i6,i

      complex(dp) :: mat,mww

      complex(dp), dimension(2,2) :: xp3,xp4,xp5,xp6,
     &                              yp3,yp4,yp5,yp6

      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,xg36,
     &                                    ya36,yz36,yg36
      complex(dp), dimension(0:3,n5,n4) :: ea54,ez54
      complex(dp), dimension(2,2,n5,n4) :: xa54,xz54,xg54,
     &                                    ya54,yz54,yg54
      complex(dp), dimension(0:3,n3,n4) :: ew34
      complex(dp), dimension(0:3,n5,n6) :: ew56
      complex(dp), dimension(2,2,n3,n4) :: xw34,yw34
      complex(dp), dimension(2,2,n5,n6) :: xw56,yw56

      complex(dp), dimension(n1,n3,n4,n5,n6,n2) :: amp1
      complex(dp), dimension(n5,n6,n3,n4,n1,n2) :: amp2
      complex(dp), dimension(n3,n1,n2,n5,n6,n4) :: amp3
      complex(dp), dimension(n3,n5,n6,n1,n2,n4) :: amp4
      complex(dp), dimension(n5,n1,n2,n3,n4,n6) :: amp5
      complex(dp), dimension(n5,n3,n4,n1,n2,n6) :: amp6
      complex(dp), dimension(n1,n5,n4,n3,n6,n2) :: amp7
      complex(dp), dimension(n1,n3,n6,n5,n4,n2) :: amp8
      complex(dp), dimension(n3,n1,n2,n5,n4,n6) :: amp9
      complex(dp), dimension(n3,n5,n4,n1,n2,n6) :: amp10
      complex(dp), dimension(n5,n1,n2,n3,n6,n4) :: amp11
      complex(dp), dimension(n5,n3,n6,n1,n2,n4) :: amp12
      complex(dp), dimension(n3,n1,n2,n5,n4,n6) :: amp13
      complex(dp), dimension(n3,n5,n4,n1,n2,n6) :: amp14
      complex(dp), dimension(n5,n1,n2,n3,n6,n4) :: amp15
      complex(dp), dimension(n5,n3,n6,n1,n2,n4) :: amp16

      save ebeam

      if (p1(0).ne.ebeam) then
       call inipart(p1,p2,ps1,ps2)
       ebeam=p1(0)
      end if

c External particle momenta in the spinor representation:

      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)

c Spinors of the external fermions:

      call spinornew(p3,ps3,m3,1,1,n3,u3)
      call spinornew(p4,ps4,m4,-1,0,n4,u4)
      call spinornew(p5,ps5,m5,1,1,n5,u5)
      call spinornew(p6,ps6,m6,-1,0,n6,u6)

c Building blocks of the diagrams:

c Vector boson couplings sandwiched between spinors:

      call fvf(u3,n3,p3,m3,wff,ocom,zero,mw2,u4,n4,p4,m4,
     &                                                 ew34,xw34,yw34)
      call fvf(u5,n5,p5,m5,wff,ocom,zero,mw2,u6,n6,p6,m6,
     &                                                 ew56,xw56,yw56)

      if (iww /= 1) then
       if (ga3m /= ocom) then
        call fvf(u3,n3,p3,m3,ga3m,ga3p,zero,ma2,u6,n6,p6,m6,
     &                                                 ea36,xa36,ya36)
       else
        xa36=ocom
        ya36=ocom
       end if ! ga3m
       call fvf(u3,n3,p3,m3,gz3m,gz3p,zero,mz2,u6,n6,p6,m6,
     &                                                 ez36,xz36,yz36)
       call fvf(u5,n5,p5,m5,ga5m,ga5p,zero,ma2,u4,n4,p4,m4,
     &                                                 ea54,xa54,ya54)
       call fvf(u5,n5,p5,m5,gz5m,gz5p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez54,xz54,yz54)
      end if ! iww

c Amplitudes:

      call fefef(u1,-p1,xw34,yw34,wff,ocom,
     &          -xp1+xp3+xp4,-yp1+yp3+yp4,ocom,ocom,
     &           xw56,yw56,wff,ocom,u2,-p2,amp1,n1,n3*n4,n5*n6,n2)

      call eee(p5+p6,ew56,-p3-p4,ew34,p1+p2,
     &                       gww*ea12+zww*ez12,amp2,n5*n6,n3*n4,n1*n2)


      if (iww /= 1) then

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           xw56,yw56,wff,ocom,u4,p4,amp3,n3,n1*n2,n5*n6,n4)

      call fefef(u3,p3,xw56,yw56,wff,ocom,
     &           xp3+xp5+xp6,yp3+yp5+yp6,cm4,ocom,
     &        ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,u4,p4,
     &                                         amp4,n3,n5*n6,n1*n2,n4)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,ga5m*ya12+gz5m*yz12,
     &           onec,onec,xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,
     &           xw34,yw34,wff,ocom,u6,p6,amp5,n5,n1*n2,n3*n4,n6)

      call fefef(u5,p5,xw34,yw34,wff,ocom,
     &           xp5+xp3+xp4,yp5+yp3+yp4,cm6,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,p6,amp6,n5,n3*n4,n1*n2,n6)

      call fefef(u1,-p1,ga1p*xa54+gz1p*xz54,
     &                                ga1m*ya54+gz1m*yz54,onec,onec,
     &          -xp1+xp5+xp4,-yp1+yp5+yp4,cm1,ocom,
     &           ga2p*xa36+gz2p*xz36,ga2m*ya36+gz2m*yz36,onec,onec,
     &                                u2,-p2,amp7,n1,n5*n4,n3*n6,n2)

      call fefef(u1,-p1,ga1p*xa36+gz1p*xz36,
     &                                ga1m*ya36+gz1m*yz36,onec,onec,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &           ga2p*xa54+gz2p*xz54,ga2m*ya54+gz2m*yz54,onec,onec,
     &                                u2,-p2,amp8,n1,n3*n6,n5*n4,n2)

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,onec,onec,
     &                                u6,p6,amp9,n3,n1*n2,n5*n4,n6)

      call fefef(u3,p3,ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,
     &           onec,onec,xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,p6,amp10,n3,n5*n4,n1*n2,n6)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,
     &                                ga5m*ya12+gz5m*yz12,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm5,ocom,
     &           ga4p*xa36+gz4p*xz36,ga4m*ya36+gz4m*yz36,onec,onec,
     &                                u4,p4,amp11,n5,n1*n2,n3*n6,n4)


      call fefef(u5,p5,ga5p*xa36+gz5p*xz36,
     &                                ga5m*ya36+gz5m*yz36,onec,onec,
     &           xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &           ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                                u4,p4,amp12,n5,n3*n6,n1*n2,n4)


      if (iew /= 1) then

      xg36=gqq/ga3m*xa36
      yg36=gqq/ga3m*ya36
      xg54=gqq/ga5m*xa54
      yg54=gqq/ga5m*ya54

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           xg54,yg54,gqq,gqq,u6,p6,amp13,n3,n1*n2,n5*n4,n6)

      call fefef(u3,p3,xg54,yg54,gqq,gqq,
     &           xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &           ga3p*xa12+gz3p*xz12,ga3m*ya12+gz3m*yz12,onec,onec,
     &                              u6,p6,amp14,n3,n5*n4,n1*n2,n6)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,
     &                                ga5m*ya12+gz5m*yz12,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm5,ocom,
     &           xg36,yg36,gqq,gqq,u4,p4,amp15,n5,n1*n2,n3*n6,n4)

      call fefef(u5,p5,xg36,yg36,gqq,gqq,
     &           xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &           ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                                u4,p4,amp16,n5,n3*n6,n1*n2,n4)

      end if    ! iew
      end if    ! iww


c The spin averaged squared matrix element:

      mat2=zero
      mat2ww=zero

      do i1=1,n1
      do i2=1,n2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

c A sum of the diagrams:

      do i=1,2

        mww=cevec43(i,1)*amp1(i1,i3,i4,i5,i6,i2)
     &     +cevec43(i,2)*amp2(i5,i6,i3,i4,i1,i2)

        mat=mww

        if (iww.ne.1) then
         mat=mat                                          
     &      +cevec43(i,3)*amp3(i3,i1,i2,i5,i6,i4)
     &      +cevec43(i,4)*amp4(i3,i5,i6,i1,i2,i4)
     &      +cevec43(i,5)*amp5(i5,i1,i2,i3,i4,i6)
     &      +cevec43(i,6)*amp6(i5,i3,i4,i1,i2,i6)
     &      -cevec43(i,7)*amp7(i1,i5,i4,i3,i6,i2)
     &      -cevec43(i,8)*amp8(i1,i3,i6,i5,i4,i2)
     &      -cevec43(i,9)*amp9(i3,i1,i2,i5,i4,i6)
     &      -cevec43(i,10)*amp10(i3,i5,i4,i1,i2,i6)
     &      -cevec43(i,11)*amp11(i5,i1,i2,i3,i6,i4)
     &      -cevec43(i,12)*amp12(i5,i3,i6,i1,i2,i4)
      if (iew /= 1) then
         mat=mat                                          
     &      -cevec43(i,13)*amp13(i3,i1,i2,i5,i4,i6)
     &      -cevec43(i,14)*amp14(i3,i5,i4,i1,i2,i6)
     &      -cevec43(i,15)*amp15(i5,i1,i2,i3,i6,i4)
     &      -cevec43(i,16)*amp16(i5,i3,i6,i1,i2,i4)
        end if ! iew
        end if

        mat2=mat2+ceval43(i)*abs(mat)**2
        mat2ww=mat2ww+ceval43(i)*abs(mww)**2

       end do  ! i

      end do
      end do
      end do
      end do
      end do
      end do

      mat2=comf1*mat2
      mat2ww=comf1*mat2ww

      end 



