      subroutine eeffffgid3(p1,p2,p3,p4,p5,p6,p7,
     &                            ps1,ps2,ps3,ps4,ps5,ps6,ps7,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c   e+(p1) + e-(p2) --> ve(p3) + ve~(p4) + ve(p5) + ve~(p6) + gamma(p7)
c
c for the neutral current processes containing identical leptons
c
c Input: pi, i=1,...,7 - external particle momenta,
c        psi, i=1,...,7 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 21 March, 2001;  modified on 2 November, 2001
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use genpv12
      use genpv12g

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer i1,i2,i3,i4,i5,i6,i7,i

      real(dp), dimension(0:3) ::  p1,p2,p3,p4,p5,p6,p7
      real(dp), dimension(0:4) ::  ps1,ps2,ps3,ps4,ps5,ps6,ps7
      real(dp) mat2,srr,e7(0:3,n7)

      real(dp) dbe

      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,xp5,xp6,xp7,
     &                              yp1,yp2,yp3,yp4,yp5,yp6,yp7

      complex(dp), dimension(4,n1) :: u1
      complex(dp), dimension(4,n2) :: u2
      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(2,2,n7) :: xe7,ye7

      complex(dp), dimension(1,1,n7) :: h00g

      complex(dp) :: mat
      complex(dp), dimension(2,2,n3,n4,n5,n6,2) :: ml34567
      complex(dp), dimension(2,2,n5,n4,n3,n6,2) :: ml54367
      complex(dp), dimension(2,2,n3,n4,n5,n6,2) :: me34567
      complex(dp), dimension(2,2,n5,n4,n3,n6,2) :: me54367
      complex(dp), dimension(2,2,n3,n6,n5,n4,2) :: me36547
      complex(dp), dimension(2,2,n5,n6,n3,n6,2) :: me56347

      complex(dp), dimension(0:3,n5,n4) :: ea54,ez54
      complex(dp), dimension(0:3,n5,n4,n7) :: ea54g,ez54g
      complex(dp), dimension(2,2,n5,n4) :: xa54,xz54,x154,x354,
     &                                    ya54,yz54,y154,y354
      complex(dp), dimension(2,2,n5,n4,n7) :: xa54g,xz54g,x154g,x354g,
     &                                       ya54g,yz54g,y154g,y354g
      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(0:3,n3,n6,n7) :: ea36g,ez36g
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,x136,x536,
     &                                    ya36,yz36,y136,y536
      complex(dp), dimension(2,2,n3,n6,n7) :: xa36g,xz36g,x136g,x536g,
     &                                       ya36g,yz36g,y136g,y536g

c External particle momenta in the spinor representation:

      call spinr(p1,xp1,yp1)
      call spinr(p2,xp2,yp2)
      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)
      call spinr(p7,xp7,yp7)

c Spinors of the external fermions:

      call spinornew(p1,ps1,m1,-1,1,n1,u1)
      call spinornew(p2,ps2,m2,1,0,n2,u2)
      call spinornew(p3,ps3,m3,1,1,n3,u3)
      call spinornew(p4,ps4,m4,-1,0,n4,u4)
      call spinornew(p5,ps5,m5,1,1,n5,u5)
      call spinornew(p6,ps6,m6,-1,0,n6,u6)

c Photon polarization vectors:

      call recpol(p7,ps7,zero,e7)

      do i7=1,n7
ckk>
c Test gauge invariance:

c      e7(:,i7)=p7
ckk<
        call spinr(e7(:,i7),xe7(:,:,i7),ye7(:,:,i7))
      end do

c Fermion denominators:

c      dbe=1.d0-sqrt(1.d0-(m1/p1(0))**2)
      dbe=(m1/p1(0))**2/2.d0

      d17=cmplx(-2.d0*p1(0)*p7(0)*(1.d0-ps7(1)+dbe*ps7(1)),zero,dp)
c     &                                         m1*gam1,dp)
      d27=cmplx(-2.d0*p2(0)*p7(0)*(1.d0+ps7(1)-dbe*ps7(1)),zero,dp)
c     &                                         m2*gam2,dp)

      call fvf(u1,n1,-p1,m1,ga1m,ga1p,zero,ma2,u2,n2,-p2,m2,
     &                                                 ea12,xa12,ya12)
      call ffvgnew(u1,n1,-p1,-xp1,-yp1,cm1,ga1m,d17,
     &          u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,ga1m,ga1p,ma2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea12,ea12g,xa12g,ya12g,
     &          0,hff(1),mh2,h00g)

      call fvf(u1,n1,-p1,m1,gz1m,gz1p,zero,mz2,u2,n2,-p2,m2,
     &                                                 ez12,xz12,yz12)

      call ffvgnew(u1,n1,-p1,-xp1,-yp1,cm1,ga1m,d17,
     &          u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,gz1m,gz1p,mz2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea12,ez12g,xz12g,yz12g,
     &          0,hff(1),mh2,h00g)

      x312=ga3p*xa12+gz3p*xz12
      y312=ga3m*ya12+gz3m*yz12

      x412=ga4p*xa12+gz4p*xz12
      y412=ga4m*ya12+gz4m*yz12

      x512=ga5p*xa12+gz5p*xz12
      y512=ga5m*ya12+gz5m*yz12

      x612=ga6p*xa12+gz6p*xz12
      y612=ga6m*ya12+gz6m*yz12

      call matllllg(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &              p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &              p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,
     &              p7,ps7,xp7,yp7,e7,xe7,ye7,ml34567)

      call matllllg(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &              p5,ps5,xp5,yp5,u5,p4,ps4,xp4,yp4,u4,
     &              p3,ps3,xp3,yp3,u3,p6,ps6,xp6,yp6,u6,
     &              p7,ps7,xp7,yp7,e7,xe7,ye7,ml54367)

      call matnnnng(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &              p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &              p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,
     &              p7,ps7,xp7,yp7,e7,xe7,ye7,me34567)

      call matnnnng(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &              p5,ps5,xp5,yp5,u5,p4,ps4,xp4,yp4,u4,
     &              p3,ps3,xp3,yp3,u3,p6,ps6,xp6,yp6,u6,
     &              p7,ps7,xp7,yp7,e7,xe7,ye7,me54367)

      call matnnnng(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &              p3,ps3,xp3,yp3,u3,p6,ps6,xp6,yp6,u6,
     &              p5,ps5,xp5,yp5,u5,p4,ps4,xp4,yp4,u4,
     &              p7,ps7,xp7,yp7,e7,xe7,ye7,me36547)

      call matnnnng(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &              p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,
     &              p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &              p7,ps7,xp7,yp7,e7,xe7,ye7,me56347)


c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,2
      do i2=1,2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6
      do i7=1,2

      mat=ml34567(i1,i2,i3,i4,i5,i6,i7)
     &   -ml54367(i1,i2,i5,i4,i3,i6,i7)
     &   +me34567(i1,i2,i3,i4,i5,i6,i7)
     &   -me54367(i1,i2,i5,i4,i3,i6,i7)
     &   -me36547(i1,i2,i3,i6,i5,i4,i7)
     &   +me56347(i1,i2,i5,i6,i3,i4,i7)

      mat2=mat2+(abs(mat))**2

      end do
      end do
      end do
      end do
      end do
      end do
      end do

      mat2=comf2*mat2

      end 

