      subroutine eeffffgnc2(p1,p2,p3,p4,p5,p6,p7,
     &                         ps1,ps2,ps3,ps4,ps5,ps6,ps7,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c   e+(p1) + e-(p2) --> f1(p3) + e+(p4) + e-(p5) + f1~(p6) + gamma(p7)
c
c for the neutral current processes not containing virtual gluons
c
c Input: pi, i=1,...,7 - external particle momenta,
c        psi, i=1,...,7 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 12 September, 2000  modified 20 January, 2000; 21 June, 2000;
c                              15 February, 2001; 25 April, 2001;
c                              2 November, 2001; 20 June, 2003
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use inistate

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer i1,i2,i3,i4,i5,i6,i7,i

      real(dp), dimension(0:3) ::  p1,p2,p3,p4,p5,p6,p7
      real(dp), dimension(0:4) ::  ps1,ps2,ps3,ps4,ps5,ps6,ps7
      real(dp) :: mat2,srr,e7(0:3,n7),ebeam=0.d0

      real(dp) dbe,t52
      complex(dp) d17,d27,d37,d47,d57,d67

      complex(dp), dimension(2,2) :: xp3,xp4,xp5,xp6,xp7,
     &                              yp3,yp4,yp5,yp6,yp7

      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(2,2,n7) :: xe7,ye7

      complex(dp) :: mat,scc

      complex(dp), dimension(0:3,n1,n2,n7) :: ea12g,ez12g
      complex(dp), dimension(2,2,n1,n2,n7) :: xa12g,xz12g,ya12g,yz12g
      complex(dp), dimension(0:3,n1,n4) :: ea14,ez14
      complex(dp), dimension(0:3,n1,n4,n7) :: ea14g,ez14g
      complex(dp), dimension(2,2,n1,n4) :: xa14,xz14,ya14,yz14,
     &                                    x514,y514,x614,y614
      complex(dp), dimension(2,2,n1,n4,n7) :: xa14g,xz14g,
     &                                       ya14g,yz14g
      complex(dp), dimension(0:3,n5,n2) :: ea52,ez52
      complex(dp), dimension(0:3,n5,n2,n7) :: ea52g,ez52g
      complex(dp), dimension(2,2,n5,n2) :: xa52,xz52,x152,x352,
     &                                    ya52,yz52,y152,y352
      complex(dp), dimension(2,2,n5,n2,n7) :: xa52g,xz52g,x152g,x352g,
     &                                       ya52g,yz52g,y152g,y352g
      complex(dp), dimension(0:3,n5,n4) :: ea54,ez54
      complex(dp), dimension(0:3,n5,n4,n7) :: ea54g,ez54g
      complex(dp), dimension(2,2,n5,n4) :: xa54,xz54,x154,x354,
     &                                    ya54,yz54,y154,y354
      complex(dp), dimension(2,2,n5,n4,n7) :: xa54g,xz54g,x154g,x354g,
     &                                       ya54g,yz54g,y154g,y354g
      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(0:3,n3,n6,n7) :: ea36g,ez36g
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,x136,x236,x536,
     &                                    ya36,yz36,y136,y236,y536
      complex(dp), dimension(2,2,n3,n6,n7) :: xa36g,xz36g,x136g,x536g,
     &                                       ya36g,yz36g,y136g,y536g

      complex(dp), dimension(n3,n6) :: h36
      complex(dp), dimension(2,2,n7) :: h00g
      complex(dp), dimension(n3,n6,n7) :: h36g

      complex(dp), dimension(n1,n7,n5,n4,n3,n6,n2) :: m7_1
      complex(dp), dimension(n1,n5,n4,n3,n6,n7,n2) :: m7_2
      complex(dp), dimension(n1,n5,n4,n7,n3,n6,n2) :: m7_3
      complex(dp), dimension(n1,n5,n4,n7,n3,n6,n2) :: m7_4
      complex(dp), dimension(n1,n5,n4,n3,n6,n7,n2) :: m7_5
      complex(dp), dimension(n1,n7,n3,n6,n5,n4,n2) :: m8_1
      complex(dp), dimension(n1,n3,n6,n5,n4,n7,n2) :: m8_2
      complex(dp), dimension(n1,n3,n6,n7,n5,n4,n2) :: m8_3
      complex(dp), dimension(n1,n3,n6,n7,n5,n4,n2) :: m8_4
      complex(dp), dimension(n1,n3,n6,n5,n4,n7,n2) :: m8_5
      complex(dp), dimension(n3,n7,n1,n2,n5,n4,n6) :: m9_1
      complex(dp), dimension(n3,n1,n2,n5,n4,n7,n6) :: m9_2
      complex(dp), dimension(n3,n1,n2,n7,n5,n4,n6) :: m9_3
      complex(dp), dimension(n3,n1,n2,n7,n5,n4,n6) :: m9_4
      complex(dp), dimension(n3,n1,n2,n5,n4,n7,n6) :: m9_5
      complex(dp), dimension(n3,n7,n5,n4,n1,n2,n6) :: m10_1
      complex(dp), dimension(n3,n5,n4,n1,n2,n7,n6) :: m10_2
      complex(dp), dimension(n3,n5,n4,n7,n1,n2,n6) :: m10_3
      complex(dp), dimension(n3,n5,n4,n7,n1,n2,n6) :: m10_4
      complex(dp), dimension(n3,n5,n4,n1,n2,n7,n6) :: m10_5
      complex(dp), dimension(n5,n7,n1,n2,n3,n6,n4) :: m11_1
      complex(dp), dimension(n5,n1,n2,n3,n6,n7,n4) :: m11_2
      complex(dp), dimension(n5,n1,n2,n7,n3,n6,n4) :: m11_3
      complex(dp), dimension(n5,n1,n2,n7,n3,n6,n4) :: m11_4
      complex(dp), dimension(n5,n1,n2,n3,n6,n7,n4) :: m11_5
      complex(dp), dimension(n5,n7,n3,n6,n1,n2,n4) :: m12_1
      complex(dp), dimension(n5,n3,n6,n1,n2,n7,n4) :: m12_2
      complex(dp), dimension(n5,n3,n6,n7,n1,n2,n4) :: m12_3
      complex(dp), dimension(n5,n3,n6,n7,n1,n2,n4) :: m12_4
      complex(dp), dimension(n5,n3,n6,n1,n2,n7,n4) :: m12_5
      complex(dp), dimension(n3,n5,n2,n7,n1,n4,n6) :: m23_1
      complex(dp), dimension(n3,n5,n2,n1,n4,n7,n6) :: m23_2
      complex(dp), dimension(n3,n7,n5,n2,n1,n4,n6) :: m23_3
      complex(dp), dimension(n3,n5,n2,n7,n1,n4,n6) :: m23_4
      complex(dp), dimension(n3,n5,n2,n1,n4,n7,n6) :: m23_5
      complex(dp), dimension(n3,n1,n4,n7,n5,n2,n6) :: m24_1
      complex(dp), dimension(n3,n1,n4,n5,n2,n7,n6) :: m24_2
      complex(dp), dimension(n3,n7,n1,n4,n5,n2,n6) :: m24_3
      complex(dp), dimension(n3,n1,n4,n7,n5,n2,n6) :: m24_4
      complex(dp), dimension(n3,n1,n4,n5,n2,n7,n6) :: m24_5
      complex(dp), dimension(n5,n7,n3,n6,n1,n4,n2) :: m25_1
      complex(dp), dimension(n5,n3,n6,n7,n1,n4,n2) :: m25_2
      complex(dp), dimension(n5,n3,n6,n1,n4,n7,n2) :: m25_3
      complex(dp), dimension(n5,n3,n6,n1,n4,n7,n2) :: m25_4
      complex(dp), dimension(n5,n3,n6,n7,n1,n4,n2) :: m25_5
      complex(dp), dimension(n5,n7,n1,n4,n3,n6,n2) :: m30_1
      complex(dp), dimension(n5,n1,n4,n7,n3,n6,n2) :: m30_2
      complex(dp), dimension(n5,n1,n4,n3,n6,n7,n2) :: m30_3
      complex(dp), dimension(n5,n1,n4,n7,n3,n6,n2) :: m30_4
      complex(dp), dimension(n5,n1,n4,n3,n6,n7,n2) :: m30_5
      complex(dp), dimension(n1,n7,n3,n6,n5,n2,n4) :: m31_1
      complex(dp), dimension(n1,n3,n6,n7,n5,n2,n4) :: m31_2
      complex(dp), dimension(n1,n3,n6,n5,n2,n7,n4) :: m31_3
      complex(dp), dimension(n1,n3,n6,n5,n2,n7,n4) :: m31_4
      complex(dp), dimension(n1,n3,n6,n7,n5,n2,n4) :: m31_5
      complex(dp), dimension(n1,n7,n5,n2,n3,n6,n4) :: m33_1
      complex(dp), dimension(n1,n5,n2,n7,n3,n6,n4) :: m33_2
      complex(dp), dimension(n1,n5,n2,n3,n6,n7,n4) :: m33_3
      complex(dp), dimension(n1,n5,n2,n7,n3,n6,n4) :: m33_4
      complex(dp), dimension(n1,n5,n2,n3,n6,n7,n4) :: m33_5

      save ebeam

      if (p1(0).ne.ebeam) then
       call inipart(p1,p2,ps1,ps2)
       ebeam=p1(0)
      end if

c External particle momenta in the spinor representation:

      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)
      call spinr(p7,xp7,yp7)

c Spinors of the external fermions:

      call spinornew(p3,ps3,m3,1,1,n3,u3)
      call spinornew(p4,ps4,m4,-1,0,n4,u4)
      call spinornew(p5,ps5,m5,1,1,n5,u5)
      call spinornew(p6,ps6,m6,-1,0,n6,u6)

c Photon polarization vectors:

      call recpol(p7,ps7,zero,e7)

      do i7=1,n7
ckk>
c Test gauge invariance:

c      e7(:,i7)=p7
ckk<
        call spinr(e7(:,i7),xe7(:,:,i7),ye7(:,:,i7))
      end do

c Fermion denominators:

c      dbe=1.d0-sqrt(1.d0-(m1/p1(0))**2)
      dbe=(m1/p1(0))**2/2.d0

      d17=cmplx(-2.d0*p1(0)*p7(0)*(1.d0-ps7(1)+dbe*ps7(1)),zero,dp)
c     &                            +(gam1/2.d0)**2,m1*gam1,dp)
      d27=cmplx(-2.d0*p2(0)*p7(0)*(1.d0+ps7(1)-dbe*ps7(1)),zero,dp)
c     &                            +(gam2/2.d0)**2,m2*gam2,dp)
      d37=cmplx(srr(p3+p7,p3+p7),zero,dp)-cm3**2
      d47=cmplx(srr(p4+p7,p4+p7),zero,dp)-cm4**2
      d57=cmplx(srr(p5+p7,p5+p7),zero,dp)-cm5**2
      d67=cmplx(srr(p6+p7,p6+p7),zero,dp)-cm6**2

      call ffvgnew(u1,n1,-p1,-xp1,-yp1,cm1,ga1m,d17,
     &          u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,ga1m,ga1p,ma2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea12,ea12g,xa12g,ya12g,
     &          0,hff(1),mh2,h00g)

      call ffvgnew(u1,n1,-p1,-xp1,-yp1,cm1,ga1m,d17,
     &          u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,gz1m,gz1p,mz2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea12,ez12g,xz12g,yz12g,
     &          0,hff(1),mh2,h00g)

      if (ga3m == ocom) then
       ea36=ocom
       xa36=ocom
       ya36=ocom
       ea36g=ocom
       xa36g=ocom
       ya36g=ocom
      else
        call fvf(u3,n3,p3,m3,ga3m,ga3p,zero,ma2,u6,n6,p6,m6,
     &                                                 ea36,xa36,ya36)
       call ffvgnew(u3,n3,p3,xp3,yp3,cm3,ga3m,d37,
     &           u6,n6,p6,xp6,yp6,cm6,ga6m,d67,ga3m,ga3p,ma2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea36,ea36g,xa36g,ya36g,
     &           0,hff(3),mh2,h00g)
      end if

      call fvf(u3,n3,p3,m3,gz3m,gz3p,zero,mz2,u6,n6,p6,m6,
     &                                                 ez36,xz36,yz36)

      if (ihiggs == 1) then
       call fhf(u3,n3,p3,hff(3),zero,mh2,u6,n6,p6,h36)
      end if

      call ffvgnew(u3,n3,p3,xp3,yp3,cm3,ga3m,d37,
     &           u6,n6,p6,xp6,yp6,cm6,ga6m,d67,gz3m,gz3p,mz2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ez36,ez36g,xz36g,yz36g,
     &           ihiggs,hff(3),mh2,h36g)

      if (ga5m /= ocom) then
       call fvf(u1,n1,-p1,m1,ga1m,ga1p,zero,ma2,u4,n4,p4,m4,
     &                                                 ea14,xa14,ya14)
       call fvf(u1,n1,-p1,m1,gz1m,gz1p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez14,xz14,yz14)
      call ffvgnew(u1,n1,-p1,-xp1,-yp1,cm1,ga1m,d17,
     &          u4,n4,p4,xp4,yp4,cm4,ga4m,d47,ga4m,ga4p,ma2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea14,ea14g,xa14g,ya14g,
     &          0,hff(1),mh2,h00g)

      call ffvgnew(u1,n1,-p1,-xp1,-yp1,cm1,ga1m,d17,
     &          u4,n4,p4,xp4,yp4,cm4,ga4m,d47,gz4m,gz4p,mz2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ez14,ez14g,xz14g,yz14g,
     &          0,hff(1),mh2,h00g)

       t52=-2.d0*ps2(0)*ps5(0)*(1.d0+ps5(1)
     &    +(m2*(p2(0)-p5(0))/(p2(0)*p5(0)))**2/2.d0)
       call fvf(u5,n5,p5,m5,ga5m,ga5p,t52,ma2,u2,n2,-p2,m2,
     &                                                 ea52,xa52,ya52)
       call fvf(u5,n5,p5,m5,gz5m,gz5p,t52,mz2,u2,n2,-p2,m2,
     &                                                 ez52,xz52,yz52)
      call ffvgnew(u5,n5,p5,xp5,yp5,cm5,ga5m,d57,
     &          u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,ga5m,ga5p,ma2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea52,ea52g,xa52g,ya52g,
     &          0,hff(5),mh2,h00g)

      call ffvgnew(u5,n5,p5,xp5,yp5,cm5,ga5m,d57,
     &          u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,gz5m,gz5p,mz2,
     &          e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea52,ez52g,xz52g,yz52g,
     &          0,hff(5),mh2,h00g)

       call fvf(u5,n5,p5,m5,ga5m,ga5p,zero,ma2,u4,n4,p4,m4,
     &                                                 ea54,xa54,ya54)
       call ffvgnew(u5,n5,p5,xp5,yp5,cm5,ga5m,d57,
     &           u4,n4,p4,xp4,yp4,cm4,ga4m,d47,ga5m,ga5p,ma2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ea54,ea54g,xa54g,ya54g,
     &           0,hff(5),mh2,h00g)

      else
       ea54=ocom
       xa54=ocom
       ya54=ocom
       ea54g=ocom
       xa54g=ocom
       ya54g=ocom
      endif

      call fvf(u5,n5,p5,m5,gz5m,gz5p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez54,xz54,yz54)

      call ffvgnew(u5,n5,p5,xp5,yp5,cm5,ga5m,d57,
     &           u4,n4,p4,xp4,yp4,cm4,ga4m,d47,gz5m,gz5p,mz2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ez54,ez54g,xz54g,yz54g,
     &           0,hff(5),mh2,h00g)

      x136=ga1p*xa36+gz1p*xz36
      y136=ga1m*ya36+gz1m*yz36
      x136g=ga1p*xa36g+gz1p*xz36g
      y136g=ga1m*ya36g+gz1m*yz36g

      x236=ga2p*xa36+gz2p*xz36
      y236=ga2m*ya36+gz2m*yz36

      x536=ga5p*xa36+gz5p*xz36
      y536=ga5m*ya36+gz5m*yz36
      x536g=ga5p*xa36g+gz5p*xz36g
      y536g=ga5m*ya36g+gz5m*yz36g

      x154=ga1p*xa54+gz1p*xz54
      y154=ga1m*ya54+gz1m*yz54
      x154g=ga1p*xa54g+gz1p*xz54g
      y154g=ga1m*ya54g+gz1m*yz54g

      x354=ga3p*xa54+gz3p*xz54
      y354=ga3m*ya54+gz3m*yz54
      x354g=ga3p*xa54g+gz3p*xz54g
      y354g=ga3m*ya54g+gz3m*yz54g


      call fefefef(u1,-p1,xe7,ye7,ga1m,ga1p,
     &                xp7,yp7,cm1,d17,x154,y154,onec,onec,
     &                xp2-xp3-xp6,yp2-yp3-yp6,cm2,ocom,
     &                x136,y136,onec,onec,u2,-p2,
     &                m7_1,n1,n7,n5*n4,n3*n6,n2)

      call fefefef(u1,-p1,x154,y154,onec,onec,
     &                -xp1+xp5+xp4,-yp1+yp5+yp4,cm1,ocom,
     &                x136,y136,onec,onec,xp7,yp7,cm2,d27,
     &                xe7,ye7,ga2m,ga2p,u2,-p2,
     &                m7_2,n1,n5*n4,n3*n6,n7,n2)

      call fefefef(u1,-p1,x154,y154,onec,onec,
     &                -xp1+xp5+xp4,-yp1+yp5+yp4,cm1,ocom,
     &                xe7,ye7,ga1m,ga1p,
     &                xp2-xp3-xp6,yp2-yp3-yp6,cm2,ocom,
     &                x136,y136,onec,onec,u2,-p2,
     &                m7_3,n1,n5*n4,n7,n3*n6,n2)

      call fefef(u1,-p1,x154g,y154g,onec,onec,
     &           xp2-xp3-xp6,yp2-yp3-yp6,cm2,ocom,
     &           x136,y136,onec,onec,u2,-p2,
     &           m7_4,n1,n5*n4*n7,n3*n6,n2)

      call fefef(u1,-p1,x154,y154,onec,onec,
     &          -xp1+xp5+xp4,-yp1+yp5+yp4,cm1,ocom,
     &           x136g,y136g,onec,onec,u2,-p2,
     &           m7_5,n1,n5*n4,n3*n6*n7,n2)

      call fefefef(u1,-p1,xe7,ye7,ga1m,ga1p,
     &                xp7,yp7,cm1,d17,x136,y136,onec,onec,
     &                xp2-xp5-xp4,yp2-yp5-yp4,cm2,ocom,
     &                x154,y154,onec,onec,u2,-p2,
     &                m8_1,n1,n7,n3*n6,n5*n4,n2)

      call fefefef(u1,-p1,x136,y136,onec,onec,
     &                -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &                x154,y154,onec,onec,xp7,yp7,cm2,d27,
     &                xe7,ye7,ga2m,ga2p,u2,-p2,
     &                m8_2,n1,n3*n6,n5*n4,n7,n2)

      call fefefef(u1,-p1,x136,y136,onec,onec,
     &                -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &                xe7,ye7,ga1m,ga1p,
     &                xp2-xp5-xp4,yp2-yp5-yp4,cm2,ocom,
     &                x154,y154,onec,onec,u2,-p2,
     &                m8_3,n1,n3*n6,n7,n5*n4,n2)

      call fefef(u1,-p1,x136g,y136g,onec,onec,
     &           xp2-xp5-xp4,yp2-yp5-yp4,cm2,ocom,
     &           x154,y154,onec,onec,u2,-p2,
     &           m8_4,n1,n3*n6*n7,n5*n4,n2)

      call fefef(u1,-p1,x136,y136,onec,onec,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &           x154g,y154g,onec,onec,u2,-p2,
     &           m8_5,n1,n3*n6,n5*n4*n7,n2)

      call fefef(u3,p3,
     &           ga3p*xa12g+gz3p*xz12g,ga3m*ya12g+gz3m*yz12g,onec,onec,
     &          -xp6-xp5-xp4,-yp6-yp5-yp4,cm6,ocom,
     &           x354,y354,onec,onec,u6,p6,m9_4,n3,n1*n2*n7,n5*n4,n6)

      call fefef(u3,p3,x312,y312,onec,onec,
     &           xp3-xp1-xp2,yp3-yp1-yp2,cm3,ocom,
     &           x354g,y354g,onec,onec,u6,p6,m9_5,n3,n1*n2,n5*n4*n7,n6)

      if (ga3m /= ocom) then

      call fefefef(u3,p3,xe7,ye7,ga3m,ga3p,
     &             xp7,yp7,cm3,d37,x312,y312,onec,onec,
     &            -xp6-xp5-xp4,-yp6-yp5-yp4,cm6,ocom,
     &             x354,y354,onec,onec,u6,p6,
     &             m9_1,n3,n7,n1*n2,n5*n4,n6)

      call fefefef(u3,p3,x312,y312,onec,onec,
     &             xp3-xp1-xp2,yp3-yp1-yp2,cm3,ocom,
     &             x354,y354,onec,onec,xp7,yp7,cm6,d67,
     &             xe7,ye7,ga6m,ga6p,u6,p6,
     &             m9_2,n3,n1*n2,n5*n4,n7,n6)

      call fefefef(u3,p3,x312,y312,onec,onec,
     &             xp3-xp1-xp2,yp3-yp1-yp2,cm3,ocom,
     &             xe7,ye7,ga3m,ga3p,
     &            -xp6-xp5-xp4,-yp6-yp5-yp4,cm6,ocom,
     &             x354,y354,onec,onec,u6,p6,
     &             m9_3,n3,n1*n2,n7,n5*n4,n6)

      call fefefef(u3,p3,xe7,ye7,ga3m,ga3p,
     &             xp7,yp7,cm3,d37,x354,y354,onec,onec,
     &            -xp6+xp1+xp2,-yp6+yp1+yp2,cm6,ocom,
     &             x312,y312,onec,onec,u6,p6,m10_1,n3,n7,n5*n4,n1*n2,n6)

      call fefefef(u3,p3,x354,y354,onec,onec,
     &             xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &             x312,y312,onec,onec,xp7,yp7,cm6,d67,
     &             xe7,ye7,ga6m,ga6p,u6,p6,m10_2,n3,n5*n4,n1*n2,n7,n6)

      call fefefef(u3,p3,x354,y354,onec,onec,
     &             xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &             xe7,ye7,ga3m,ga3p,
     &            -xp6+xp1+xp2,-yp6+yp1+yp2,cm6,ocom,
     &             x612,y612,onec,onec,u6,p6,m10_3,n3,n5*n4,n7,n1*n2,n6)

      end if ! ga3m

      call fefef(u3,p3,x354g,y354g,onec,onec,
     &          -xp6+xp1+xp2,-yp6+yp1+yp2,cm6,ocom,
     &           x612,y612,onec,onec,u6,p6,m10_4,n3,n5*n4*n7,n1*n2,n6)

      call fefef(u3,p3,x354,y354,onec,onec,
     &           xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &           ga6p*xa12g+gz6p*xz12g,ga6m*ya12g+gz6m*yz12g,onec,onec,
     &           u6,p6,m10_5,n3,n5*n4,n1*n2*n7,n6)

      call fefefef(u5,p5,xe7,ye7,ga5m,ga5p,
     &             xp7,yp7,cm5,d57,x512,y512,onec,onec,
     &            -xp4-xp3-xp6,-yp4-yp3-yp6,cm4,ocom,
     &             x536,y536,onec,onec,u4,p4,
     &             m11_1,n5,n7,n1*n2,n3*n6,n4)

      call fefefef(u5,p5,x512,y512,onec,onec,
     &             xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,
     &             x536,y536,onec,onec,xp7,yp7,cm4,d47,
     &             xe7,ye7,ga4m,ga4p,u4,p4,
     &             m11_2,n5,n1*n2,n3*n6,n7,n4)

      call fefefef(u5,p5,x512,y512,onec,onec,
     &             xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,
     &             xe7,ye7,ga5m,ga5p,
     &            -xp4-xp3-xp6,-yp4-yp3-yp6,cm4,ocom,
     &             x536,y536,onec,onec,u4,p4,
     &             m11_3,n5,n1*n2,n7,n3*n6,n4)

      call fefef(u5,p5,
     &           ga5p*xa12g+gz5p*xz12g,ga5m*ya12g+gz5m*yz12g,onec,onec,
     &          -xp4-xp3-xp6,-yp4-yp3-yp6,cm4,ocom,
     &           x536,y536,onec,onec,u4,p4,m11_4,n5,n1*n2*n7,n3*n6,n4)

      call fefef(u5,p5,x512,y512,onec,onec,
     &           xp5-xp1-xp2,yp5-yp1-yp2,cm5,ocom,
     &           x536g,y536g,onec,onec,u4,p4,m11_5,n5,n1*n2,n3*n6*n7,n4)

      call fefefef(u5,p5,xe7,ye7,ga5m,ga5p,
     &             xp7,yp7,cm5,d57,x536,y536,onec,onec,
     &            -xp4+xp1+xp2,-yp4+yp1+yp2,cm4,ocom,
     &             x412,y412,onec,onec,u4,p4,m12_1,n5,n7,n3*n6,n1*n2,n4)

      call fefefef(u5,p5,x536,y536,onec,onec,
     &             xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &             x512,y512,onec,onec,xp7,yp7,cm4,d47,
     &             xe7,ye7,ga4m,ga4p,u4,p4,m12_2,n5,n3*n6,n1*n2,n7,n4)

      call fefefef(u5,p5,x536,y536,onec,onec,
     &             xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &             xe7,ye7,ga5m,ga5p,
     &            -xp4+xp1+xp2,-yp4+yp1+yp2,cm4,ocom,
     &             x412,y412,onec,onec,u4,p4,m12_3,n5,n3*n6,n7,n1*n2,n4)

      call fefef(u5,p5,x536g,y536g,onec,onec,
     &          -xp4+xp1+xp2,-yp4+yp1+yp2,cm4,ocom,
     &           x412,y412,onec,onec,u4,p4,m12_4,n5,n3*n6*n7,n1*n2,n4)

      call fefef(u5,p5,x536,y536,onec,onec,
     &           xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &           ga4p*xa12g+gz4p*xz12g,ga4m*ya12g+gz4m*yz12g,onec,onec,
     &           u4,p4,m12_5,n5,n3*n6,n1*n2*n7,n4)

      if (ga5m /= ocom) then

      x152=ga1p*xa52+gz1p*xz52
      y152=ga1m*ya52+gz1m*yz52
      x352=ga3p*xa52+gz3p*xz52
      y352=ga3m*ya52+gz3m*yz52

      x152g=ga1p*xa52g+gz1p*xz52g
      y152g=ga1m*ya52g+gz1m*yz52g
      x352g=ga3p*xa52g+gz3p*xz52g
      y352g=ga3m*ya52g+gz3m*yz52g

      x514=ga5p*xa14+gz5p*xz14
      y514=ga5m*ya14+gz5m*yz14
      x614=ga6p*xa14+gz6p*xz14
      y614=ga6m*ya14+gz6m*yz14

      call fefef(u3,p3,x352g,y352g,onec,onec,
     &           xp1-xp4-xp6,yp1-yp4-yp6,cm3,ocom,
     &           x614,y614,onec,onec,u6,p6,m23_1,n3,n5*n2*n7,n1*n4,n6)

      call fefef(u3,p3,x352,y352,onec,onec,
     &          -xp2+xp3+xp5,-yp2+yp3+yp5,cm3,ocom,
     &           ga6p*xa14g+gz6p*xz14g,ga6m*ya14g+gz6m*yz14g,onec,onec,
     &           u6,p6,m23_2,n3,n5*n2,n1*n4*n7,n6)

      call fefef(u3,p3,
     &           ga3p*xa14g+gz3p*xz14g,ga3m*ya14g+gz3m*yz14g,onec,onec,
     &           xp2-xp5-xp6,yp2-yp5-yp6,cm3,ocom,
     &           x352,y352,onec,onec,u6,p6,m24_1,n3,n1*n4*n7,n5*n2,n6)

      call fefef(u3,p3,x614,y614,onec,onec,
     &          -xp1+xp3+xp4,-yp1+yp3+yp4,cm3,ocom,
     &           x352g,y352g,onec,onec,u6,p6,m24_2,n3,n1*n4,n5*n2*n7,n6)

      call fefefef(u5,p5,xe7,ye7,ga5m,ga5p,xp7,yp7,cm5,d57,
     &             x536,y536,onec,onec,
     &             xp1+xp2-xp4,yp1+yp2-yp4,cm5,ocom,
     &             x514,y514,onec,onec,u2,-p2,
     &                                    m25_1,n5,n7,n3*n6,n1*n4,n2)

      call fefefef(u5,p5,x536,y536,onec,onec,
     &             xp3+xp5+xp6,yp3+yp5+yp6,cm5,ocom,
     &             xe7,ye7,ga5m,ga5p,
     &             xp1+xp2-xp4,yp1+yp2-yp4,cm5,ocom,
     &             x514,y514,onec,onec,u2,-p2,
     &                                     m25_2,n5,n3*n6,n7,n1*n4,n2)

      call fefefef(u5,p5,x536,y536,onec,onec,
     &             xp3+xp5+xp6,yp3+yp5+yp6,cm5,ocom,
     &             x514,y514,onec,onec,
     &             xp7,yp7,cm2,d27,xe7,ye7,ga2m,ga2p,
     &                               u2,-p2,m25_3,n5,n3*n6,n1*n4,n7,n2)

      call fefef(u5,p5,x536,y536,onec,onec,
     &           xp3+xp5+xp6,yp3+yp5+yp6,cm5,ocom,
     &           ga2p*xa14g+gz2p*xz14g,ga2m*ya14g+gz2m*yz14g,onec,onec,
     &                               u2,-p2,m25_4,n5,n3*n6,n1*n4*n7,n2)

      call fefefef(u5,p5,xe7,ye7,ga5m,ga5p,xp7,yp7,cm5,d57,
     &             x514,y514,onec,onec,
     &             xp2-xp3-xp6,yp2-yp3-yp6,cm5,ocom,
     &             x236,y236,onec,onec,u2,-p2,
     &             m30_1,n5,n7,n1*n4,n3*n6,n2)

      call fefefef(u5,p5,x514,y514,onec,onec,
     &            -xp1+xp4+xp5,-yp1+yp4+yp5,cm5,ocom,
     &             xe7,ye7,ga5m,ga5p,
     &             xp2-xp3-xp6,yp2-yp3-yp6,cm5,ocom,
     &             x236,y236,onec,onec,u2,-p2,
     &             m30_2,n5,n1*n4,n7,n3*n6,n2)

      call fefefef(u5,p5,x514,y514,onec,onec,
     &            -xp1+xp4+xp5,-yp1+yp4+yp5,cm5,ocom,
     &             x236,y236,onec,onec,
     &             xp7,yp7,cm2,d27,xe7,ye7,ga2m,ga2p,
     &             u2,-p2,m30_3,n5,n1*n4,n3*n6,n7,n2)

      call fefef(u5,p5,ga5p*xa14g+gz5p*xz14g,
     &                                ga5m*ya14g+gz5m*yz14g,onec,onec,
     &           xp2-xp3-xp6,yp2-yp3-yp6,cm5,ocom,
     &           x236,y236,onec,onec,u2,-p2,m30_4,n5,n1*n4*n7,n3*n6,n2)

      call fefefef(u1,-p1,xe7,ye7,ga1m,ga1p,xp7,yp7,cm1,d17,
     &             x136,y136,onec,onec,
     &             xp2-xp4-xp5,yp2-yp4-yp5,cm1,ocom,
     &             x152,y152,onec,onec,u4,p4,m31_1,n1,n7,n3*n6,n5*n2,n4)

      call fefefef(u1,-p1,x136,y136,onec,onec,
     &            -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &             xe7,ye7,ga1m,ga1p,
     &             xp2-xp4-xp5,yp2-yp4-yp5,cm1,ocom,
     &             x152,y152,onec,onec,u4,p4,m31_2,n1,n3*n6,n7,n5*n2,n4)

      call fefefef(u1,-p1,x136,y136,onec,onec,
     &            -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &             x152,y152,onec,onec,
     &             xp7,yp7,cm4,d47,xe7,ye7,ga4m,ga4p,
     &             u4,p4,m31_3,n1,n3*n6,n5*n2,n7,n4)

      call fefef(u1,-p1,x136,y136,onec,onec,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &           x152g,y152g,onec,onec,u4,p4,m31_4,n1,n3*n6,n5*n2*n7,n4)

      call fefefef(u1,-p1,xe7,ye7,ga1m,ga1p,xp7,yp7,cm1,d17,
     &             x152,y152,onec,onec,
     &            -xp3-xp4-xp6,-yp3-yp4-yp6,cm1,ocom,
     &             x536,y536,onec,onec,u4,p4,m33_1,n1,n7,n5*n2,n3*n6,n4)

      call fefefef(u1,-p1,x152,y152,onec,onec,
     &            -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,
     &             xe7,ye7,ga1m,ga1p,
     &            -xp3-xp4-xp6,-yp3-yp4-yp6,cm1,ocom,
     &             x536,y536,onec,onec,u4,p4,m33_2,n1,n5*n2,n7,n3*n6,n4)

      call fefefef(u1,-p1,x152,y152,onec,onec,
     &            -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,
     &             x536,y536,onec,onec,
     &             xp7,yp7,cm4,d47,xe7,ye7,ga4m,ga4p,
     &             u4,p4,m33_3,n1,n5*n2,n3*n6,n7,n4)

      call fefef(u1,-p1,x152g,y152g,onec,onec,
     &          -xp3-xp4-xp6,-yp3-yp4-yp6,cm1,ocom,
     &           x536,y536,onec,onec,u4,p4,m33_4,n1,n5*n2*n7,n3*n6,n4)


      if (ga3m /= ocom) then

      call fefefef(u3,p3,xe7,ye7,ga3m,ga3p,
     &             xp7,yp7,cm3,d37,x352,y352,onec,onec,
     &             xp1-xp4-xp6,yp1-yp4-yp6,cm6,ocom,
     &             x614,y614,onec,onec,u6,p6,
     &             m23_3,n3,n7,n5*n2,n1*n4,n6)

      call fefefef(u3,p3,x352,y352,onec,onec,
     &            -xp2+xp3+xp5,-yp2+yp3+yp5,cm3,ocom,
     &             xe7,ye7,ga3m,ga3p,
     &             xp1-xp4-xp6,yp1-yp4-yp6,cm6,ocom,
     &             x614,y614,onec,onec,u6,p6,
     &             m23_4,n3,n5*n2,n7,n1*n4,n6)

      call fefefef(u3,p3,x352,y352,onec,onec,
     &            -xp2+xp3+xp5,-yp2+yp3+yp5,cm3,ocom,
     &             x614,y614,onec,onec,xp7,yp7,cm6,d67,
     &             xe7,ye7,ga6m,ga6p,u6,p6,
     &             m23_5,n3,n5*n2,n1*n4,n7,n6)

      call fefefef(u3,p3,xe7,ye7,ga3m,ga3p,
     &             xp7,yp7,cm3,d37,x614,y614,onec,onec,
     &             xp2-xp5-xp6,yp2-yp5-yp6,cm3,ocom,
     &             x352,y352,onec,onec,u6,p6,
     &             m24_3,n3,n7,n1*n4,n5*n2,n6)

      call fefefef(u3,p3,x614,y614,onec,onec,
     &            -xp1+xp3+xp4,-yp1+yp3+yp4,cm3,ocom,
     &             xe7,ye7,ga3m,ga3p,
     &             xp2-xp5-xp6,yp2-yp5-yp6,cm6,ocom,
     &             x352,y352,onec,onec,u6,p6,
     &             m24_4,n3,n1*n4,n7,n5*n2,n6)

      call fefefef(u3,p3,x614,y614,onec,onec,
     &            -xp1+xp3+xp4,-yp1+yp3+yp4,cm3,ocom,
     &             x352,y352,onec,onec,xp7,yp7,cm6,d67,
     &             xe7,ye7,ga6m,ga6p,u6,p6,
     &             m24_5,n3,n1*n4,n5*n2,n7,n6)

      call fefef(u5,p5,x536g,y536g,onec,onec,
     &           xp1+xp2-xp4,yp1+yp2-yp4,cm5,ocom,
     &           x514,y514,onec,onec,u2,-p2,m25_5,n5,n3*n6*n7,n1*n4,n2)

      call fefef(u5,p5,x514,y514,onec,onec,
     &          -xp1+xp4+xp5,-yp1+yp4+yp5,cm5,ocom,
     &           x536g,y536g,onec,onec,u2,-p2,
     &                                    m30_5,n5,n1*n4,n3*n6*n7,n2)

      call fefef(u1,-p1,x136g,y136g,onec,onec,
     &           xp2-xp4-xp5,yp2-yp4-yp5,cm1,ocom,
     &           x152,y152,onec,onec,u4,p4,m31_5,n1,n3*n6*n7,n5*n2,n4)

      call fefef(u1,-p1,x152,y152,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,
     &           x536g,y536g,onec,onec,u4,p4,m33_5,n1,n5*n2,n3*n6*n7,n4)

      end if ! ga3m


      end if ! ga5m

c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,n1
      do i2=1,n2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6
      do i7=1,n7

      mat=
     &   -m7_1(i1,i7,i5,i4,i3,i6,i2)
     &   -m7_2(i1,i5,i4,i3,i6,i7,i2)
     &   -m7_3(i1,i5,i4,i7,i3,i6,i2)
     &   -m7_4(i1,i5,i4,i7,i3,i6,i2)
     &   -m7_5(i1,i5,i4,i3,i6,i7,i2)
     &   -m8_1(i1,i7,i3,i6,i5,i4,i2)
     &   -m8_2(i1,i3,i6,i5,i4,i7,i2)
     &   -m8_3(i1,i3,i6,i7,i5,i4,i2)
     &   -m8_4(i1,i3,i6,i7,i5,i4,i2)
     &   -m8_5(i1,i3,i6,i5,i4,i7,i2)
     &   -m9_4(i3,i1,i2,i7,i5,i4,i6)
     &   -m9_5(i3,i1,i2,i5,i4,i7,i6)
     &   -m10_4(i3,i5,i4,i7,i1,i2,i6)
     &   -m10_5(i3,i5,i4,i1,i2,i7,i6)

      if (ga3m /= ocom) then
       mat=mat
     &    -m9_1(i3,i7,i1,i2,i5,i4,i6)
     &    -m9_2(i3,i1,i2,i5,i4,i7,i6)
     &    -m9_3(i3,i1,i2,i7,i5,i4,i6)
     &    -m10_1(i3,i7,i5,i4,i1,i2,i6)
     &    -m10_2(i3,i5,i4,i1,i2,i7,i6)
     &    -m10_3(i3,i5,i4,i7,i1,i2,i6)
      end if ! ga3m

      mat=mat
     &   -m11_1(i5,i7,i1,i2,i3,i6,i4)
     &   -m11_2(i5,i1,i2,i3,i6,i7,i4)
     &   -m11_3(i5,i1,i2,i7,i3,i6,i4)
     &   -m11_4(i5,i1,i2,i7,i3,i6,i4)
     &   -m11_5(i5,i1,i2,i3,i6,i7,i4)
     &   -m12_1(i5,i7,i3,i6,i1,i2,i4)
     &   -m12_2(i5,i3,i6,i1,i2,i7,i4)
     &   -m12_3(i5,i3,i6,i7,i1,i2,i4)
     &   -m12_4(i5,i3,i6,i7,i1,i2,i4)
     &   -m12_5(i5,i3,i6,i1,i2,i7,i4)
      if (ga5m /= ocom) then
       mat=mat
     &    +m23_1(i3,i5,i2,i7,i1,i4,i6)
     &    +m23_2(i3,i5,i2,i1,i4,i7,i6)
     &    +m24_1(i3,i1,i4,i7,i5,i2,i6)
     &    +m24_2(i3,i1,i4,i5,i2,i7,i6)
     &    +m25_1(i5,i7,i3,i6,i1,i4,i2)
     &    +m25_2(i5,i3,i6,i7,i1,i4,i2)
     &    +m25_3(i5,i3,i6,i1,i4,i7,i2)
     &    +m25_4(i5,i3,i6,i1,i4,i7,i2)
     &    +m30_1(i5,i7,i1,i4,i3,i6,i2)
     &    +m30_2(i5,i1,i4,i7,i3,i6,i2)
     &    +m30_3(i5,i1,i4,i3,i6,i7,i2)
     &    +m30_4(i5,i1,i4,i7,i3,i6,i2)
       mat=mat
     &    +m31_1(i1,i7,i3,i6,i5,i2,i4)
     &    +m31_2(i1,i3,i6,i7,i5,i2,i4)
     &    +m31_3(i1,i3,i6,i5,i2,i7,i4)
     &    +m31_4(i1,i3,i6,i5,i2,i7,i4)
     &    +m33_1(i1,i7,i5,i2,i3,i6,i4)
     &    +m33_2(i1,i5,i2,i7,i3,i6,i4)
     &    +m33_3(i1,i5,i2,i3,i6,i7,i4)
     &    +m33_4(i1,i5,i2,i7,i3,i6,i4)
       if (ga3m /= ocom) then
       mat=mat
     &    +m23_3(i3,i7,i5,i2,i1,i4,i6)
     &    +m23_4(i3,i5,i2,i7,i1,i4,i6)
     &    +m23_5(i3,i5,i2,i1,i4,i7,i6)
     &    +m24_3(i3,i7,i1,i4,i5,i2,i6)
     &    +m24_4(i3,i1,i4,i7,i5,i2,i6)
     &    +m24_5(i3,i1,i4,i5,i2,i7,i6)
     &    +m25_5(i5,i3,i6,i7,i1,i4,i2)
     &    +m30_5(i5,i1,i4,i3,i6,i7,i2)
     &    +m31_5(i1,i3,i6,i7,i5,i2,i4)
     &    +m33_5(i1,i5,i2,i3,i6,i7,i4)
      end if ! ga3m
      end if ! ga5m

      if (ihiggs == 1) then
       mat=mat
     &    +hzz*(h36g(i3,i6,i7)*(scc(ez14(:,i1,i4),ez52(:,i5,i2))
     &                         -scc(ez12(:,i1,i2),ez54(:,i5,i4)))
     &         +h36(i3,i6)*(scc(ez14g(:,i1,i4,i7),ez52(:,i5,i2))
     &                     +scc(ez14(:,i1,i4),ez52g(:,i5,i2,i7))
     &                     -scc(ez12g(:,i1,i2,i7),ez54(:,i5,i4))
     &                     -scc(ez12(:,i1,i2),ez54g(:,i5,i4,i7))))
      end if

      mat2=mat2+(abs(mat))**2

      end do
      end do
      end do
      end do
      end do
      end do
      end do

      mat2=comf2*mat2

      end 

