      real(kind(1.d0)) function eeffffgs(ibr,p1,p2,p3,p4,p5,p6,ecut)
c----------------------------------------------------------------------c
c Returns the soft Bremsstrahlung correction for 
c
c      e+(p1) + e-(p2) --> f1(p3) + f2~(p4) + f3(p5) + f4~(p6)
c
c Input: ibr = 1 (initial state bremsstrahlung)/ 2 (final state 
c              bremsstrahlung)/ 0 (complete bremsstrahlung),
c        p(0:3,i), i=1,...,6 - external particle momenta,
c        ecut - the soft photon cut
c
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 16 March, 1998;  modified on 8 February, 2000; 2 June, 2000;
c                              9 November, 2001; 20 June, 2003
c----------------------------------------------------------------------c

      use inprms 
      use mathprms 
      use parproc  ! Defines initial parameters for prcess iproc

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: ibr,init=1,i,j,i1,i2,j2

      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp) :: p(0:3,6),ecut

      real(dp) :: mf(6),mf2(6),c(6,6),coeff,pipj,srr,iomega,dbi,bi

      save c,mf,mf2,coeff,i1,i2,j2


      if(init == 1) then

      init=0

c Coefficients:

      c(1,1)=abs(ga1m)**2
      c(1,2)=-2.d0*ga1m*conjg(ga2m)
      c(1,3)=2.d0*ga1m*conjg(ga3m)
      c(1,4)=-2.d0*ga1m*conjg(ga4m)
      c(1,5)=2.d0*ga1m*conjg(ga5m)
      c(1,6)=-2.d0*ga1m*conjg(ga6m)
      c(2,2)=abs(ga2m)**2
      c(2,3)=-2.d0*ga2m*conjg(ga3m)
      c(2,4)=2.d0*ga2m*conjg(ga4m)
      c(2,5)=-2.d0*ga2m*conjg(ga5m)
      c(2,6)=2.d0*ga2m*conjg(ga6m)
      c(3,3)=abs(ga3m)**2
      c(3,4)=-2.d0*ga3m*conjg(ga4m)
      c(3,5)=2.d0*ga3m*conjg(ga5m)
      c(3,6)=-2.d0*ga3m*conjg(ga6m)
      c(4,4)=abs(ga4m)**2
      c(4,5)=-2.d0*ga4m*conjg(ga5m)
      c(4,6)=2.d0*ga4m*conjg(ga6m)
      c(5,5)=abs(ga5m)**2
      c(5,6)=-2.d0*ga5m*conjg(ga6m)
      c(6,6)=abs(ga6m)**2

c The common factor:

      coeff=(e0/abs(cew))**2/(4.d0*pi**2)

c Squared masses:

      mf=(/m1,m2,m3,m4,m5,m6/)

      mf2=mf**2

      if (ibr == 1) then
        i1=1
        i2=2
        j2=2
      else if (ibr == 2) then
        i1=3
        i2=6
        j2=6
      else
        i1=1
        i2=6
        j2=6
      end if

      end if ! init

      p(:,1)=p1
      p(:,2)=p2
      p(:,3)=p3
      p(:,4)=p4
      p(:,5)=p5
      p(:,6)=p6

      eeffffgs=0.d0

      do i=i1,i2

        dbi=(mf(i)/p(0,i))**2
        bi=sqrt(1.d0-dbi)

        do j=i,j2

          if (c(i,j) /= zero) then

           if (i == j) then
             eeffffgs=eeffffgs+c(i,i)*(log(2.d0*ecut/mgamma)
     &                       -log((1.d0+bi)**2/dbi)/(2.d0*bi))
           else
             pipj=srr(p(:,i),p(:,j))
             if (pipj <= mf(i)*mf(j)) then
               if (mf(i) /= mf(j)) then
                write(*,*)
     &    'Warning: The soft bremsstrahlung formula might be wrong'
               end if
               eeffffgs=eeffffgs+c(i,j)*(log(2.d0*ecut/mgamma)
     &                       -log((1.d0+bi)**2/dbi)/(2.d0*bi))
             else
              eeffffgs=eeffffgs+c(i,j)*pipj*iomega(ecut,mgamma,
     &            p(0,i),mf(i),mf2(i),p(0,j),mf(j),mf2(j),pipj)
             end if
           end if
          end if
        end do
      end do

      eeffffgs=coeff*eeffffgs

      end
