      subroutine eeffffid1(iproc,p1,p2,p3,p4,p5,p6,
     &                           ps1,ps2,ps3,ps4,ps5,ps6,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c      e+(p1) + e-(p2) --> l(p3) + l~(p4) + l(p5) + l~(p6)
c
c for the neutral current processes containing identical leptons
c
c Input: pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 27 September, 2000;  modified on 1 November, 2001
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use genpv12

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer iproc,i1,i2,i3,i4,i5,i6,i
      real(dp) :: mat2
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,xp5,xp6,
     &                              yp1,yp2,yp3,yp4,yp5,yp6

      complex(dp), dimension(4,2) :: u1
      complex(dp), dimension(4,2) :: u2
      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp) :: mat
      complex(dp), dimension(2,2,n3,n4,n5,n6) :: ml3456
      complex(dp), dimension(2,2,n5,n4,n3,n6) :: ml5436
      complex(dp), dimension(2,2,n3,n4,n5,n6) :: me3456
      complex(dp), dimension(2,2,n5,n4,n3,n6) :: me5436
      complex(dp), dimension(2,2,n3,n6,n5,n4) :: me3654
      complex(dp), dimension(2,2,n5,n6,n3,n6) :: me5634

c External particle momenta in the spinor representation:

      call spinr(p1,xp1,yp1)
      call spinr(p2,xp2,yp2)
      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)

c Spinors of the external fermions:

      call spinornew(p1,ps1,m1,-1,1,2,u1)
      call spinornew(p2,ps2,m2,1,0,2,u2)
      call spinornew(p3,ps3,m3,1,1,n3,u3)
      call spinornew(p4,ps4,m4,-1,0,n4,u4)
      call spinornew(p5,ps5,m5,1,1,n5,u5)
      call spinornew(p6,ps6,m6,-1,0,n6,u6)

c Building blocks of the diagrams:

      call fvf(u1,2,-p1,m1,ga1m,ga1p,zero,ma2,u2,2,-p2,m2,
     &                                                 ea12,xa12,ya12)
      call fvf(u1,2,-p1,m1,gz1m,gz1p,zero,mz2,u2,2,-p2,m2,
     &                                                 ez12,xz12,yz12)

      call matllll(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &             p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &             p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,ml3456)

      call matllll(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &             p5,ps5,xp5,yp5,u5,p4,ps4,xp4,yp4,u4,
     &             p3,ps3,xp3,yp3,u3,p6,ps6,xp6,yp6,u6,ml5436)

      if (iproc == 405) then

      call mateeee(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &             p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &             p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,me3456)

      call mateeee(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &             p5,ps5,xp5,yp5,u5,p4,ps4,xp4,yp4,u4,
     &             p3,ps3,xp3,yp3,u3,p6,ps6,xp6,yp6,u6,me5436)

      call mateeee(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &             p3,ps3,xp3,yp3,u3,p6,ps6,xp6,yp6,u6,
     &             p5,ps5,xp5,yp5,u5,p4,ps4,xp4,yp4,u4,me3654)

      call mateeee(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &             p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,
     &             p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,me5634)

      end if

c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,n1
      do i2=1,n2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

c A sum of the diagrams:

      mat=ml3456(i1,i2,i3,i4,i5,i6)
     &   -ml5436(i1,i2,i5,i4,i3,i6)

      if (iproc == 405) then
       mat=mat
     &   +me3456(i1,i2,i3,i4,i5,i6)
     &   -me5436(i1,i2,i5,i4,i3,i6)
     &   -me3654(i1,i2,i3,i6,i5,i4)
     &   +me5634(i1,i2,i5,i6,i3,i4)
      end if

      mat2=mat2+abs(mat)**2

      end do
      end do
      end do
      end do
      end do
      end do

      mat2=comf1*mat2


      end 


