      subroutine eeffffnc3(iproc,p1,p2,p3,p4,p5,p6,
     &                             ps1,ps2,ps3,ps4,ps5,ps6,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c      e+(p1) + e-(p2) --> ve(p3) + f~(p4) + f(p5) + ve~(p6)
c
c for the neutral current processes
c
c Input: iproc - specifies the proces,
c        pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 19 September, 2000  modified 16 February, 2001; 24 April, 2001;
c                              1 November, 2001; 20 June, 2003
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use inistate

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: iproc
      real(dp) :: mat2,ebeam=0.d0
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      integer :: i1,i2,i3,i4,i5,i6,i

      complex(dp) :: mat,scc

      complex(dp), dimension(2,2) :: xp3,xp4,xp5,xp6,
     &                              yp3,yp4,yp5,yp6

      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,
     &                                    ya36,yz36
      complex(dp), dimension(0:3,n5,n4) :: ea54,ez54
      complex(dp), dimension(2,2,n5,n4) :: xa54,xz54,ya54,yz54
      complex(dp), dimension(0:3,n1,n6) :: ew16
      complex(dp), dimension(2,2,n1,n6) :: xw16,yw16
      complex(dp), dimension(0:3,n3,n2) :: ew32
      complex(dp), dimension(2,2,n3,n2) :: xw32,yw32

      complex(dp), dimension(n5,n4) :: h54

      complex(dp), dimension(n1,n5,n4,n3,n6,n2) :: amp7
      complex(dp), dimension(n1,n3,n6,n5,n4,n2) :: amp8
      complex(dp), dimension(n3,n1,n2,n5,n4,n6) :: amp9
      complex(dp), dimension(n3,n5,n4,n1,n2,n6) :: amp10
      complex(dp), dimension(n5,n1,n2,n3,n6,n4) :: amp11
      complex(dp), dimension(n5,n3,n6,n1,n2,n4) :: amp12
      complex(dp), dimension(n1,n6,n3,n2,n5,n4) :: amp26
      complex(dp), dimension(n1,n5,n4,n3,n2,n6) :: amp27
      complex(dp), dimension(n1,n3,n2,n5,n4,n6) :: amp28
      complex(dp), dimension(n5,n1,n6,n3,n2,n4) :: amp29u,ampu
      complex(dp), dimension(n5,n3,n2,n1,n6,n4) :: amp29d,ampd
      complex(dp), dimension(n3,n1,n6,n5,n4,n2) :: amp32
      complex(dp), dimension(n3,n5,n4,n1,n6,n2) :: amp34

      save ebeam

      if (p1(0).ne.ebeam) then
       call inipart(p1,p2,ps1,ps2)
       ebeam=p1(0)
      end if

c External particle momenta in the spinor representation:

      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)

c Spinors of the external fermions:

      call spinornew(p3,ps3,m3,1,1,n3,u3)
      call spinornew(p4,ps4,m4,-1,0,n4,u4)
      call spinornew(p5,ps5,m5,1,1,n5,u5)
      call spinornew(p6,ps6,m6,-1,0,n6,u6)

c Building blocks of the diagrams:

c Vector boson couplings sandwiched between spinors:

       call fvf(u1,n1,-p1,m1,wff,ocom,zero,mw2,u6,n6,p6,m6,
     &                                                 ew16,xw16,yw16)
       call fvf(u3,n3,p3,m3,wff,ocom,zero,mw2,u2,n2,-p2,m2,
     &                                                 ew32,xw32,yw32)

        xa36=ocom
        ya36=ocom
       call fvf(u3,n3,p3,m3,gz3m,gz3p,zero,mz2,u6,n6,p6,m6,
     &                                                 ez36,xz36,yz36)

      if (ga5m == ocom) then
       ea54=ocom
       xa54=ocom
       ya54=ocom
       h54=ocom
      else
       call fvf(u5,n5,p5,m5,ga5m,ga5p,zero,ma2,u4,n4,p4,m4,
     &                                                 ea54,xa54,ya54)
       if (ihiggs == 1) then
        call fhf(u5,n5,p5,hff(5),zero,mh2,u4,n4,p4,h54)
       end if
      end if

      call fvf(u5,n5,p5,m5,gz5m,gz5p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez54,xz54,yz54)

c Amplitudes:

      call fefef(u1,-p1,ga1p*xa54+gz1p*xz54,
     &                                ga1m*ya54+gz1m*yz54,onec,onec,
     &          -xp1+xp5+xp4,-yp1+yp5+yp4,cm1,ocom,
     &           ga2p*xa36+gz2p*xz36,ga2m*ya36+gz2m*yz36,onec,onec,
     &                                u2,-p2,amp7,n1,n5*n4,n3*n6,n2)

      call fefef(u1,-p1,ga1p*xa36+gz1p*xz36,
     &                                ga1m*ya36+gz1m*yz36,onec,onec,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &           ga2p*xa54+gz2p*xz54,ga2m*ya54+gz2m*yz54,onec,onec,
     &                                u2,-p2,amp8,n1,n3*n6,n5*n4,n2)

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,onec,onec,
     &                                u6,p6,amp9,n3,n1*n2,n5*n4,n6)

      call fefef(u3,p3,ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,
     &           onec,onec,xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,p6,amp10,n3,n5*n4,n1*n2,n6)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,
     &                                ga5m*ya12+gz5m*yz12,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm5,ocom,
     &           ga4p*xa36+gz4p*xz36,ga4m*ya36+gz4m*yz36,onec,onec,
     &                                u4,p4,amp11,n5,n1*n2,n3*n6,n4)

      call fefef(u5,p5,ga5p*xa36+gz5p*xz36,
     &                                ga5m*ya36+gz5m*yz36,onec,onec,
     &           xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &           ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                                u4,p4,amp12,n5,n3*n6,n1*n2,n4)

      call eee(-p1+p6,ew16,p2-p3,ew32,-p4-p5,
     &                       gww*ea54+zww*ez54,amp26,n1*n6,n3*n2,n5*n4)


      if (iproc < 214) then

       amp29u=ocom

       do i=1,3
        if (ckm(i) > zero) then
         call fefef(u5,p5,xw16,yw16,wff,ocom,
     &           -xp1+xp5+xp6,-yp1+yp5+yp6,cmq(i),ocom,
     &            xw32,yw32,wff,ocom,u4,p4,ampu,n5,n1*n6,n3*n2,n4)
         amp29u=amp29u+ckm(i)*ampu
        end if
       end do

      else if (iproc < 217) then

       amp29d=ocom

       do i=1,3
        if (ckm(i) > zero) then
         call fefef(u5,p5,xw32,yw32,wff,ocom,
     &              xp1-xp4-xp6,yp1-yp4-yp6,cmq(i),ocom,
     &              xw16,yw16,wff,ocom,u4,p4,ampd,n5,n3*n2,n1*n6,n4)
         amp29d=amp29d+ckm(i)*ampd
        end if
       end do

      else if (iproc < 219) then

       call fefef(u5,p5,xw32,yw32,wff,ocom,
     &            xp1-xp4-xp6,yp1-yp4-yp6,ocom,ocom,ocom,
     &            xw16,yw16,wff,ocom,u4,p4,amp29d,n5,n3*n2,n1*n6,n4)

      else 

       call fefef(u5,p5,xw16,yw16,wff,ocom,
     &           -xp1+xp5+xp6,-yp1+yp5+yp6,cml,ocom,
     &            xw32,yw32,wff,ocom,u4,p4,amp29u,n5,n1*n6,n3*n2,n4)

      end if ! iproc

      call fefef(u1,-p1,ga1p*xa54+gz1p*xz54,
     &                                ga1m*ya54+gz1m*yz54,onec,onec,
     &          -xp1+xp4+xp5,-yp1+yp4+yp5,cm1,ocom,
     &           xw32,yw32,wff,ocom,u6,p6,amp27,n1,n5*n4,n3*n2,n6)

      call fefef(u1,-p1,xw32,yw32,wff,ocom,
     &          -xp4-xp5-xp6,-yp4-yp5-yp6,ocom,ocom,ocom,
     &           xz54,yz54,gz6m,gz6p,u6,p6,amp28,n1,n3*n2,n5*n4,n6)

      call fefef(u3,p3,xw16,yw16,wff,ocom,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm2,ocom,
     &           ga2p*xa54+gz2p*xz54,ga2m*ya54+gz2m*yz54,onec,onec,
     &                               u2,-p2,amp32,n3,n1*n6,n5*n4,n2)

      call fefef(u3,p3,xz54,yz54,gz3m,gz3p,
     &           xp3+xp4+xp5,yp3+yp4+yp5,ocom,ocom,ocom,
     &           xw16,yw16,wff,ocom,u2,-p2,amp34,n3,n5*n4,n1*n6,n2)


c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,n1
      do i2=1,n2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

c A sum of the diagrams:

      mat=
     &   +amp26(i1,i6,i3,i2,i5,i4)
     &     -amp7(i1,i5,i4,i3,i6,i2)
     &     -amp8(i1,i3,i6,i5,i4,i2)
     &     -amp9(i3,i1,i2,i5,i4,i6)
     &     -amp10(i3,i5,i4,i1,i2,i6)
     &     -amp11(i5,i1,i2,i3,i6,i4)
     &     -amp12(i5,i3,i6,i1,i2,i4)
     &    +amp27(i1,i5,i4,i3,i2,i6)
     &    +amp28(i1,i3,i2,i5,i4,i6)
     &    +amp32(i3,i1,i6,i5,i4,i2)
     &    +amp34(i3,i5,i4,i1,i6,i2)
      if (iproc < 214 .or. iproc >= 219) then
       mat=mat
     &    +amp29u(i5,i1,i6,i3,i2,i4)
      else
       mat=mat
     &    +amp29d(i5,i3,i2,i1,i6,i4)
      end if ! iproc

      if (ihiggs == 1) then
       mat=mat
     &    +h54(i5,i4)*(-hzz*scc(ez12(:,i1,i2),ez36(:,i3,i6))
     &                 +hww*scc(ew16(:,i1,i6),ew32(:,i3,i2)))
      end if



      mat2=mat2+abs(mat)**2

      end do
      end do
      end do
      end do
      end do
      end do

      mat2=comf1*mat2

      end 



