      subroutine fefef(bu,pb,e1p,e1m,g1m,g1p,p1p,p1m,cm1,den1,
     &                 e2p,e2m,g2m,g2p,u,p,mat,nb,n1,n2,n)
c----------------------------------------------------------------------c
c Returns matrix elements: 
c
c  mat(nb,n1,n2,n)=bu*e1*(g1m*P- + g1p*P+)*(p1+m1)/den1
c                                             *e2*(g2m*P- + g2p*P+)*u
c
c Input: bu,u - spinors,
c        -pb/+pb,-p/+p - momenta of incoming/outgoing
c        e1,e2 - generalized polarization vectors in the spinor 
c                representation,
c        g1p,g2p - vector boson couplings to the fermion line of the 
c                  positive chirality, P+=(1+gamma_5)/2,
c        g1m,g2m - vector boson couplings to the fermion line of the 
c                  negative chirality, P-=(1-gamma_5)/2,
c        p1 - intermidiate fermion momentum in the spinor representation,
c        cm1 = m1-1/2*gam1 - intermidiate fermion mass and width, relevant 
c                            only if the corresponding denominator den1=0,
c        nb,n1,n2,n - numbers of polarization states.
c
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 19 February, 1999; modified on 29 September, 1999; 13 February, 2001;
c                                1 November, 2001; 20 June, 2003
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: nb,n1,n2,n

      complex(dp) :: g1p,g2p,g1m,g2m
      real(dp), dimension(0:3) :: pb,p
      complex(dp) :: den1
      complex(dp), dimension(2,2) :: p1p,p1m
      complex(dp), dimension(4,nb) :: bu
      complex(dp), dimension(4,n) :: u
      complex(dp), dimension(2,2,n1) :: e1p,e1m
      complex(dp), dimension(2,2,n2) :: e2p,e2m
      complex(dp), dimension(nb,n1,n2,n) :: mat

      integer :: ib,i1,i2,i

      complex(dp), dimension(4,nb,n1) :: bu1,bu11
      complex(dp), dimension(4,n,n2) :: u2
      complex(dp) :: cm1,cd1
      complex(dp), parameter :: ocom=(0.d0,0.d0),icom=(0.d0,1.d0)
ckk>
c      real(dp) :: m1,gam1,mgam1
c      m1=cm1
c      gam1=2.d0*icom*cm1
ckk<
      if (den1 == ocom) then
       cd1=p1p(1,1)*p1p(2,2)-p1p(1,2)*p1p(2,1)-cm1**2
      else
       cd1=den1
      end if

      do ib=1,nb
       do i1=1,n1
        bu1(:2,ib,i1)=g1m*matmul(bu(3:,ib),e1m(:,:,i1))
        bu1(3:,ib,i1)=g1p*matmul(bu(:2,ib),e1p(:,:,i1))
ckk>
        bu11(:2,ib,i1)=cm1*bu1(:2,ib,i1)+matmul(bu1(3:,ib,i1),p1m)
        bu11(3:,ib,i1)=matmul(bu1(:2,ib,i1),p1p)+cm1*bu1(3:,ib,i1)
c        bu11(:2,ib,i1)=m1*bu1(:2,ib,i1)+matmul(bu1(3:,ib,i1),p1m)
c        bu11(3:,ib,i1)=matmul(bu1(:2,ib,i1),p1p)+m1*bu1(3:,ib,i1)
ckk<
       end do
      end do

      do i=1,n
       do i2=1,n2
        u2(:2,i,i2)=g2p*matmul(e2p(:,:,i2),u(3:,i))
        u2(3:,i,i2)=g2m*matmul(e2m(:,:,i2),u(:2,i))
       end do
      end do

      do ib=1,nb
       do i1=1,n1
        do i2=1,n2
         do i=1,n
ckk>
c          mat(ib,i1,i2,i)=dot_product(conjg(bu11(:,ib,i1)),
c     &                                             u2(:,i,i2))/cd1
          mat(ib,i1,i2,i)=sum(bu11(:,ib,i1)*u2(:,i,i2))/cd1
ckk<
         end do
        end do
       end do
      end do

      end




