      subroutine fefefefh(bu,pb,e1p,e1m,g1m,g1p,p1p,p1m,cm1,den1,
     &                   e2p,e2m,g2m,g2p,p2p,p2m,cm2,den2,
     &                   e3p,e3m,g3m,g3p,u,p,h,ih,nh,mat,nb,n1,n2,n3,n)
c----------------------------------------------------------------------c
c Returns matrix elements: 
c
c  mat(nb,n1,n2,n3,n)=bu*e1*(g1m*P- + g1p*P+)*(p1+m1)/d1
c              *e2*(g2m*P- + g2p*P+)*(p2+m2)/d2*e3*(g3m*P- + g3p*P+)*u
c                               
c
c Input: bu,u - spinors,
c        -pb/+pb,-p/+p - momenta of incoming/outgoing
c        e1,e2,e3 - generalized polarization vectors in the spinor 
c                   representation,
c        g1p,g2p,g3p - vector boson couplings to the fermion line of the 
c                      positive chirality, P+=(1+gamma_5)/2,
c        g1m,g2m,g3m - vector boson couplings to the fermion line of the 
c                      negative chirality, P-=(1-gamma_5)/2,
c        pi - intermidiate fermion momenta in the spinor representation
c             if denominator deni=0, or the photon momentum if deni.ne.0,
c        cmi - intermidiate fermion masses, relevant only if the
c              corresponding denominator deni=0,
c        h - generalized scalar representing the Higgs boson contribution,
c        ih - specifies the position of the scalar coupling, ih=1,2,3 if
c             Higgs couples instead of e1,e2,e3 respectively,
c        nh,nb,n1,n2,n3,n - numbers of polarization states.
c
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 19 April, 2001; modified on 4 May, 2001; 2 November, 2001; 
c                             20 June, 2003
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: nb,n1,n2,n3,n,ih,nh

      complex(dp) :: g1p,g2p,g3p,g1m,g2m,g3m
      real(dp), dimension(0:3) :: pb,p
      complex(dp) :: den1,den2,cm1,cm2,cim1,cim2
      complex(dp), dimension(2,2) :: p1p,p1m,p2p,p2m
      complex(dp), dimension(4,nb) :: bu
      complex(dp), dimension(4,n) :: u
      complex(dp), dimension(2,2,n1) :: e1p,e1m
      complex(dp), dimension(2,2,n2) :: e2p,e2m
      complex(dp), dimension(2,2,n3) :: e3p,e3m
      complex(dp), dimension(nh) :: h
      complex(dp), dimension(nb,n1,n2,n3,n) :: mat

c Local variables:

      integer ib,i1,i2,i3,i

      real(dp) :: d1,d2,im1,im2

      complex(dp), dimension(4,nb,n1) :: bu1,bu11
      complex(dp), dimension(4,n,n3) :: u3,u32
      complex(dp), dimension(n1) :: pbe1
      complex(dp), dimension(n3) :: pe3
      complex(dp) :: cd1,cd2,cp,cm
      complex(dp), parameter :: ocom=(0.d0,0.d0),icom=(0.d0,1.d0)

      im1=-icom*cm1
      im2=-icom*cm2
      cim1=cmplx(0.d0,im1,dp)
      cim2=cmplx(0.d0,im2,dp)

      if (den1 == ocom) then
       cd1=p1p(1,1)*p1p(2,2)-p1p(1,2)*p1p(2,1)-cm1**2
       do ib=1,nb
        do i1=1,n1
         bu1(:2,ib,i1)=g1m*matmul(bu(3:,ib),e1m(:,:,i1))
         bu1(3:,ib,i1)=g1p*matmul(bu(:2,ib),e1p(:,:,i1))
         if (ih == 1) then
          bu1(:,ib,i1)=bu1(:,ib,i1)+h(i1)*bu(:,ib)
         end if
         bu11(:2,ib,i1)=cm1*bu1(:2,ib,i1)+matmul(bu1(3:,ib,i1),p1m)
         bu11(3:,ib,i1)=matmul(bu1(:2,ib,i1),p1p)+cm1*bu1(3:,ib,i1)
        end do
       end do
      else if (g1m == g1p) then
       cd1=den1
       do i1=1,n1
        pbe1(i1)=pb(0)*(e1p(1,1,i1)+e1p(2,2,i1))
     &          +pb(1)*(e1p(2,1,i1)+e1p(1,2,i1))
     &     -icom*pb(2)*(e1p(2,1,i1)-e1p(1,2,i1))
     &          +pb(3)*(e1p(1,1,i1)-e1p(2,2,i1))
        do ib=1,nb
         bu1(:2,ib,i1)=g1m*matmul(bu(3:,ib),e1m(:,:,i1))
         bu1(3:,ib,i1)=g1p*matmul(bu(:2,ib),e1p(:,:,i1))
         if (ih == 1) then
          bu1(:,ib,i1)=bu1(:,ib,i1)+h(i1)*bu(:,ib)
         end if
         bu11(:2,ib,i1)=cim1*bu1(:2,ib,i1)+matmul(bu1(3:,ib,i1),p1m)
         bu11(3:,ib,i1)=matmul(bu1(:2,ib,i1),p1p)+cim1*bu1(3:,ib,i1)
         bu11(:,ib,i1)=g1m*pbe1(i1)*bu(:,ib)+bu11(:,ib,i1)
        end do
       end do
      end if

      if (den2 == ocom) then
       cd2=p2p(1,1)*p2p(2,2)-p2p(1,2)*p2p(2,1)-cm2**2
       do i=1,n
        do i3=1,n3
         u3(:2,i,i3)=g3p*matmul(e3p(:,:,i3),u(3:,i))
         u3(3:,i,i3)=g3m*matmul(e3m(:,:,i3),u(:2,i))
         if (ih == 3) then
          u3(:,i,i3)=u3(:,i,i3)+h(i3)*u(:,i)
         end if
         u32(:2,i,i3)=cm2*u3(:2,i,i3)+matmul(p2p,u3(3:,i,i3))
         u32(3:,i,i3)=matmul(p2m,u3(:2,i,i3))+cm2*u3(3:,i,i3)
        end do
       end do
      else if (g3m == g3p) then
       cd2=den2
       do i3=1,n3
        pe3(i3)=p(0)*(e3p(1,1,i3)+e3p(2,2,i3))
     &         +p(1)*(e3p(2,1,i3)+e3p(1,2,i3))
     &    -icom*p(2)*(e3p(2,1,i3)-e3p(1,2,i3))
     &         +p(3)*(e3p(1,1,i3)-e3p(2,2,i3))
        do i=1,n
         u3(:2,i,i3)=g3p*matmul(e3p(:,:,i3),u(3:,i))
         u3(3:,i,i3)=g3m*matmul(e3m(:,:,i3),u(:2,i))
         if (ih == 3) then
          u3(:,i,i3)=u3(:,i,i3)+h(i3)*u(:,i)
         end if
         u32(:2,i,i3)=cim2*u3(:2,i,i3)-matmul(p2p,u3(3:,i,i3))
         u32(3:,i,i3)=-matmul(p2m,u3(:2,i,i3))+cim2*u3(3:,i,i3)
         u32(:,i,i3)=-g3m*pe3(i3)*u(:,i)+u32(:,i,i3)
        end do
       end do
      end if

      cp=g2p/(cd1*cd2)
      cm=g2m/(cd1*cd2)

      do ib=1,nb
       do i1=1,n1
        do i2=1,n2
         do i3=1,n3
          do i=1,n
ckk>
c           mat(ib,i1,i2,i3,i)=cm*dot_product(conjg(bu11(3:,ib,i1)),
c     &                            matmul(e2m(:,:,i2),u32(:2,i,i3)))
           mat(ib,i1,i2,i3,i)=cm*sum(bu11(3:,ib,i1)*
     &                            matmul(e2m(:,:,i2),u32(:2,i,i3)))
ckk<
           if (cp /= ocom) then
ckk>
c            mat(ib,i1,i2,i3,i)=mat(ib,i1,i2,i3,i)
c     &                       +cp*dot_product(conjg(bu11(:2,ib,i1)),
c     &                            matmul(e2p(:,:,i2),u32(3:,i,i3)))
            mat(ib,i1,i2,i3,i)=mat(ib,i1,i2,i3,i)
     &                       +cp*sum(bu11(:2,ib,i1)*
     &                            matmul(e2p(:,:,i2),u32(3:,i,i3)))
ckk<
           end if
           if (ih == 2) then
ckk>
c            mat(ib,i1,i2,i3,i)=mat(ib,i1,i2,i3,i)+h(i2)/(cd1*cd2)
c     &               *dot_product(conjg(bu11(:,ib,i1)),u32(:,i,i3))
            mat(ib,i1,i2,i3,i)=mat(ib,i1,i2,i3,i)+h(i2)/(cd1*cd2)
     &               *sum(bu11(:,ib,i1)*u32(:,i,i3))
ckk<
           end if
          end do
         end do
        end do
       end do
      end do

      end




