      subroutine ffvgnew(bu1,n1,p1,xp1,yp1,m1,g1,dp1k,
     &                    u2,n2,p2,xp2,yp2,m2,g2,dp2k,
     &                   gvm,gvp,mv2,e,xe,ye,ne,k,xk,yk,
     &                   g,ew,eps,xeps,yeps,ihiggs,gh,mh2,hig)
c----------------------------------------------------------------------c
c Returns a fourvector: 
c
c     eps_mu=g*(bu1*e*(p1-k+m)*gamma_mu*(gvm*P- + gvp*P+)*u2/dp1k
c              +bu1*gamma_mu*(gvm*P- + gvp*P+)*e*(p2-k+m)*u2/dp2k)
c              + gammaWW-vertex contribution
c                               *(-g_mn + q_m*q_n/mv2)/(q^2-mv2)
c
c and, if ihigg=1, a scalar:
c
c     hig=g*(bu1*e*(p1-k+m)*u2/dp1k+bu1*e*(p2-k+m)*u2/dp2k)
c                               *gh/(q^2-mh2)
c
c Input: bu1 - bar spinor,
c        n1 - number of polarization states of u1,
c        -p1/+p1 - momentum of the incoming/outgoing fermion u1,
c        e - real polarization vectors of a photon,
c        xe,ye - the photon real polarization vector in the spinor 
c                representation,
c        g1 - photon coupling to u1,
c        k - the photon momentum,
c        xk,yk - the photon momentum in the spinor representation,
c        dp1k,dp2k - denominators of the fermion propagator,
c        gvm - vector boson coupling to the fermion line of the negative 
c              chirality, P-=(1-gamma_5)/2,
c        gvp - vector boson coupling to the fermion line of the positive
c              chirality, P+=(1+gamma_5)/2,
c        mv2 = mv^2-i*mv*gamv - gauge boson mass squared including 
c                               the width,
c        u2 - spinor.
c        n2 - number of polarization states of u2,
c       -p2/+p2 - momentum of the incoming/outgoing fermion u2,
c        +g/-g - the triple gauge boson coupling for ew coupled to W+/W-.
c        m1,m2 - complex fermion masses
c        ihiggs =1/else - calculate the Higgs boson contribution, yes/no,
c                         respecively,
c        gh - a Higgs boson coupling to the fermion line,
c        mh2 = mh^2-i*mh*gamh - scalar boson mass squared including 
c                               the width.

c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 26/27 April, 2001;  modified on 1 November, 2001; 20 June, 2003
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: n1,n2,ne,ihiggs,i1,i2,ie,lor
      real(dp) :: srr
      complex(dp) :: dp1k,dp2k,m1,m2,cvp,cvm,cg1,cg2,mg1,mg2,src,dq,
     &              coefw,coeff,mv2,mh2,gvm,gvp,gh,g1,g2,g

      complex(dp), dimension(4,n1) :: bu1
      complex(dp), dimension(4,n2) :: u2
      real(dp), dimension(0:3,ne) :: e
      complex(dp), dimension(2,2,ne) :: xe,ye
      real(dp), dimension(0:3) :: p1,p2,k,q,pw
      complex(dp), dimension(2,2) :: xp1,yp1,xp2,yp2,xk,yk
      complex(dp), dimension(2) :: bu1xe,bu1ye,xeu2,yeu2
      complex(dp), dimension(0:3,2,n1) :: bus,busb
      complex(dp), dimension(0:3,2,n2) :: su,sbu
      complex(dp), dimension(4) :: ugen
      complex(dp), dimension(0:3,n1,n2) :: ew
      complex(dp), dimension(0:3) :: epsw
      complex(dp), dimension(0:3,n1,n2,ne) :: eps1,eps2,eps
      complex(dp), dimension(2,2,n1,n2,ne) :: xeps,yeps
      complex(dp), dimension(n1,n2,ne) :: hig

      complex(dp), parameter :: ocom=(0.d0,0.d0),icom=(0.d0,1.d0)


      q=p1+p2+k

      dq=cmplx(srr(q,q),0.d0,dp)-mv2

      cvp=gvp/dq
      cvm=gvm/dq

      if (ihiggs == 1) then
       coeff=gh/(dq+mv2-mh2)
      else
       hig=ocom
      end if

      if (g1 == ocom) then

      eps1=ocom

      else

      do i2=1,n2
       sbu(0,:,i2)=cvm*(/u2(1,i2),u2(2,i2)/)
       sbu(1,:,i2)=-cvm*(/u2(2,i2),u2(1,i2)/)
       sbu(2,:,i2)=-cvm*icom*(/-u2(2,i2),u2(1,i2)/)
       sbu(3,:,i2)=-cvm*(/u2(1,i2),-u2(2,i2)/)
       if (gvp /= ocom) then
        su(0,:,i2)=cvp*(/u2(3,i2),u2(4,i2)/)
        su(1,:,i2)=cvp*(/u2(4,i2),u2(3,i2)/)
        su(2,:,i2)=cvp*icom*(/-u2(4,i2),u2(3,i2)/)
        su(3,:,i2)=cvp*(/u2(3,i2),-u2(4,i2)/)
       end if 
      end do

      cg1=g1/dp1k
      mg1=m1*cg1

      do i1=1,n1
       do ie=1,ne
        bu1ye=matmul(bu1(3:,i1),ye(:,:,ie))
        bu1xe=matmul(bu1(:2,i1),xe(:,:,ie))
        ugen(:2)=cg1*matmul(bu1xe,yp1+yk)+mg1*bu1ye
        ugen(3:)=cg1*matmul(bu1ye,xp1+xk)+mg1*bu1xe
        do i2=1,n2
         do lor=0,3
ckk>
c          eps1(lor,i1,i2,ie)=-dot_product(conjg(ugen(3:)),sbu(lor,:,i2))
          eps1(lor,i1,i2,ie)=-sum(ugen(3:)*sbu(lor,:,i2))
ckk<
          if (gvp /= ocom) then
ckk>
c           eps1(lor,i1,i2,ie)=eps1(lor,i1,i2,ie)
c     &                       -dot_product(conjg(ugen(:2)),su(lor,:,i2))
           eps1(lor,i1,i2,ie)=eps1(lor,i1,i2,ie)
     &                       -sum(ugen(:2)*su(lor,:,i2))
ckk<
          end if
         end do
         if (ihiggs == 1) then
ckk>
c          hig(i1,i2,ie)=coeff*dot_product(conjg(ugen),u2(:,i2))
          hig(i1,i2,ie)=coeff*sum(ugen*u2(:,i2))
ckk<
         end if
        end do
       end do
      end do

      end if ! g1

      if (g2 == ocom) then

      eps2=ocom

      else

      do i1=1,n1
       busb(0,:,i1)=cvm*(/bu1(3,i1),bu1(4,i1)/)
       busb(1,:,i1)=-cvm*(/bu1(4,i1),bu1(3,i1)/)
       busb(2,:,i1)=-icom*cvm*(/bu1(4,i1),-bu1(3,i1)/)
       busb(3,:,i1)=-cvm*(/bu1(3,i1),-bu1(4,i1)/)
       if (gvp /= ocom) then
        bus(0,:,i1)=cvp*(/bu1(1,i1),bu1(2,i1)/)
        bus(1,:,i1)=cvp*(/bu1(2,i1),bu1(1,i1)/)
        bus(2,:,i1)=icom*cvp*(/bu1(2,i1),-bu1(1,i1)/)
        bus(3,:,i1)=cvp*(/bu1(1,i1),-bu1(2,i1)/)
       end if 
      end do

      cg2=-g2/dp2k
      mg2=-m2*cg2

      do i2=1,n2
       do ie=1,ne
        xeu2=matmul(xe(:,:,ie),u2(3:,i2))
        yeu2=matmul(ye(:,:,ie),u2(:2,i2))
        ugen(:2)=cg2*matmul(xp2+xk,yeu2)+mg2*xeu2
        ugen(3:)=cg2*matmul(yp2+yk,xeu2)+mg2*yeu2
        do i1=1,n1
         do lor=0,3
ckk>
c          eps2(lor,i1,i2,ie)=
c     &                      -dot_product(conjg(busb(lor,:,i1)),ugen(:2))
          eps2(lor,i1,i2,ie)=-sum(busb(lor,:,i1)*ugen(:2))
ckk<
          if (gvp /= ocom) then
ckk>
c           eps2(lor,i1,i2,ie)=eps2(lor,i1,i2,ie)
c     &                       -dot_product(conjg(bus(lor,:,i1)),ugen(3:))
           eps2(lor,i1,i2,ie)=eps2(lor,i1,i2,ie)
     &                       -sum(bus(lor,:,i1)*ugen(3:))
ckk<
          end if
         end do
         if (ihiggs == 1) then
ckk>
c          hig(i1,i2,ie)=hig(i1,i2,ie)
c     &                 +coeff*dot_product(conjg(bu1(:,i1)),ugen)
          hig(i1,i2,ie)=hig(i1,i2,ie)+coeff*sum(bu1(:,i1)*ugen)
ckk<
         end if
        end do
       end do
      end do

      end if ! g2



      pw=-p1-p2

      coefw=g/dq

      do i1=1,n1
       do i2=1,n2
        do ie=1,ne
         if (g == ocom) then
          epsw=ocom
         else
          epsw=-coefw*(srr(q-pw,e(:,ie))*ew(:,i1,i2)
     &                    +src(e(:,ie),ew(:,i1,i2))*(pw+k)
     &                    +src(-k-q,ew(:,i1,i2))*e(:,ie))
         end if ! g

         eps(:,i1,i2,ie)=eps1(:,i1,i2,ie)+eps2(:,i1,i2,ie)+epsw

         if (mv2 /= ocom) then
          eps(:,i1,i2,ie)=eps(:,i1,i2,ie)
     &                    -src(q,eps(:,i1,i2,ie))*q/mv2
         end if

         call spinc(eps(:,i1,i2,ie),
     &                        xeps(:,:,i1,i2,ie),yeps(:,:,i1,i2,ie))

        end do
       end do
      end do


      end subroutine





