      subroutine fhf(bu1,n1,p1,gh,q2,mh2,u2,n2,p2,scal)
c----------------------------------------------------------------------c
c Returns a scalar: 
c
c            scal=gh/(q^2-mh2)*bu1*u2
c
c Input: bu1 - bar spinor,
c        -p1/+p1 - momentum of the incoming/outgoing fermion,
c        gh - a Higgs boson coupling to the fermion line,
c        q - four momentum of the intermidiate scalar boson,
c        mh2 = mh^2-i*mh*gamh - scalar boson mass squared including 
c                               the width,
c        q2 - the squared momentum transfer of the Higgs boson propagator,
c        u2 - spinor,
c        -p2/+p2 - momentum of the incoming/outgoing fermion,
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 18 April, 2001; modified on 8 May, 2001; 1 November, 2001
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: n1,n2,i1,i2
      real(dp) :: q2,srr
      complex(dp) :: gh
      real(dp), dimension(0:3) :: p1,p2,q
      complex(dp) :: bu1(4,n1),mh2,u2(4,n2),scal(n1,n2),nom

      q=p1+p2

      if (q2 == 0.d0) then
       nom=gh/(cmplx(srr(q,q),0.d0,dp)-mh2)
      else
       nom=gh/(cmplx(q2,0.d0,dp)-mh2)
      end if

      do i1=1,n1
      do i2=1,n2

       scal(i1,i2)=nom*(bu1(1,i1)*u2(1,i2)+bu1(2,i1)*u2(2,i2)
     &                 +bu1(3,i1)*u2(3,i2)+bu1(4,i1)*u2(4,i2))

      end do ! i1
      end do ! i2

      end subroutine



