      subroutine fsf(ub,s,gsp,gsm,u,scal,nb,ns,n)
c----------------------------------------------------------------------c
c Returns a scalar: 
c
c              scal=ub(ib)*s(is)*(gsm*P- + gsp*P+)*u(i)
c
c Input: ub - a bar spinor,
c        gsm - a complex scalar boson coupling to the fermion line of 
c              the negative chirality, P-=(1-gamma_5)/2,
c        gsp - a complex scalar boson coupling to the fermion line of 
c              the positive chirality, P+=(1+gamma_5)/2,
c        u - a spinor,
c        nb,ns,n - numbers of polarization states.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 13 November, 2000;        modiffied on 18 December, 2000
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: nb,ns,n
      complex(dp) :: ub(4,nb),s(ns),gsm,gsp,u(4,n),scal(nb,ns,n)

      integer :: ib,is,i
      complex(dp) :: sc

      do ib=1,nb
       do i=1,n
        sc=gsm*(ub(1,ib)*u(1,i)+ub(2,ib)*u(2,i))
     &    +gsp*(ub(3,ib)*u(3,i)+ub(4,ib)*u(4,i))
        do is=1,ns
        scal(ib,is,i)=sc*s(is)
        end do
       end do
      end do

      end subroutine



