      subroutine fsfa(bu1,n1,p1,gsp,gsm,q2,ms2,alpha,u2,n2,p2,scal)
c----------------------------------------------------------------------c
c Returns a scalar: 
c
c            scal=bu1*(gsm*P- + gsp*P+)*u2/(q^2-alpha*ms2)
c
c Input: bu1 - bar spinor,
c        n1 - number of polarization states of u1,
c        -p1/+p1 - momentum of the incoming/outgoing fermion,
c        gsm - a complex scalar boson coupling to the fermion line of 
c              the negative chirality, P-=(1-gamma_5)/2,
c        gsp - a complex scalar boson coupling to the fermion line of 
c              the positive chirality, P+=(1+gamma_5)/2,
c        q - four momentum of the intermidiate scalar boson,
c        ms2 = ms^2-i*ms*gams - scalar boson mass squared including 
c                               the width,
c        alpha - a real gauge parameter,
c        q2 - the squared momentum transfer of the scalar boson propagator,
c        u2 - spinor,
c        -p2/+p2 - momentum of the incoming/outgoing fermion,
c        n2 - number of polarization states of u2.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 13 November, 2000
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer n1,n2,i1,i2
      real(dp) q2,alpha,srr
      real(dp), dimension(0:3) :: p1,p2,q
      complex(dp) bu1(4,n1),gsm,gsp,ms2,u2(4,n2),scal(n1,n2),den,
     &           nomm,nomp

      q=p1+p2

      if (q2 == 0.d0) then
       den=cmplx(srr(q,q),0.d0,dp)-alpha*ms2
      else
       den=cmplx(q2,0.d0,dp)-alpha*ms2
      end if

      nomm=gsm/den
      nomp=gsp/den

      do i1=1,n1
      do i2=1,n2

       scal(i1,i2)=nomm*(bu1(1,i1)*u2(1,i2)+bu1(2,i1)*u2(2,i2))
     &            +nomp*(bu1(3,i1)*u2(3,i2)+bu1(4,i1)*u2(4,i2))

      end do ! i1
      end do ! i2

      end subroutine



