c----------------------------------------------------------------------c
c This files contains simple functions and subroutines
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 20 February, 1998
c----------------------------------------------------------------------c

      real(kind(1.d0)) function srr(a,b)
      real(kind(1.d0)) a(0:3),b(0:3)

      srr=a(0)*b(0)-a(1)*b(1)-a(2)*b(2)-a(3)*b(3)

      end


      complex(kind(1.d0)) function src(a,b)
      real(kind(1.d0)) a(0:3)
      complex(kind(1.d0)) b(0:3)

      src=a(0)*b(0)-a(1)*b(1)-a(2)*b(2)-a(3)*b(3)

      end


      complex(kind(1.d0)) function scc(a,b)
      complex(kind(1.d0)) a(0:3),b(0:3)

      scc=a(0)*b(0)-a(1)*b(1)-a(2)*b(2)-a(3)*b(3)

      end


      real(kind(1.d0)) function lamsq(x,y,z)
c----------------------------------------------------------------------c
c The kinematic lambda function, with sqrt(x),sqrt(y),sqrt(z) as 
c arguments instead of x,y,z
c----------------------------------------------------------------------c

      real(kind(1.d0)) x,y,z

      lamsq=(x-y-z)*(x+y+z)*(x-y+z)*(x+y-z)

      if (lamsq.lt.0.d0) then
         lamsq=0.d0
      end if

      end function


      complex(kind(1.d0)) function buou(bu,o,u)
c----------------------------------------------------------------------c
c Returns the product bu*o*u of the 2 x 2 complex matrix o and two 
c 2 dim subspinors
c
c        bu,u - 2 dim spinors,
c        o - 2 x 2 matrix.
c----------------------------------------------------------------------c

      implicit none

      complex(kind(1.d0)) bu(2),u(2),o(2,2)

      buou=bu(1)*(o(1,1)*u(1)+o(1,2)*u(2))
     &    +bu(2)*(o(2,1)*u(1)+o(2,2)*u(2))

      end


      complex(kind(1.d0)) function bsos(iodd,bs,o,ic,s)
c----------------------------------------------------------------------c
c Returns the product bs*o*P_ic*s of the 2 x 2 complex matrix o*P_ic
c and two 2 dim subspinors
c Input: iodd = 0 if o results from a product of an even number of 
c                                                       Dirac matrices,
c        iodd = 1 if o results from a product of an odd number of 
c                                                       Dirac matrices,
c        bs,s - 4 dim spinors,
c        ic = -1 for P-=(1-gamma_5)/2,
c              1 for P+=(1+gamma_5)/2
c----------------------------------------------------------------------c

      implicit none

      integer iodd,ic

      complex(kind(1.d0)) bs(4),s(4),o(2,2)

      if (iodd.eq.0) then

         if (ic.eq.1) then
            bsos=bs(3)*(o(1,1)*s(3)+o(1,2)*s(4))
     &          +bs(4)*(o(2,1)*s(3)+o(2,2)*s(4))
         else
            bsos=bs(1)*(o(1,1)*s(1)+o(1,2)*s(2))
     &          +bs(2)*(o(2,1)*s(1)+o(2,2)*s(2))
         end if

      else

         if (ic.eq.1) then
            bsos=bs(1)*(o(1,1)*s(3)+o(1,2)*s(4))
     &          +bs(2)*(o(2,1)*s(3)+o(2,2)*s(4))
         else
            bsos=bs(3)*(o(1,1)*s(1)+o(1,2)*s(2))
     &          +bs(4)*(o(2,1)*s(1)+o(2,2)*s(2))
         end if

      end if

      end


      subroutine mmul2(a,b,prod)
c----------------------------------------------------------------------c
c Returns a product of two 2 x 2 complex matrices
c----------------------------------------------------------------------c

      implicit none

      complex(kind(1.d0)), dimension (2,2) :: a,b,prod

c      prod=reshape((/a(1,1)*b(1,1)+a(1,2)*b(2,1),
c     &               a(2,1)*b(1,1)+a(2,2)*b(2,1),
c     &               a(1,1)*b(1,2)+a(1,2)*b(2,2),
c     &               a(2,1)*b(1,2)+a(2,2)*b(2,2)/),(/2,2/))


      prod=matmul(a,b)

      end subroutine


      subroutine mmul3(a,b,c,prod)
c----------------------------------------------------------------------c
c Returns a product of three 2 x 2 complex matrices
c----------------------------------------------------------------------c

      implicit none

      complex(kind(1.d0)), dimension (2,2) :: a,b,c,prod

      prod=matmul(matmul(a,b),c)

      end subroutine


      subroutine mmul4(a,b,c,d,prod)
c----------------------------------------------------------------------c
c Returns a product of four 2 x 2 complex matrices
c----------------------------------------------------------------------c

      implicit none

      complex(kind(1.d0)), dimension (2,2) :: a,b,c,d,prod

      prod=matmul(matmul(matmul(a,b),c),d)

      end subroutine


      subroutine mmul5(a,b,c,d,e,prod)
c----------------------------------------------------------------------c
c Returns a product of five 2 x 2 complex matrices
c----------------------------------------------------------------------c

      implicit none

      complex(kind(1.d0)), dimension (2,2) :: a,b,c,d,e,prod

      prod=matmul(matmul(matmul(matmul(a,b),c),d),e)

      end subroutine


      subroutine smul2(a,b,ic,prod)
c----------------------------------------------------------------------c
c Returns a 2 x 2 product of 2 four vectors in the spinor representation.
c Input: a,b - four vectors in the spinor representation,
c        ic = -1 for P-=(1-gamma_5)/2,
c              1 for P+=(1+gamma_5)/2
c----------------------------------------------------------------------c

      implicit none

      integer ic
      complex(kind(1.d0)), dimension (8) :: a,b
      complex(kind(1.d0)) prod(2,2)

      if (ic.eq.-1) then

      prod=reshape((/a(1)*b(5)+a(3)*b(6),
     &               a(2)*b(5)+a(4)*b(6),
     &               a(1)*b(7)+a(3)*b(8),
     &               a(2)*b(7)+a(4)*b(8)/),(/2,2/))

      else

      prod=reshape((/a(5)*b(1)+a(7)*b(2),
     &               a(6)*b(1)+a(8)*b(2),
     &               a(5)*b(3)+a(7)*b(4),
     &               a(6)*b(3)+a(8)*b(4)/),(/2,2/))

      end if

      end subroutine



      subroutine smul3(a,b,c,ic,prod)
c----------------------------------------------------------------------c
c Returns a 2 x 2 product of 3 four vectors in the spinor representation.
c Input: a,b,c - four vectors in the spinor representation,
c        ic = -1 for P-=(1-gamma_5)/2,
c              1 for P+=(1+gamma_5)/2
c----------------------------------------------------------------------c

      implicit none

      integer ic
      complex(kind(1.d0)), dimension (8) :: a,b,c
      complex(kind(1.d0)) prod(2,2)


      if (ic.eq.-1) then

      prod=reshape((/a(5)*(b(1)*c(5)+b(3)*c(6))
     &              +a(7)*(b(2)*c(5)+b(4)*c(6)),
     &               a(6)*(b(1)*c(5)+b(3)*c(6))
     &              +a(8)*(b(2)*c(5)+b(4)*c(6)),
     &               a(5)*(b(1)*c(7)+b(3)*c(8))
     &              +a(7)*(b(2)*c(7)+b(4)*c(8)),
     &               a(6)*(b(1)*c(7)+b(3)*c(8))
     &              +a(8)*(b(2)*c(7)+b(4)*c(8))/),(/2,2/))

      else

      prod=reshape((/a(1)*(b(5)*c(1)+b(7)*c(2))
     &              +a(3)*(b(6)*c(1)+b(8)*c(2)),
     &               a(2)*(b(5)*c(1)+b(7)*c(2))
     &              +a(4)*(b(6)*c(1)+b(8)*c(2)),
     &               a(1)*(b(5)*c(3)+b(7)*c(4))
     &              +a(3)*(b(6)*c(3)+b(8)*c(4)),
     &               a(2)*(b(5)*c(3)+b(7)*c(4))
     &              +a(4)*(b(6)*c(3)+b(8)*c(4))/),(/2,2/))

      end if

      end subroutine



      subroutine smul4(a,b,c,d,ic,prod)
c----------------------------------------------------------------------c
c Returns a 2 x 2 product of 4 four vectors in the spinor representation.
c Input: a,b,c,d - four vectors in the spinor representation,
c        ic = -1 for P-=(1-gamma_5)/2,
c              1 for P+=(1+gamma_5)/2
c----------------------------------------------------------------------c

      implicit none

      integer ic
      complex(kind(1.d0)), dimension (8) :: a,b,c,d
      complex(kind(1.d0)), dimension (2,2) :: prod,o1,o2


      call smul2(a,b,ic,o1)
      call smul2(c,d,ic,o2)

      prod=matmul(o1,o2)

      end subroutine


      subroutine smul5(a,b,c,d,e,ic,prod)
c----------------------------------------------------------------------c
c Returns a 2 x 2 product of 5 four vectors in the spinor representation.
c Input: a,b,c,d,e - four vectors in the spinor representation,
c        ic = -1 for P-=(1-gamma_5)/2,
c              1 for P+=(1+gamma_5)/2
c----------------------------------------------------------------------c

      implicit none

      integer ic
      complex(kind(1.d0)), dimension (8) :: a,b,c,d,e
      complex(kind(1.d0)), dimension (2,2) :: prod,o1,o2

      call smul3(a,b,c,ic,o1)
      call smul2(d,e,ic,o2)

      prod=matmul(o1,o2)


      end subroutine






