      subroutine fvf(bu1,n1,p1,m1,gvm,gvp,q2,mv2,u2,n2,p2,m2,
     &                                                   eps,epsp,epsm)
c----------------------------------------------------------------------c
c Returns a four vector: 
c
c  eps_m=bu1*gamma_n*(gvm*P- + gvp*P+)*u2*(-g_mn + q_m*q_n/mv2)/(q^2-mv2)
c
c Input: bu1 - bar spinor,
c        n1 - number of polarization states of u1,
c        -p1/+p1 - momentum of the incoming/outgoing fermion,
c        gvm - vector boson coupling to the fermion line of the negative 
c              chirality, P-=(1-gamma_5)/2,
c        gvp - vector boson coupling to the fermion line of the positive
c              chirality, P+=(1+gamma_5)/2,
c        q - four momentum of the intermidiate gauge boson,
c        mv2 = mv^2-i*mv*gamv - gauge boson mass squared including 
c                               the width,
c        q2 - the squared momentum transfer of the gauge boson propagator,
c        u2 - spinor,
c        -p2/+p2 - momentum of the incoming/outgoing fermion,
c        n2 - number of polarization states of u2.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 20 February, 1998, modiffied on 25 January, 1999; 6 June, 2000;
c                                 10 November, 2000; 1 November, 2001
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer n1,n2,i1,i2
      real(dp) :: m1,m2,q2,srr
      complex(dp) :: gvm,gvp
      real(dp), dimension(0:3) :: p1,p2,q
      complex(dp) :: bu1(4,n1),mv2,u2(4,n2),eps(0:3,n1,n2),dm,
     &              epsp(2,2,n1,n2),epsm(2,2,n1,n2),den,src
      complex(dp), parameter :: ocom=(0.d0,0.d0), icom=(0.d0,1.d0)

      q=p1+p2

      if (q2 == 0.d0) then
c       den=cmplx(m1**2+m2**2+2.d0*(p1(0)*p2(0)-p1(1)*p2(1)
c     &                        -p1(2)*p2(2)-p1(3)*p2(3)),0.d0,dp)-mv2
       den=cmplx(srr(q,q),0.d0,dp)-mv2
      else
       den=cmplx(q2,0.d0,dp)-mv2
      end if

      do i1=1,n1
      do i2=1,n2

      eps(:,i1,i2)=-gvm*(/bu1(3,i1)*u2(1,i2)+bu1(4,i1)*u2(2,i2),
     &                  -(bu1(3,i1)*u2(2,i2)+bu1(4,i1)*u2(1,i2)),
     &             +icom*(bu1(3,i1)*u2(2,i2)-bu1(4,i1)*u2(1,i2)),
     &                   -bu1(3,i1)*u2(1,i2)+bu1(4,i1)*u2(2,i2)/)

      if (gvp /= ocom) then

      eps(:,i1,i2)=eps(:,i1,i2)
     &       -gvp*(/bu1(1,i1)*u2(3,i2)+bu1(2,i1)*u2(4,i2),
     &              bu1(1,i1)*u2(4,i2)+bu1(2,i1)*u2(3,i2),
     &       -icom*(bu1(1,i1)*u2(4,i2)-bu1(2,i1)*u2(3,i2)),
     &              bu1(1,i1)*u2(3,i2)-bu1(2,i1)*u2(4,i2)/)

      end if  ! gvp

c      if (((m2.ne.0.d0).or.(m1.ne.0.d0)).and.(mv2.ne.ocom)) then
c        dm=((m1*gvm-m2*gvp)*(bu1(1,i1)*u2(1,i2)+bu1(2,i1)*u2(2,i2))
c     &     +(m1*gvp-m2*gvm)*(bu1(3,i1)*u2(3,i2)+bu1(4,i1)*u2(4,i2)))/mv2
c        eps(:,i1,i2)=eps(:,i1,i2)+dm*(p1+p2)
c      end if

      if (mv2.ne.ocom) then
c        eps(:,i1,i2)=eps(:,i1,i2)-src(p1+p2,eps(:,i1,i2))*(p1+p2)/mv2
        eps(:,i1,i2)=eps(:,i1,i2)-src(q,eps(:,i1,i2))/mv2*q
      end if

      eps(:,i1,i2)=eps(:,i1,i2)/den
      call spinc(eps(:,i1,i2),epsp(:,:,i1,i2),epsm(:,:,i1,i2))

      end do ! i1
      end do ! i2

      end subroutine

