c      complex(kind(1.d0)) function eee(g,p1,e1,p2,e2,p3,e3)
c----------------------------------------------------------------------c
c Returns a contraction of the triple gauge boson coupling with 3 
c complex four vectors
c
c Input: g - triple gauge boson coupling,
c        p1 - four momentum of the outgoing W-,
c        e1 - four vector contracted with the outgoing W-,
c        p2 - four momentum of the incoming W-,
c        e2 - four vector contracted with the incoming W-,
c        p3 - four momentum of the neutral gauge boson,
c        e3 - four vector contracted with the neutral gauge boson.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 4 March, 1998
c----------------------------------------------------------------------c

c      implicit none
c      real(kind(1.d0)) g,p1(0:3),p2(0:3),p3(0:3)
c      complex(kind(1.d0)) e1(0:3),e2(0:3),e3(0:3),src,scc

c      save

c      eee=g*(-src(p1+p2,e3)*scc(e1,e2)+scc(e2,e3)*src(p2-p3,e1)
c     &       +src(p3+p1,e2)*scc(e1,e3))

c      end function



      subroutine vee(g,p1,mv2,dp1,p2,e2,p3,e3,eps)
c----------------------------------------------------------------------c
c Returns the four vector eps which is a contraction of the triple gauge 
c              boson coupling with two four vectors
c
c Input: g - triple gauge boson coupling,
c        p1 - four momentum of the outgoing W-,
c        mv2 = mw^2-i*mw*gamw - W boson mass squared including 
c                               the width,
c        dp1 - denominator of W boson propagator,
c        p2 - four momentum of the incoming W-,
c        e2 - four vector contracted with the incoming W-,
c        p3 - four momentum of the neutral gauge boson,
c        e3 - four vector contracted with the neutral gauge boson.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 26 February, 1998
c----------------------------------------------------------------------c

      implicit none

      real(kind(1.d0)) g,p1(0:3),p2(0:3),p3(0:3)

      complex(kind(1.d0)) mv2,dp1,e2(0:3),e3(0:3),eps(0:3),src,scc

      save


      eps=-src(p1+p2,e3)*e2+scc(e2,e3)*(p2-p3)+src(p3+p1,e2)*e3

      eps=g/dp1*(-eps+src(p1,eps)/mv2*p1)


      end subroutine



      subroutine eve(g,p1,e1,p2,mv2,dp2,p3,e3,eps)
c----------------------------------------------------------------------c
c Returns the four vector eps which is a contraction of the triple gauge 
c              boson coupling with two four vectors
c
c Input: g - triple gauge boson coupling,
c        p1 - four momentum of the outgoing W-,
c        e1 - four vector contracted with the outgoing W-,
c        p2 - four momentum of the incoming W-,
c        mv2 = mw^2-i*mw*gamw - W boson mass squared including 
c                               the width,
c        dp2 - denominator of W boson propagator,
c        p3 - four momentum of the neutral gauge boson,
c        e3 - four vector contracted with the neutral gauge boson.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 4 March, 1998
c----------------------------------------------------------------------c

      implicit none

      real(kind(1.d0)) g,p1(0:3),p2(0:3),p3(0:3)

      complex(kind(1.d0)) e1(0:3),mv2,dp2,e3(0:3),eps(0:3),src,scc

      save


      eps=-src(p1+p2,e3)*e1+src(p2-p3,e1)*e3+scc(e1,e3)*(p3+p1)

      eps=g/dp2*(-eps+src(p2,eps)/mv2*p2)

      end subroutine


      subroutine eev(g,p1,e1,p2,e2,p3,mv2,dp3,eps)
c----------------------------------------------------------------------c
c Returns the four vector eps which is a contraction of the triple gauge 
c              boson coupling with two four vectors
c
c Input: g - triple gauge boson coupling,
c        p1 - four momentum of the outgoing W-,
c        e1 - four vector contracted with the outgoing W-,
c        p2 - four momentum of the incoming W-,
c        e2 - four vector contracted with the incoming W-,
c        p3 - four momentum of the neutral gauge boson.
c        mv2 = 0, for the photon,
c            = mz^2-i*mz*gamz, for the Z boson,
c        dp3 - denominator of the neutral gauge boson propagator,
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 4 March, 1998
c----------------------------------------------------------------------c

      implicit none

      real(kind(1.d0)) g,p1(0:3),p2(0:3),p3(0:3)

      complex(kind(1.d0)) e1(0:3),e2(0:3),mv2,dp3,eps(0:3),ocom,src,scc

      data ocom /(1.d0,1.d0)/

      save


      eps=-scc(e1,e2)*(p1+p2)+src(p2-p3,e1)*e2+src(p3+p1,e2)*e1

      if (mv2.eq.ocom) then
         eps=-g/dp3*eps
      else
         eps=g/dp3*(-eps+src(p3,eps)/mv2*p3)
      end if

      end subroutine





