      real(kind(1.d0)) function 
     &                     iomega(ecut,mgamma,ei,mi,m2i,ej,mj,m2j,pipj)
c----------------------------------------------------------------------c
c Returns the soft Bremsstrahlung integral 
c
c      Iomega(i,j)=1/(2pi)^3 d^3k/(2Ek)/(pi.k*pj.k), for i ne j
c
c
c Input: ecut - the soft photon cut,
c        mgamma - the photon mass,
c        ei, ej - energies,
c        mi,mj - masses,
c        m2i,m2j - squared masses,
c        pipj - pi.pj.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 8 Februar, 1999
c----------------------------------------------------------------------c

      implicit none

      real(kind(1.d0)) ecut,mgamma,ei,mi,m2i,ej,mj,m2j,pipj

      integer i
      real(kind(1.d0)) da,a(2),alpha,bi,bj,v,rbi,rbj,ddilog

      save

      da=sqrt(pipj**2-m2i*m2j)
      a(1)=(pipj-da)/m2i
      a(2)=(pipj+da)/m2i

      do i=1,2
       if(a(i)*ei > ej) then
        alpha=a(i)
        exit
       end if
      end do

      bi=sqrt(1.d0-m2i/ei**2)
      bj=sqrt(1.d0-m2j/ej**2)

      rbi=(1.d0-bi)/(1.d0+bi)
      rbj=(1.d0-bj)/(1.d0+bj)
      v=(alpha*mi-mj)*(alpha*mi+mj)/(2.d0*(alpha*ei-ej))

      iomega=alpha/(alpha*mi-mj)/(alpha*mi+mj)
     &      *(2.d0*log(alpha*mi/mj)*log(2.d0*ecut/mgamma)
     &     +((log(rbi))**2/4.d0+ddilog(1.d0-alpha/v*ei*(1.d0+bi))
     &                         +ddilog(1.d0-alpha/v*ei*(1.d0-bi)))
     &     -((log(rbj))**2/4.d0+ddilog(1.d0-ej/v*(1.d0+bj))
     &                         +ddilog(1.d0-ej/v*(1.d0-bj))))

      end

