      subroutine kineeff1(ecm,x1,x2,p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,
     &                                           p4,ps4,m4,dlips,flux)
c----------------------------------------------------------------------c
c Returns cartezian and spherical components of four momenta,
c phase space normalization, dlips, and the flux factor for a 2 --> 2
c process in the c.m.s. 
c
c                       p1 + p2 --> p3 + p4
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 27 November, 2000
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      real(dp) :: ecm,x1,x2,m1,m2,m3,m4,dlips,flux
      real(dp), dimension(0:3) :: p1,p2,p3,p4
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4

c Local variables

      real(dp) :: ecm0=0.d0,pp,pi,pp3,cps,lamsq,ct3,st3,phi3,cf3,sf3,
     &            dct,dphi

      save pi,pp3,cps,ecm0


      if (ecm0 /= ecm) then

c Initial momenta:

      pp=sqrt(lamsq(ecm,m1,m2))/(2.d0*ecm)

      p1=(/sqrt(m1**2+pp**2),0.d0,0.d0,pp/)
      p2=(/sqrt(m2**2+pp**2),0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

      pi=4.d0*atan(1.d0)

      dct=2.d0
      dphi=2.d0*pi

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(16.d0*pi**2*ecm)*dct*dphi

c Flux factor:

      flux=4.d0*ecm*pp

      ecm0=ecm

      end if

c The length of the final state momentum:

      pp3=sqrt(lamsq(ecm,m3,m4))/(2.d0*ecm)

c Angular variables:

      ct3=dct*x1-1.d0
      st3=sqrt(1.d0-ct3**2)

      phi3=dphi*x2
      cf3=cos(phi3)    
      sf3=sin(phi3)    

      p3=(/sqrt(m3**2+pp3**2),pp3*st3*cf3,pp3*st3*sf3,pp3*ct3/)
      p4=(/sqrt(m4**2+pp3**2),-p3(1),-p3(2),-p3(3)/)

      ps3=(/pp3,ct3,st3,cf3,sf3/)
      ps4=(/pp3,-ct3,st3,-cf3,-sf3/)

      dlips=cps*pp3

      end


