      subroutine kineeffff(ite,itw,iw34,iw56,ecm,mv1,gamv1,mv2,gamv2,x,
     &                     p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,
     &                     p4,ps4,m4,p5,ps5,m5,p6,ps6,m6,dlips,flux)
c----------------------------------------------------------------------c
c Returns cartezian and spherical components of four momenta,
c phase space normalization, dlips, and the flux factor for a 2 --> 4
c process in the c.m.s. 
c
c                   p1 + p2 --> p3 + p4 + p5 + p6
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: ite = 0/1(2) if 1/t pole related to the eeg-vertex is 
c                     absent/present in the electron (positron) channel,
c        ite = 3 if 1/t pole related to the eeg-vertex is 
c                     absent,
c        itw = 0 if the angle of a f3+f4 pair is uniformly generated,
c              1 if a f3+f4 pair is parallel to the initial e+,
c              2 if a f3+f4 pair is parallel to the initial e-,
c        iw34 = 0 if the vector boson --> f3 f4 is off the resonance,
c               1 0 the vector boson --> f3 f4 is in the resonance,
c        iw56 = 0 if the vector boson --> f5 f6 is off the resonance,
c               1 0 the vector boson --> f5 f6 is in the resonance,
c        ecm - the c.m.s. energy,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        x(7) - random variables, 0 < x(i) < 1,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 23 February, 1998, modiffied on 19.03.1998, 09.03.1999,
c                                 26.04.2000, 29 May 2000, 
c                                 9 October 2000; 5 May, 2001;
c                                 29 January, 2004
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: ite,itw,iw34,iw56

      real(dp) :: ecm,mv1,gamv1,mv2,gamv2,x(10),m1,m2,m3,m4,m5,m6,
     &           s34,s56,dlips,flux

      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p34
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6

      real(dp) e1,e2,pp,lamsq,s34min,s34max,ds34,s56min,s56max,
     &        ds56,pi,dphi,dlips34,dlips56
      real(dp) ct34,pst
      real(dp) sqs34,sqs56,st34
      real(dp) pp34,e34,e56,pp3,pp5,p0,px,cps
      real(dp) b34,g34,a34,n34,ob34,rb34,dct34,b56,g56,
     &        v34(1:3),pv34,v56(1:3),pv56
      real(dp) :: gvmv1,mv12,gvmv2,mv22,ats34,rs34,ns34,ats56,rs56,ns56
      real(dp) :: s346min,s346max,ds346,s346,sqs346,m12,m22,
     &           m52,a5,da5,ra5,ct5,st5,dct5,v346(1:3),e346,g346,pv346,
     &           dlips346,srr

      real(dp) :: ecm0=0.d0

      save pi,dphi,cps,ecm0,e1,e2,pp,m12,m22


      if (ecm.ne.ecm0) then

      pp=sqrt(lamsq(ecm,m1,m2))/(2.d0*ecm)

      m12=m1**2
      m22=m2**2

      e1=sqrt(m12+pp**2)
      e2=sqrt(m22+pp**2)

      pi=4.d0*atan(1.d0)

      dphi=2.d0*pi

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(dphi)**7/4.d0

c Flux factor:

      flux=4.d0*ecm*pp

      ecm0=ecm

      end if

c Initial momenta:

      p1=(/e1,0.d0,0.d0,pp/)
      p2=(/e2,0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

      gvmv1=gamv1*mv1
      mv12=mv1**2
      gvmv2=gamv2*mv2
      mv22=mv2**2

      if (ite == 0) then

c Integration limits:

      s34min=(m3+m4)**2
      s34max=(ecm-m5-m6)**2

c Independent kinematic variables:

      if (iw34 == 0) then
       ds34=s34max-s34min
       s34=ds34*x(1)+s34min
      else if (iw34 == 1) then
       ats34=atan((s34min-mv12)/gvmv1)
       ns34=gvmv1/(atan((s34max-mv12)/gvmv1)-ats34)
       s34=gvmv1*tan(gvmv1/ns34*x(1)+ats34)+mv12
       if (s34.lt.s34min) then
        s34=s34min
       end if
       ds34=((s34-mv12)**2+gvmv1**2)/ns34
      else if (iw34 == 2) then
       rs34=s34max/s34min
       ns34=1.d0/log(rs34)
       s34=s34min*rs34**x(1)
       ds34=s34/ns34
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

      s56min=(m5+m6)**2
      s56max=(ecm-sqs34)**2

      if (iw56 == 0) then
       ds56=s56max-s56min
       s56=ds56*x(2)+s56min
      else if (iw56 == 1) then
       ats56=atan((s56min-mv22)/gvmv2)
       ns56=gvmv2/(atan((s56max-mv22)/gvmv2)-ats56)
       s56=gvmv2*tan(gvmv2/ns56*x(2)+ats56)+mv22
       if (s56.lt.s56min) then
        s56=s56min
       end if
       ds56=((s56-mv22)**2+gvmv2**2)/ns56
      else if (iw56 == 2) then
       rs56=s56max/s56min
       if(s56max < s56min) then
        write(*,*)'kineeffff.f: s56max < s56min',s56max,s56min
       end if
       ns56=1.d0/log(rs56)
       s56=s56min*rs56**x(2)
       ds56=s56/ns56
      else
       write(*,*)'iw56 =',iw56,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)

      pp34=sqrt(lamsq(ecm,sqs34,sqs56))/(2.d0*ecm)

      e34=sqrt(s34+pp34**2)
      e56=sqrt(s56+pp34**2)
      b34=pp34/e34
      g34=e34/sqs34
      b56=-pp34/e56
      g56=e56/sqs56

      if (itw == 0) then

       dct34=2.d0
       ct34=dct34*x(3)-1.d0

      else

       a34=(ecm*e34-s34)/(ecm*pp34)
       ob34=a34-1.d0
       if (abs(ob34) < 1.d-9) then
        ob34=2.d0*s34*s56/(ecm**2-s34-s56)**2
        rb34=ob34/(a34+1.d0)
       else
        rb34=ob34/(a34+1.d0)
       end if

       n34=-1.d0/log(rb34)

       if (itw == 1) then
        ct34=a34-(a34+1.d0)*rb34**x(3)
        dct34=(a34-ct34)/n34
       else if (itw == 2) then
        ct34=ob34/rb34**x(3)-a34
        dct34=(a34+ct34)/n34
       else
        write(*,*)'kineeff.f: itw =',itw,' IS NOT ALLOWED'
        stop
       end if

      end if

c The momentum p3 + p4 has been generated in the xOz plane.

c Other useful variables:

      if (ct34 >= 1.d0) then
       ct34=1.d0
       st34=0.d0
      else if (ct34 <= -1.d0) then
       ct34=-1.d0
       st34=0.d0
      else
       st34=sqrt(1.d0-ct34**2)
      end if

c p3 and p4 in the the c.m.s. frame of particles 3 and 4:

      call kinff(sqs34,m3,m4,x(4:5),p3,p4,pp3,dlips34)

c p5 and p6 in the the c.m.s. frame of particles 5 and 6:

      call kinff(sqs56,m5,m6,x(6:7),p5,p6,pp5,dlips56)

c Velocity of the c.m.s. of particles 3 and 4 in the c.m.s. of particles
c 3, 4, 5 and 6:

      v34=b34*(/st34,0.d0,ct34/)

c Boost back to the c.m.s. of particles 3, 4, 5 and 6:

      pv34=dot_product(v34,p3(1:))
      p0=p3(0)
      p3(0)=g34*(p0+pv34)
      p3(1:)=p3(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

      pv34=dot_product(v34,p4(1:))
      p0=p4(0)
      p4(0)=g34*(p0+pv34)
      p4(1:)=p4(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

c Velocity of the c.m.s. of particles 5 and 6 in the c.m.s. of particles
c 3, 4, 5 and 6:

      v56=b56*(/st34,0.d0,ct34/)

c Boost back to the c.m.s. of particles 3, 4, 5 and 6:

      pv56=dot_product(v56,p5(1:))
      p0=p5(0)
      p5(0)=g56*(p0+pv56)
      p5(1:)=p5(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

      pv56=dot_product(v56,p6(1:))
      p0=p6(0)
      p6(0)=g56*(p0+pv56)
      p6(1:)=p6(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

c The phase space normalization:

      dlips=cps*pp34/ecm*ds34*ds56*dct34*dlips34*dlips56

      else if (ite > 0) then

c Generate s346=(p3+p4+p6)^2

      s346min=(m3+m4+m6)**2
      s346max=(ecm-m5)**2
      ds346=s346max-s346min

      s346=ds346*x(1)+s346min
      sqs346=sqrt(s346)

c p5(0) in the c.m.s.:

      m52=m5**2
      p5(0)=(ecm**2-s346+m52)/(2.d0*ecm)

      pp5=sqrt(p5(0)**2-m52)

c Generate ct5 in the c.m.s.:

      if (ite < 3) then

      if (ite == 1) then
       da5=(p2(0)-p5(0))*(m52*p2(0)-m22*p5(0))/(2.d0*(p2(0)*p5(0))**2)
      else if (ite == 2) then
       da5=(p1(0)-p5(0))*(m52*p1(0)-m12*p5(0))/(2.d0*(p1(0)*p5(0))**2)
      end if

      if (da5 < 1.d-14) then
       da5=1.d-14
      end if

      if (da5 < 1.d-9) then
       a5=1.d0+da5
      else
       if (ite == 1) then
        a5=(p2(0)*p5(0)-(m22+m52)/2.d0)/(pp*pp5)
       else if (ite == 2) then
        a5=(p1(0)*p5(0)-(m12+m52)/2.d0)/(pp*pp5)
       end if
       da5=a5-1.d0
      end if

      ra5=(a5+1.d0)/da5

      if (ite == 1) then
       ct5=da5*ra5**x(2)-a5
      else if (ite == 2) then
       ct5=a5-(a5+1.d0)/ra5**x(2)
      end if

      else ! ite >= 3

      dct5=2.d0
      ct5=dct5*x(2)-1.d0

      end if ! ite

      if (abs(ct5) <= 1.d0) then
       st5=sqrt(1.d0-ct5**2)
      else if (ct5 < -1.d0) then
       ct5=-1.d0
       st5=0.d0
      else if (ct5 > 1.d0) then
       ct5=1.d0
       st5=0.d0
      end if

      if (ite == 1) then
       dct5=log(ra5)*(a5+ct5)
      else if (ite == 2) then
       dct5=log(ra5)*(a5-ct5)
      end if

      p5(1:)=pp5*(/st5,0.d0,ct5/)

c Generate s34=(p3+p4)^2

      s34min=(m3+m4)**2
      s34max=(sqs346-m6)**2

      if (iw34 == 0) then
       ds34=s34max-s34min
       s34=ds34*x(3)+s34min
      else if (iw34 == 1) then
       ats34=atan((s34min-mv12)/gvmv1)
       ns34=gvmv1/(atan((s34max-mv12)/gvmv1)-ats34)
       s34=gvmv1*tan(gvmv1/ns34*x(3)+ats34)+mv12
       if (s34 < s34min) then
        s34=s34min
       end if
       ds34=((s34-mv12)**2+gvmv1**2)/ns34
      else if (iw34 == 2) then
       rs34=s34max/s34min
       ns34=1.d0/log(rs34)
       s34=s34min*rs34**x(3)
       ds34=s34/ns34
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

c p3 and p4 in the the c.m.s. frame of particles 3 and 4:

      call kinff(sqs34,m3,m4,x(4:5),p3,p4,pp3,dlips34)

c p3+p4 and p6 in the the c.m.s. frame of particles 3, 4 and 6:

      call kinff(sqs346,sqs34,m6,x(6:7),p34,p6,pp34,dlips346)

c Velocity of the c.m.s. of particles 3 and 4 in the c.m.s. of particles
c 3, 4 and 6:

      e34=sqrt(s34+pp34**2)
      v34=p34(1:)/e34
      g34=e34/sqs34

c Boost p3 and p4 back to the c.m.s. of particles 3, 4 and 6:

      pv34=dot_product(v34,p3(1:))
      p0=p3(0)
      p3(0)=g34*(p0+pv34)
      p3(1:)=p3(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

      pv34=dot_product(v34,p4(1:))
      p0=p4(0)
      p4(0)=g34*(p0+pv34)
      p4(1:)=p4(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

c Velocity of the c.m.s. of particles 3, 4 and 6 in the c.m.s:

      e346=sqrt(s346+pp5**2)
      v346=-p5(1:)/e346
      g346=e346/sqs346

c Boost p3, p4 and p6 back to the c.m.s.:

      pv346=dot_product(v346,p3(1:))
      p0=p3(0)
      p3(0)=g346*(p0+pv346)
      p3(1:)=p3(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

      pv346=dot_product(v346,p4(1:))
      p0=p4(0)
      p4(0)=g346*(p0+pv346)
      p4(1:)=p4(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

      pv346=dot_product(v346,p6(1:))
      p0=p6(0)
      p6(0)=g346*(p0+pv346)
      p6(1:)=p6(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

c The phase space normalization:

      dlips=cps*pp5/ecm*ds346*dct5*ds34*dlips346*dlips34

      else ! ite

      write(*,*)'ite =',ite,' is not allowed'
      stop

      end if ! ite


c Spherical components of the momenta p3,...,p6:

        ps3(0)=sqrt(p3(1)**2+p3(2)**2+p3(3)**2)
        ps3(1)=p3(3)/ps3(0)
        if (abs(ps3(1)) < 1.d0) then
           ps3(2)=sqrt(1.d0-ps3(1)**2)
           pst=ps3(0)*ps3(2)
           ps3(3)=p3(1)/pst
           ps3(4)=p3(2)/pst
        else if (ps3(1) >=1.d0) then
           ps3(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps3(1) <= -1.d0) then
           ps3(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps4(0)=sqrt(p4(1)**2+p4(2)**2+p4(3)**2)
        ps4(1)=p4(3)/ps4(0)
        if (abs(ps4(1)) < 1.d0) then
           ps4(2)=sqrt(1.d0-ps4(1)**2)
           pst=ps4(0)*ps4(2)
           ps4(3)=p4(1)/pst
           ps4(4)=p4(2)/pst
        else if (ps4(1) >= 1.d0) then
           ps4(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps4(1) <= -1.d0) then
           ps4(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps5(0)=sqrt(p5(1)**2+p5(2)**2+p5(3)**2)
        ps5(1)=p5(3)/ps5(0)
        if (abs(ps5(1)) < 1.d0) then
           ps5(2)=sqrt(1.d0-ps5(1)**2)
           pst=ps5(0)*ps5(2)
           ps5(3)=p5(1)/pst
           ps5(4)=p5(2)/pst
        else if (ps5(1) >= 1.d0) then
           ps5(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps5(1) <= -1.d0) then
           ps5(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps6(0)=sqrt(p6(1)**2+p6(2)**2+p6(3)**2)
        ps6(1)=p6(3)/ps6(0)
        if (abs(ps6(1)) < 1.d0) then
           ps6(2)=sqrt(1.d0-ps6(1)**2)
           pst=ps6(0)*ps6(2)
           ps6(3)=p6(1)/pst
           ps6(4)=p6(2)/pst
        else if (ps6(1) >= 1.d0) then
           ps6(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps6(1) <= -1.d0) then
           ps6(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

      end


