      subroutine kineeffffg(ibr,ict,ite,itw,iw34,iw56,ie3,ict3,
     &                 ecm,ecut,mv1,gamv1,mv2,gamv2,x,m1,m2,m3,m4,m5,m6,
     &                 p1,ps1,p2,ps2,p3,ps3,p4,ps4,p5,ps5,p6,ps6,p7,ps7,
     &                                                       dlips,flux)
c----------------------------------------------------------------------c
c Returns a set of cartezian and spherical components of external 
c particle four momenta, phase space normalization, dlips, and the flux 
c factor for a 2 --> 5  process in c the c.m.s. 
c                p1 + p2 --> p3 + p4 + p5 + p6 + p7,
c
c where p7 is considered massless.
c The phase space integral for massless particles is given by
c          Lips=(2.d0/3.d0)**2/(8.d0*pi)**7*(ecm**2)**3.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p). 
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum p(1:3,i),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: ibr = 1, for the initial state bremssrahlung,
c              2, for the initial state bremssrahlung,
c        ict = 0 if the initial state photon angle is uniformly 
c                distributed,
c              1 if the initial state photon angle is distributed as
c                1/(1-betae*ct),
c              2 if the initial state photon angle is distributed as
c                1/(1+betae*ct),
c        ite = 0/1 if 1/t pole related to the eeg-vertex is absent/present,
c        itw = 0 if the angle of a f3+f4 pair is uniformly generated,
c              1 if a f3+f4 pair is parallel to the initial e+,
c              2 if a f3+f4 pair is parallel to the initial e-,
c        iw34 = 0 if the W --> f3 f4 is off the resonance,
c               1 0 the W --> f3 f4 is in the resonance,
c        iw56 = 0 if the W --> f5 f6 is off the resonance,
c               1 0 the W --> f5 f6 is in the resonance,
c        ie3 = 0 if energy of particle 3 is uniformly generated,
c              1 if it is generated according to 1/(p4.p7),
c        ict3 = 0/1 if 1/t pole related to the eeg-vertex is absent/present
c               in the 3 body decay channel,
c        ecm - the c.m.s. energy,
c        ecut - lower cut on the energy of the photon,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        x(10) - random variables, 0 < x(i) < 1,
c        m1,...,m6 - particle masses.
c
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 24 February, 1999; modified on 30 May, 2000; 1 June, 2000; 
c                              6 June, 2000; 5 May, 2001;
c                              2 February, 2004
c----------------------------------------------------------------------c

      use cutvalues

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: ibr,iw34,iw56,ie3,ite,ict3,itw,ict

      real(dp) :: ecm,ecut,mv1,gamv1,mv2,gamv2,x(10),m1,m2,m3,m4,m5,m6,
     &           dlips,flux
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p34,p347
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7

c Local variables:

      integer :: i

      real(dp) :: ecm0=0.d0,s,m12,m22,m34,m56,m346,msum,pp,lamsq,
     &           e7min,e7max,de7,s34min,s34max,s56min,s56max,pi,dphi,
     &           cps1,cps20,cps21

      real(dp) ct7,s34,s56,dct34,ct34

      real(dp) sp,sqsp,pp7,st7,ds34,sqs34,ds56,sqs56,
     &        st34,pst

      real(dp) pp34,e34,e56,pp3,pp5,p0,px,py,pz,b34,ob34,rb34,g34,
     &        a34,b56,g56,gamma,beta,e3456,b347,g347

      real(dp) betae,dbe2,dbe,rbe,nct7,re7,ne7,x7,dct7,ct70
      real(dp) :: gvmv1,mv12,gvmv2,mv22,ats34,rs34,ns34,ats56,rs56,ns56,
     &           ats347,rs347,ns347

      real(dp) ds347,s347,s347min,s347max,sqs347,pp347,e347,dct347,
     &        ct347,st347,dlips347,dlips34,dlips56,ecutp,a347,
     &        ob347,rb347,n34,n347
      real(dp) pv34,pv346,pv56,pv,pv347,phi34,de5,s3467,sqs3467,
     &        e3467,b3467,g3467,pv3467

      real(dp), dimension(1:3) :: r34,v34,v346,v56,v3456,v347,r5,r7,
     &                           v3467,r347,p5p,p2p

      real(dp) ptot(0:3),dev,dsqs34,dsqs56,srr
      real(dp) s346,sqs346,ds346,s346min,s346max,e346,g346,g3456,m52,
     &        dlips346,e2,pv3456,da5,a5,ra5,ct5,st5,dct5,phi5,dlips3467,
     &        dm2,de,e5,db5,beta5,rb5,phi7,pp5p,ct5p,st5p,cf5p,sf5p,
     &        pp2p,ct2p,st2p

      save s,m12,m22,dbe2,dbe,betae,rbe,pp,pi,dphi,cps1,
     &     cps20,cps21,ecm0


      if (ecm.ne.ecm0) then

      pi=4.d0*atan(1.d0)
      dphi=2.d0*pi

      s=ecm**2

c Initial momenta for m1=m2:

      m12=m1**2
      m22=m12

      p1(0)=ecm/2.d0
      p2(0)=p1(0)

      dbe2=(m1/p1(0))**2
      dbe=dbe2/2.d0
      betae=1.d0-dbe
c      dbe=1.d0-sqrt(1.d0-dbe2)
c      betae=sqrt(1.d0-dbe2)

      if (icuts == 1) then
       rbe=(1.d0+ctgb)/(1.d0-ctgb)
      else
       rbe=(1.d0+betae)**2/dbe2
      end if

      pp=betae*p1(0)

      p1(1:)=(/0.d0,0.d0,pp/)
      p2(1:)=(/0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

c Constant parts of the phase space and Jakobian:

      cps1=1.d0/(dphi)**10/2.d0/4.d0*dphi
      cps20=1.d0/dphi**10/4.d0
      cps21=cps20*2.d0

c Flux factor:

      flux=4.d0*ecm*pp

      ecm0=ecm

      end if ! ecm0

      gvmv1=gamv1*mv1
      mv12=mv1**2
      gvmv2=gamv2*mv2
      mv22=mv2**2

      m34=m3+m4
      m56=m5+m6
      msum=m34+m56

c Kinematic variables:

      if (ibr == 1) then

c energy and angles of particle 7 in the c.m.s.:

      e7min=ecut
      e7max=(ecm-msum)*(ecm+msum)/(2.d0*ecm)
      re7=e7max/e7min
      ne7=1.d0/log(re7)

      p7(0)=e7min*re7**x(1)
      de7=p7(0)/ne7
      pp7=p7(0)

      if (ict == 0) then
        dct7=2.d0
        ct7=dct7*x(2)-1.d0
        st7=sqrt(1.d0-ct7**2)
      else
       nct7=betae/log(rbe)
       if (ict == 1) then

        if (icuts == 1) then
         ct7=(1.d0-(1.d0+ctgb*betae)/rbe**x(2))/betae
        else
         ct7=(1.d0-(1.d0+betae)/rbe**x(2))/betae
        end if

        if (ct7 >= 1.d0) then
         ct7=1.d0
         dct7=dbe/nct7
         st7=0.d0
        else if (ct7 <= -1.d0) then
         ct7=-1.d0
         dct7=(2.d0-dbe)/nct7
         st7=0.d0
        else
c         dct7=(1.d0-betae*ct7)/nct7
         dct7=(1.d0-ct7+dbe*ct7)/nct7
         st7=sqrt(1.d0-ct7**2)
        end if
       else if (ict == 2) then
c        ct7=((1.d0-betae)*rbe**x(2)-1.d0)/betae
        if (icuts == 1) then
         ct7=((1.d0-ctgb*betae)*rbe**x(2)-1.d0)/betae
        else
         ct7=(dbe*rbe**x(2)-1.d0)/betae
        end if
        if (ct7 <= -1.d0) then
         ct7=-1.d0
         dct7=dbe/nct7
         st7=0.d0
        else if (ct7 >= 1.d0) then
         ct7=1.d0
         dct7=(2.d0-dbe)/nct7
         st7=0.d0
        else
c         dct7=(1.d0+betae*ct7)/nct7
         dct7=(1.d0+ct7-dbe*ct7)/nct7
         st7=sqrt(1.d0-ct7**2)
        end if
       else
        write(*,*)'ict=',ict,' IS NOT ALLOWED'
       end if
      end if

c      nct7=betae/log(rbe)

c      x7=rbe**(2.d0*x(2)-1.d0)
c      ct7=(x7-1.d0)/(x7+1.d0)/betae

c      if (ct7 >= 1.d0) then
c       ct7=1.d0
c       dct7=dbe2/nct7
c       st7=0.d0
c      else if (ct7 <= -1.d0) then
c       ct7=-1.d0
c       dct7=dbe2/nct7
c       st7=0.d0
c      else
c       dct7=(1.d0-ct7**2+ct7**2*dbe2)/nct7
c       st7=sqrt(1.d0-ct7**2)
c      end if

c momentum of the photon in the c.m.s.:

       r7=(/st7,0.d0,ct7/)
       p7(1:)=pp7*r7
       ps7=(/pp7,ct7,st7,1.d0,0.d0/)

c sp=(p3+p4+p5+p6)^2:

      sp=ecm*(ecm-2.d0*p7(0))
      sqsp=sqrt(sp)

      if (ite == 0) then

c s34=(p3+p4)^2:

       s34min=m34**2
       s34max=(sqsp-m56)**2

      if (iw34 == 0) then
       ds34=s34max-s34min
       s34=ds34*x(4)+s34min
      else if (iw34 == 1) then
       ats34=atan((s34min-mv12)/gvmv1)
       ns34=gvmv1/(atan((s34max-mv12)/gvmv1)-ats34)
       s34=gvmv1*tan(gvmv1/ns34*x(4)+ats34)+mv12
       ds34=((s34-mv12)**2+gvmv1**2)/ns34
      else if (iw34 == 2) then
       rs34=s34max/s34min
       s34=s34min*rs34**x(4)
       ds34=s34*log(rs34)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

c s56=(p5+p6)^2:

      s56min=m56**2
      s56max=(sqsp-sqs34)**2

      if (iw56 == 0) then
       ds56=s56max-s56min
       s56=ds56*x(5)+s56min
      else if (iw56 == 1) then
       ats56=atan((s56min-mv22)/gvmv2)
       ns56=gvmv2/(atan((s56max-mv22)/gvmv2)-ats56)
       s56=gvmv2*tan(gvmv2/ns56*x(5)+ats56)+mv22
       ds56=((s56-mv22)**2+gvmv2**2)/ns56
      else if (iw56 == 2) then
       rs56=s56max/s56min
       s56=s56min*rs56**x(5)
       ds56=s56*log(rs56)
      else
       write(*,*)'iw56 =',iw56,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)

c the length of p3+p4 in the frame where p3+p4+p5+p6=0:

      pp34=sqrt(lamsq(sqsp,sqs34,sqs56))/(2.d0*sqsp)

      if (pp34 < 0.d0) then
       pp34=0.d0
      end if

      e34=sqrt(s34+pp34**2)
      g34=e34/sqs34
      b34=pp34/e34
      e56=sqrt(s56+pp34**2)
      g56=e56/sqs56
      b56=-pp34/e56


c polar angle of p3+p4 in the c.m.s. of particles 3,4,5 and 6, p3+p4+p5+p6=0,
c azimuthal angle is assumed to be zero, ie. the three momentum p3+p4 is 
c generated in the xOz plane in this frame:


      if (itw == 0) then

       dct34=2.d0
       ct34=dct34*x(6)-1.d0

      else

       a34=(sqsp*e34-s34)/(sqsp*pp34)
       ob34=a34-1.d0
       if (abs(ob34) < 1.d-9) then
        ob34=2.d0*s34*s56/(sp-s34-s56)**2
        rb34=ob34/(a34+1.d0)
       else
        rb34=ob34/(a34+1.d0)
       end if

       n34=-1.d0/log(rb34)

       if (itw == 1) then
        ct34=a34-(a34+1.d0)*rb34**x(6)
        dct34=(a34-ct34)/n34
       else if (itw == 2) then
        ct34=ob34/rb34**x(6)-a34
        dct34=(a34+ct34)/n34
       else
        write(*,*)'kineeff.f: itw =',itw,' IS NOT ALLOWED'
        stop
       end if

      end if

c The momentum p3 + p4 has been generated in the xOz plane.

c Other useful variables:

      if (ct34 >= 1.d0) then
       ct34=1.d0
       st34=0.d0
      else if (ct34 <= -1.d0) then
       ct34=-1.d0
       st34=0.d0
      else
       st34=sqrt(1.d0-ct34**2)
      end if

      phi34=dphi*x(3)

c p3 and p4 in the the c.m.s. of particles 3 and 4:

      call kinff(sqs34,m3,m4,x(7:8),p3,p4,pp3,dlips34)

c p5 and p6 in the the c.m.s. of particles 5 and 6:

      call kinff(sqs56,m5,m6,x(9:10),p5,p6,pp5,dlips56)

c Velocity of the c.m.s. of particles 3 and 4 in the c.m.s. of particles
c 3, 4, 5 and 6:

      r34=(/st34*cos(phi34),st34*sin(phi34),ct34/)
      v34=b34*r34

c Boost back to the c.m.s. of particles 3, 4, 5 and 6:

      pv34=dot_product(v34,p3(1:))
      p0=p3(0)
      p3(0)=g34*(p0+pv34)
      p3(1:)=p3(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

      pv34=dot_product(v34,p4(1:))
      p0=p4(0)
      p4(0)=g34*(p0+pv34)
      p4(1:)=p4(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

c Velocity of the c.m.s. of particles 5 and 6 in the c.m.s. of particles
c 3, 4, 5 and 6:

      v56=b56*r34

c Boost back to the c.m.s. of particles 3, 4, 5 and 6:

      pv56=dot_product(v56,p5(1:))
      p0=p5(0)
      p5(0)=g56*(p0+pv56)
      p5(1:)=p5(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

      pv56=dot_product(v56,p6(1:))
      p0=p6(0)
      p6(0)=g56*(p0+pv56)
      p6(1:)=p6(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

c Boost back to the c.m.s.:

c      e3456=sqrt(sp+pp7**2)
      e3456=ecm-pp7

      gamma=e3456/sqsp
      beta=-pp7/e3456

      v3456=beta*r7

      pv=dot_product(v3456,p3(1:))
      p0=p3(0)
      p3(0)=gamma*(p0+pv)
      p3(1:)=p3(1:)+gamma*(gamma*pv/(gamma+1.d0)+p0)*v3456

      pv=dot_product(v3456,p4(1:))
      p0=p4(0)
      p4(0)=gamma*(p0+pv)
      p4(1:)=p4(1:)+gamma*(gamma*pv/(gamma+1.d0)+p0)*v3456

      pv=dot_product(v3456,p5(1:))
      p0=p5(0)
      p5(0)=gamma*(p0+pv)
      p5(1:)=p5(1:)+gamma*(gamma*pv/(gamma+1.d0)+p0)*v3456

      pv=dot_product(v3456,p6(1:))
      p0=p6(0)
      p6(0)=gamma*(p0+pv)
      p6(1:)=p6(1:)+gamma*(gamma*pv/(gamma+1.d0)+p0)*v3456


c The phase space normalization including jakobian
c                          (=de7*dct7*ds34*ds56*(2.d0)**3*(dphi)**3)

      dlips=cps1*pp7*pp34/sqsp*de7*dct7*dct34*ds34*ds56*dlips34*dlips56

c The difference in normalization of the 4 and 5 particle phase space:
c      dlips=pp7*de7*dct7/2.d0/(dphi)**2

      else if (ite == 1) then

c Generate s346=(p3+p4+p6)^2

      s346min=(m34+m6)**2
      s346max=(sqsp-m5)**2
      ds346=s346max-s346min

      s346=ds346*x(4)+s346min
      sqs346=sqrt(s346)

c Generate s34=(p3+p4)^2

      s34min=(m34)**2
      s34max=(sqs346-m6)**2

      if (iw34 == 0) then
       ds34=s34max-s34min
       s34=ds34*x(5)+s34min
      else if (iw34 == 1) then
       ats34=atan((s34min-mv12)/gvmv1)
       ns34=gvmv1/(atan((s34max-mv12)/gvmv1)-ats34)
       s34=gvmv1*tan(gvmv1/ns34*x(5)+ats34)+mv12
       if (s34 < s34min) then
        s34=s34min
       end if
       ds34=((s34-mv12)**2+gvmv1**2)/ns34
      else if (iw34 == 2) then
       rs34=s34max/s34min
       ns34=1.d0/log(rs34)
       s34=s34min*rs34**x(5)
       ds34=s34/ns34
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

c p3 and p4 in the the c.m.s. frame of particles 3 and 4:

      call kinff(sqs34,m3,m4,x(6:7),p3,p4,pp3,dlips34)

c p3+p4 and p6 in the the c.m.s. frame of particles 3, 4 and 6:

      call kinff(sqs346,sqs34,m6,x(8:9),p34,p6,pp34,dlips346)

c Velocity of the c.m.s. of particles 3, 4, 5 and 6 in the c.m.s.:

      e3456=sqrt(sp+pp7**2)
      v3456=-p7(1:)/e3456
      g3456=e3456/sqsp

c Boost p2 to the c.m.s. of particles 3, 4, 5 and 6:

      if (ict < 2) then
c       pv3456=dot_product(v3456,p2(1:))
       pv3456=v3456(3)*p2(3)
       p0=p2(0)
       e2=g3456*(p0-pv3456)
       p2p=p2(1:)+g3456*(g3456*pv3456/(g3456+1.d0)-p0)*v3456
      else 
       p2p=p2(1:)-p7(1:)
       pv3456=dot_product(v3456,p2p)
       p0=p2(0)-p7(0)
       e2=g3456*(p0-pv3456)
       p2p=p2p+g3456*(g3456*pv3456/(g3456+1.d0)-p0)*v3456
      end if

c Spherical components of p2p:

      pp2p=sqrt(p2p(1)**2+p2p(2)**2+p2p(3)**2)

      ct2p=p2p(3)/pp2p
      st2p=p2p(1)/pp2p

c Momentum of particle 5 in the c.m.s. of particles 3, 4, 5 and 6.

      pp5=sqrt(lamsq(sqsp,sqs346,m5))/(2.d0*sqsp)
      m52=m5**2
      p5(0)=sqrt(m52+pp5**2)


      if (ict < 2) then
       da5=(e2-p5(0))*(m52*e2-m22*p5(0))/(2.d0*(e2*p5(0))**2)
       if (da5 < 1.d-9) then
        a5=1.d0+da5
       else
        a5=(e2*p5(0)-(m22+m52)/2.d0)/(pp2p*pp5)
        da5=a5-1.d0
       end if
      else
       a5=(e2*p5(0)-(e2**2-pp2p**2+m52)/2.d0)/(pp2p*pp5)
       da5=a5-1.d0
       if (da5 < 1.d-14) then
        da5=1.d-14
        a5=1.d0+da5
       end if
      end if

      ra5=(a5+1.d0)/da5

      ct5=a5-(a5+1.d0)/ra5**x(10)

      if (abs(ct5) < 1.d0) then
       st5=sqrt(1.d0-ct5**2)
      else if (ct5 <= -1.d0) then
       ct5=-1.d0
       st5=0.d0
      else if (ct5 >= 1.d0) then
       ct5=1.d0
       st5=0.d0
      end if

      dct5=log(ra5)*(a5-ct5)

      phi5=dphi*x(3)

      p5(1:)=pp5*(/st5*cos(phi5),st5*sin(phi5),ct5/)

c Rotation of the frame by angle theta_2p in the xOz plane:

      px=p5(1)
      pz=p5(3)
      p5(1)=px*ct2p+pz*st2p
      p5(3)=-px*st2p+pz*ct2p

c Velocity of the c.m.s. of particles 3 and 4 in the c.m.s. of particles
c 3, 4 and 6:

      e34=sqrt(s34+pp34**2)
      v34=p34(1:)/e34
      g34=e34/sqs34

c Boost p3 and p4 back to the c.m.s. of particles 3, 4 and 6:

      pv34=dot_product(v34,p3(1:))
      p0=p3(0)
      p3(0)=g34*(p0+pv34)
      p3(1:)=p3(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

      pv34=dot_product(v34,p4(1:))
      p0=p4(0)
      p4(0)=g34*(p0+pv34)
      p4(1:)=p4(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

c Velocity of the c.m.s. of particles 3, 4 and 6 in the c.m.s. of particles
c 3, 4, 5 and 6:

      e346=sqrt(s346+pp5**2)
      v346=-p5(1:)/e346
      g346=e346/sqs346

c Boost p3, p4 and p6 back to the c.m.s. of particles 3, 4, 5 and 6:

      pv346=dot_product(v346,p3(1:))
      p0=p3(0)
      p3(0)=g346*(p0+pv346)
      p3(1:)=p3(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

      pv346=dot_product(v346,p4(1:))
      p0=p4(0)
      p4(0)=g346*(p0+pv346)
      p4(1:)=p4(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

      pv346=dot_product(v346,p6(1:))
      p0=p6(0)
      p6(0)=g346*(p0+pv346)
      p6(1:)=p6(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

c Boost p3, p4, p5 and p6 back to the c.m.s.:

      pv3456=dot_product(v3456,p3(1:))
      p0=p3(0)
      p3(0)=g3456*(p0+pv3456)
      p3(1:)=p3(1:)+g3456*(g3456*pv3456/(g3456+1.d0)+p0)*v3456

      pv3456=dot_product(v3456,p4(1:))
      p0=p4(0)
      p4(0)=g3456*(p0+pv3456)
      p4(1:)=p4(1:)+g3456*(g3456*pv3456/(g3456+1.d0)+p0)*v3456

      pv3456=dot_product(v3456,p5(1:))
      p0=p5(0)
      p5(0)=g3456*(p0+pv3456)
      p5(1:)=p5(1:)+g3456*(g3456*pv3456/(g3456+1.d0)+p0)*v3456

      pv3456=dot_product(v3456,p6(1:))
      p0=p6(0)
      p6(0)=g3456*(p0+pv3456)
      p6(1:)=p6(1:)+g3456*(g3456*pv3456/(g3456+1.d0)+p0)*v3456

c The phase space normalization including jakobian

      dlips=cps1*pp7*pp5/sqsp*de7*dct7*ds346*dct5*ds34*dlips346*dlips34

      else ! ite

      write(*,*)'ite =',ite,' is not allowed'
      stop

      end if ! ite


      else if (ibr == 2) then

      if (ite == 0) then

c s347=(p3+p4+p7)^2:

c      s347min=m34*(m34+2.d0*ecut)
      s347min=m34**2

      s347max=(ecm-m56)**2

      if (iw34 == 0) then
       ds347=s347max-s347min
       s347=ds347*x(1)+s347min
      else if (iw34 == 1) then
       ats347=atan((s347min-mv12)/gvmv1)
       ns347=gvmv1/(atan((s347max-mv12)/gvmv1)-ats347)
       s347=gvmv1*tan(gvmv1/ns347*x(1)+ats347)+mv12
       ds347=((s347-mv12)**2+gvmv1**2)/ns347
      else if (iw34 == 2) then
       rs347=s347max/s347min
       s347=s347min*rs347**x(1)
       ds347=s347*log(rs347)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs347=sqrt(s347)
      
c The length of p3+p4+p7=-p5-p6 in the c.m.s.:
c s56=(p5+p6)^2:

      s56min=m56**2
      s56max=(ecm-sqs347)**2

      if (iw56 == 0) then
       ds56=s56max-s56min
       s56=ds56*x(2)+s56min
      else if (iw56 == 1) then
       ats56=atan((s56min-mv22)/gvmv2)
       ns56=gvmv2/(atan((s56max-mv22)/gvmv2)-ats56)
       s56=gvmv2*tan(gvmv2/ns56*x(2)+ats56)+mv22
       ds56=((s56-mv22)**2+gvmv2**2)/ns56
      else if (iw56 == 2) then
       rs56=s56max/s56min
       s56=s56min*rs56**x(2)
       ds56=s56*log(rs56)
      else
       write(*,*)'iw56 =',iw56,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)

      pp347=sqrt(lamsq(ecm,sqs347,sqs56))/(2.d0*ecm)

c Necessary boost parameters:

      e347=sqrt(s347+pp347**2)
      b347=pp347/e347
      g347=e347/sqs347

      e56=sqrt(s56+pp347**2)
      b56=-pp347/e56
      g56=e56/sqs56

c The polar angle of p3+p4+p7 in the c.m.s.:

      if (itw == 0) then

       dct347=2.d0
       ct347=dct347*x(10)-1.d0

      else

       a347=(ecm*e347-s347)/(ecm*pp347)
       ob347=a347-1.d0
       if (abs(ob347) < 1.d-9) then
        ob347=2.d0*s347*s56/(s-s347-s56)**2
        rb34=ob347/(a347+1.d0)
       else
        rb347=ob347/(a347+1.d0)
       end if

       n347=-1.d0/log(rb347)

       if (itw == 1) then
        ct347=a347-(a347+1.d0)*rb347**x(10)
        dct347=(a347-ct347)/n347
       else if (itw == 2) then
        ct347=ob347/rb347**x(10)-a347
        dct347=(a347+ct347)/n347
       else
        write(*,*)'kineeff.f: itw =',itw,' IS NOT ALLOWED'
        stop
       end if

      end if

c The momentum p3 + p4 has been generated in the xOz plane.

c Other useful variables:

      if (ct347 >= 1.d0) then
       ct347=1.d0
       st347=0.d0
      else if (ct347 <= -1.d0) then
       ct347=-1.d0
       st347=0.d0
      else
       st347=sqrt(1.d0-ct347**2)
      end if

c Velocity of the c.m.s. of particles 3, 4 and 7 in the c.m.s.:

      r347=(/st347,0.d0,ct347/)
      v347=b347*r347

c Velocity of the c.m.s. of particles 5 and 6 in the c.m.s.:

      v56=b56*r347

c Generate the momenta of particles 5 and 6 in their c.m.s.:


      call kinff(sqs56,m5,m6,x(3:4),p5,p6,pp5,dlips56)

c Generate the momenta of particles 3, 4 and 7 in their c.m.s.:
c The minimum value of ecut in the c.m.s. of particle 3, 4 and 7:

      if (ict3 == 1) then

c Boost p2(0) to the c.m.s. of particles 3, 4 and 7:

c      pv347=dot_product(v347,p2(1:))
      pv347=v347(3)*p2(3)
      e2=g347*(p2(0)-pv347)

      end if ! ict3

      ecutp=ecut*sqrt((1.d0-abs(b347))/(1.d0+abs(b347)))

      call kinffg(ie3,ict3,sqs347,m3,m4,ecutp,m22,e2,x(5:9),
     &                                          p3,p4,p7,dlips347)

c Boost back to the c.m.s.:

      pv347=dot_product(v347,p7(1:))
      p0=p7(0)
      p7(0)=g347*(p0+pv347)
      p7(1:)=p7(1:)+g347*(g347*pv347/(g347+1.d0)+p0)*v347

      if (p7(0) < ecut) then
       dlips=0.d0
       return
      end if

      pv347=dot_product(v347,p3(1:))
      p0=p3(0)
      p3(0)=g347*(p0+pv347)
      p3(1:)=p3(1:)+g347*(g347*pv347/(g347+1.d0)+p0)*v347

      pv347=dot_product(v347,p4(1:))
      p0=p4(0)
      p4(0)=g347*(p0+pv347)
      p4(1:)=p4(1:)+g347*(g347*pv347/(g347+1.d0)+p0)*v347

c Boost back to the c.m.s.:

      pv56=dot_product(v56,p5(1:))
      p0=p5(0)
      p5(0)=g56*(p0+pv56)
      p5(1:)=p5(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

      pv56=dot_product(v56,p6(1:))
      p0=p6(0)
      p6(0)=g56*(p0+pv56)
      p6(1:)=p6(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

c The phase space normalization:

      dlips=cps20*pp347/ecm*ds347*ds56*dct347*dlips347*dlips56

      else if (ite > 0) then

c p5 in the c.m.s.:

      m346=m34+m6
      m52=m5**2

      de5=((ecm-m346)*(ecm+m346)+m52)/(2.d0*ecm)-m5

      p5(0)=de5*x(1)+m5
      pp5=sqrt(p5(0)**2-m52)

      da5=(p2(0)-p5(0))*(m52*p2(0)-m22*p5(0))/(2.d0*(p2(0)*p5(0))**2)

      if (da5 < 1.d-9) then
       a5=1.d0+da5
      else
       a5=(p2(0)*p5(0)-(m22+m52)/2.d0)/(pp*pp5)
       da5=a5-1.d0
      end if

      ra5=(a5+1.d0)/da5
      ct5=da5*ra5**x(2)-a5

      if (abs(ct5) < 1.d0) then
       st5=sqrt(1.d0-ct5**2)
      else if (ct5 <= -1.d0) then
       ct5=-1.d0
       st5=0.d0
      else if (ct5 >= 1.d0) then
       ct5=1.d0
       st5=0.d0
      end if

      dct5=log(ra5)*(a5+ct5)

      r5=(/st5,0.d0,ct5/)
      p5(1:)=pp5*r5

c s3467=(p3+p4+p6+p7)^2:

      s3467=ecm*(ecm-2.d0*p5(0))+m52
      sqs3467=sqrt(s3467)

c Velocity of the c.m.s. of particles 3, 4, 6 and 7 in the c.m.s.:

      e3467=sqrt(s3467+pp5**2)
      b3467=-pp5/e3467
      v3467=b3467*r5
      g3467=e3467/sqs3467

      if (ite == 1) then

c s347=(p3+p4+p7)^2:

      s347min=m34**2
      s347max=(sqs3467-m6)**2

      if (iw34 == 0) then
       ds347=s347max-s347min
       s347=ds347*x(3)+s347min
      else if (iw34 == 1) then
       ats347=atan((s347min-mv12)/gvmv1)
       ns347=gvmv1/(atan((s347max-mv12)/gvmv1)-ats347)
       s347=gvmv1*tan(gvmv1/ns347*x(3)+ats347)+mv12
       ds347=((s347-mv12)**2+gvmv1**2)/ns347
      else if (iw34 == 2) then
       rs347=s347max/s347min
       s347=s347min*rs347**x(3)
       ds347=s347*log(rs347)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs347=sqrt(s347)
      
c p3+p4+p7 and p6 in the the c.m.s. frame of particles 3, 4, 6 and 7:

      call kinff(sqs3467,sqs347,m6,x(4:5),p347,p6,pp347,dlips3467)

c Boost ecut to the c.m.s. of particles 3, 4 and 7:

      ecutp=ecut*sqrt((1.d0-abs(b3467))/(1.d0+abs(b3467)))

c Velocity of the c.m.s. of particles 3, 4 and 7 in the c.m.s. of particles
c 3, 4, 6 and 7:

      e347=sqrt(s347+pp347**2)
      b347=pp347/e347
      v347=p347(1:)/e347
      g347=e347/sqs347

      ecutp=ecutp*sqrt((1.d0-abs(b347))/(1.d0+abs(b347)))

      call kinffg(ie3,0,sqs347,m3,m4,ecutp,m22,e2,x(6:10),
     &                                          p3,p4,p7,dlips347)

c Boost p3, p4 and p7 back to the c.m.s. of particles 3, 4, 6 and 7:

      pv347=dot_product(v347,p3(1:))
      p0=p3(0)
      p3(0)=g347*(p0+pv347)
      p3(1:)=p3(1:)+g347*(g347*pv347/(g347+1.d0)+p0)*v347

      pv347=dot_product(v347,p4(1:))
      p0=p4(0)
      p4(0)=g347*(p0+pv347)
      p4(1:)=p4(1:)+g347*(g347*pv347/(g347+1.d0)+p0)*v347

      pv347=dot_product(v347,p7(1:))
      p0=p7(0)
      p7(0)=g347*(p0+pv347)
      p7(1:)=p7(1:)+g347*(g347*pv347/(g347+1.d0)+p0)*v347

c Boost p3, p4, p6 and p7 back to the c.m.s.:

      pv3467=dot_product(v3467,p7(1:))
      p0=p7(0)
      p7(0)=g3467*(p0+pv3467)
      p7(1:)=p7(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467

      if (p7(0) < ecut) then
       dlips=0.d0
       return
      end if

      pv3467=dot_product(v3467,p3(1:))
      p0=p3(0)
      p3(0)=g3467*(p0+pv3467)
      p3(1:)=p3(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467

      pv3467=dot_product(v3467,p4(1:))
      p0=p4(0)
      p4(0)=g3467*(p0+pv3467)
      p4(1:)=p4(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467

      pv3467=dot_product(v3467,p6(1:))
      p0=p6(0)
      p6(0)=g3467*(p0+pv3467)
      p6(1:)=p6(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467

c The phase space normalization:

      dlips=cps21*pp5*de5*dct5*ds347*dlips3467*dlips347

      else if (ite == 2) then

c s346=(p3+p4+p6)^2:

      s346min=m346**2
      ds346=s3467-s346min
      s346=ds346*x(3)+s346min
      sqs346=sqrt(s346)

c Boost p5 to the c.m.s. of particles 3, 4, 6 and 7:

c      pv3467=dot_product(v3467,p5(1:))
      pv3467=-pp5**2/e3467
      p0=p5(0)
      e5=g3467*(p0-pv3467)
      p5p(1:)=p5(1:)+g3467*(g3467*pv3467/(g3467+1.d0)-p0)*v3467

c Spherical components of p5p:

      pp5p=sqrt(p5p(1)**2+p5p(2)**2+p5p(3)**2)
      ct5p=p5p(3)/pp5p
      st5p=p5p(1)/pp5p

c p3+p4+p6 and p7 in the the c.m.s. frame of particles 3, 4, 6 and 7:

      db5=(m5/e5)**2
      beta5=sqrt(1.d0-db5)
      rb5=(1.d0+beta5)**2/db5
      nct7=beta5/log(rb5)
      ct7=(1.d0-(1.d0+beta5)/rb5**x(4))/beta5

      if (ct7 >= 1.d0) then
       ct7=1.d0
       dct7=db5/(2.d0*nct7)
       st7=0.d0
      else if (ct7 <= -1.d0) then
       ct7=-1.d0
       dct7=(4.d0-db5)/(2.d0*nct7)
       st7=0.d0
      else
       dct7=(1.d0-ct7+db5*ct7/2.d0)/nct7
       st7=sqrt(1.d0-ct7**2)
      end if

      phi7=dphi*x(5)

      pp7=sqrt(lamsq(sqs3467,sqs346,0.d0))/(2.d0*sqs3467)

      p7=pp7*(/1.d0,st7*cos(phi7),st7*sin(phi7),ct7/)

c Rotation of the frame by angle theta_5p in the xOz plane:

      px=p7(1)
      pz=p7(3)
      p7(1)=px*ct5p+pz*st5p
      p7(3)=-px*st5p+pz*ct5p

      dlips3467=pi/2.d0*pp7/sqs3467*dct7

c Generate s34=(p3+p4)^2

      s34min=(m34)**2
      s34max=(sqs346-m6)**2

      if (iw34 == 0) then
       ds34=s34max-s34min
       s34=ds34*x(6)+s34min
      else if (iw34 == 1) then
       ats34=atan((s34min-mv12)/gvmv1)
       ns34=gvmv1/(atan((s34max-mv12)/gvmv1)-ats34)
       s34=gvmv1*tan(gvmv1/ns34*x(6)+ats34)+mv12
       if (s34 < s34min) then
        s34=s34min
       end if
       ds34=((s34-mv12)**2+gvmv1**2)/ns34
      else if (iw34 == 2) then
       rs34=s34max/s34min
       ns34=1.d0/log(rs34)
       s34=s34min*rs34**x(6)
       ds34=s34/ns34
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

c p3+p4 and p6 in the the c.m.s. frame of particles 3, 4 and 6:

      call kinff(sqs346,sqs34,m6,x(7:8),p34,p6,pp34,dlips346)

c p3 and p4 in the the c.m.s. frame of particles 3 and 4:

      call kinff(sqs34,m3,m4,x(9:10),p3,p4,pp3,dlips34)

c Velocity of the c.m.s. of particles 3 and 4 in the c.m.s. of particles
c 3, 4 and 6:

      e34=sqrt(s34+pp34**2)
      v34=p34(1:)/e34
      g34=e34/sqs34

c Boost p3 and p4 back to the c.m.s. of particles 3, 4 and 6:

      pv34=dot_product(v34,p3(1:))
      p0=p3(0)
      p3(0)=g34*(p0+pv34)
      p3(1:)=p3(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

      pv34=dot_product(v34,p4(1:))
      p0=p4(0)
      p4(0)=g34*(p0+pv34)
      p4(1:)=p4(1:)+g34*(g34*pv34/(g34+1.d0)+p0)*v34

c Velocity of the c.m.s. of particles 3, 4 and 6 in the c.m.s. of particles
c 3, 4, 6 and 7:

      e346=sqrt(s346+pp7**2)
      v346=-p7(1:)/e346
      g346=e346/sqs346

c Boost p3, p4 and p6 back to the c.m.s.:

      pv346=dot_product(v346,p3(1:))
      p0=p3(0)
      p3(0)=g346*(p0+pv346)
      p3(1:)=p3(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

      pv346=dot_product(v346,p4(1:))
      p0=p4(0)
      p4(0)=g346*(p0+pv346)
      p4(1:)=p4(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

      pv346=dot_product(v346,p6(1:))
      p0=p6(0)
      p6(0)=g346*(p0+pv346)
      p6(1:)=p6(1:)+g346*(g346*pv346/(g346+1.d0)+p0)*v346

c Velocity of the c.m.s. of particles 3, 4, 6 and 7 in the c.m.s.:

      e3467=sqrt(s3467+pp5**2)
      v3467=-p5(1:)/e3467
      g3467=e3467/sqs3467

c Boost p3, p4, p6 and p7 back to the c.m.s.:

      pv3467=dot_product(v3467,p7(1:))
      p0=p7(0)
      p7(0)=g3467*(p0+pv3467)
      p7(1:)=p7(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467

      if (p7(0) < ecut) then
       dlips=0.d0
       return
      end if

      pv3467=dot_product(v3467,p3(1:))
      p0=p3(0)
      p3(0)=g3467*(p0+pv3467)
      p3(1:)=p3(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467

      pv3467=dot_product(v3467,p4(1:))
      p0=p4(0)
      p4(0)=g3467*(p0+pv3467)
      p4(1:)=p4(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467

      pv3467=dot_product(v3467,p6(1:))
      p0=p6(0)
      p6(0)=g3467*(p0+pv3467)
      p6(1:)=p6(1:)+g3467*(g3467*pv3467/(g3467+1.d0)+p0)*v3467


c The phase space normalization:

      dlips=cps21*pp5*de5*dct5*ds346*dlips3467*ds34*dlips346*dlips34

      else ! ite=1/ite=2

      write(*,*)'ite =',ite,' is not allowed'
      stop

      end if ! ite=1/ite=2

      end if ! ite

      else ! ibr

      write(*,*)'ibr =',ibr,' IS NOT ALLOWED'
      stop

      end if ! ibr

c Spherical components of the momenta in the c.m.s.:


      if (dlips > 0.d0) then
        ps3(0)=sqrt(p3(1)**2+p3(2)**2+p3(3)**2)
        ps3(1)=p3(3)/ps3(0)
        if (abs(ps3(1)) < 1.d0) then
           ps3(2)=sqrt(1.d0-ps3(1)**2)
           pst=ps3(0)*ps3(2)
           ps3(3)=p3(1)/pst
           ps3(4)=p3(2)/pst
        else if (ps3(1) >= 1.d0) then
           ps3(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps3(1) <= -1.d0) then
           ps3(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps4(0)=sqrt(p4(1)**2+p4(2)**2+p4(3)**2)
        ps4(1)=p4(3)/ps4(0)
        if (abs(ps4(1)) < 1.d0) then
           ps4(2)=sqrt(1.d0-ps4(1)**2)
           pst=ps4(0)*ps4(2)
           ps4(3)=p4(1)/pst
           ps4(4)=p4(2)/pst
        else if (ps4(1) >= 1.d0) then
           ps4(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps4(1) <= -1.d0) then
           ps4(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps5(0)=sqrt(p5(1)**2+p5(2)**2+p5(3)**2)
        ps5(1)=p5(3)/ps5(0)
        if (abs(ps5(1)) < 1.d0) then
           ps5(2)=sqrt(1.d0-ps5(1)**2)
           pst=ps5(0)*ps5(2)
           ps5(3)=p5(1)/pst
           ps5(4)=p5(2)/pst
        else if (ps5(1) >= 1.d0) then
           ps5(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps5(1) <= -1.d0) then
           ps5(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps6(0)=sqrt(p6(1)**2+p6(2)**2+p6(3)**2)
        ps6(1)=p6(3)/ps6(0)
        if (abs(ps6(1)) < 1.d0) then
           ps6(2)=sqrt(1.d0-ps6(1)**2)
           pst=ps6(0)*ps6(2)
           ps6(3)=p6(1)/pst
           ps6(4)=p6(2)/pst
        else if (ps6(1) >= 1.d0) then
           ps6(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps6(1) <= -1.d0) then
           ps6(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        if (ibr /= 1) then
         ps7(0)=sqrt(p7(1)**2+p7(2)**2+p7(3)**2)
         ps7(1)=p7(3)/ps7(0)
         if (abs(ps7(1)) < 1.d0) then
            ps7(2)=sqrt(1.d0-ps7(1)**2)
            pst=ps7(0)*ps7(2)
            ps7(3)=p7(1)/pst
            ps7(4)=p7(2)/pst
         else if (ps7(1) >= 1.d0) then
            ps7(1:)=(/1.d0,0.d0,1.d0,0.d0/)
         else if (ps7(1) <= -1.d0) then
            ps7(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
         end if
        end if

      end if

c      if (dlips > 0.d0) then
c       dm2=srr(p1,p1)-m1**2
c       if (dm2 > 1.d-6) then 
c         write(*,*)'p1',p1,dm2
c       end if
c       dm2=srr(p2,p2)-m2**2
c       if (dm2 > 1.d-6) then 
c         write(*,*)'p2',p2,dm2
c       end if
c       dm2=srr(p3,p3)-m3**2
c       if (dm2 > 1.d-6) then 
c         write(*,*)'p3',p3,dm2
c       end if
c       dm2=srr(p4,p4)-m4**2
c       if (dm2 > 1.d-6) then 
c         write(*,*)'p4',p4,dm2
c       end if
c       dm2=srr(p5,p5)-m5**2
c       if (dm2 > 1.d-6) then 
c         write(*,*)'p5',p5,dm2
c       end if
c       dm2=srr(p6,p6)-m6**2
c       if (dm2 > 1.d-6) then 
c         write(*,*)'p6',p6,dm2
c       end if
c       dm2=srr(p7,p7)
c       if (dm2 > 1.d-6) then 
c         write(*,*)'p7',p7,dm2
c       end if

c       de=p3(0)+p4(0)+p5(0)+p6(0)+p7(0)
c       if (de-ecm > 1.d-6) then 
c         write(*,*)'de',p3(0),p4(0),p5(0),p6(0),p7(0),de
c       end if

c       dp=p3(1:)+p4(1:)+p5(1:)+p6(1:)+p7(1:)
c       do i=1,3
c       if (dp(i) > 1.d-6) then 
c         write(*,*)'dp',i,p3(i),p4(i),p5(i),p6(i),p7(i),dp(i)
c       end if
c       end do
c      end if

      end subroutine


