      subroutine kineetbffg(ipr,iv1,iv2,ecm,ecut,mv1,gamv1,mv2,gamv2,x,
     &                      p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,p4,ps4,m4,
     &                      p5,ps5,m5,p6,ps6,m6,p7,ps7,dlips,flux)
c----------------------------------------------------------------------c
c Returns cartezian and spherical components of four momenta,
c phase space normalization, dlips, and the flux factor for 
c a bremsstrahlung to 2 --> 4 process in the c.m.s. 
c
c                   p1 + p2 --> p3 + (p4 + p5 + p6) + p7
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: ipr = 1,..,6 -specifies which particle radiates a photon,
c        iv1 = 0/1 or 2 if the intermediate boson V1 is off/in resonance
c                                                            or massless,
c        iv2 = 0/1 or 2 if the intermediate boson V2 is off/in resonance
c                                                            or massless
c        ecm - the c.m.s. energy,
c        ecut - lower cut on the energy of the photon,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        x(10) - random variables, 0 < x(i) < 1,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 13 July, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 
      use cutvalues

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: ipr,iv1,iv2

      real(dp) :: ecm,ecut,mv1,gamv1,mv2,gamv2,x(10),m1,m2,m3,m4,m5,m6,
     &           msum,s456,s56,dlips,flux

      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p56
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7
      real(dp), dimension(1:3) :: v

      real(dp) :: pp,lamsq,s456min,s456max,ds456,s56min,s56max,
     &           ds56,dlips56,gvmv1,mv12,gvmv2,mv22,
     &           dct3,ct3,st3,pp3,
     &           pp5,m32,pp56,rs56,rs456,ns56,ns456,ats56,ats456,sqs56,
     &           sqs456,e,g,dlips456,
     &           srr,sp,sqsp,phi,re7,de7,e7,nct7,dct7,ct7,st7

      real(dp) :: ecm0=0.d0,s,m12,m22,dbe2,dbe,betae,rbe,dphi,cps

      save s,m12,m22,dbe2,dbe,betae,rbe,nct7,dphi,cps,ecm0

      if (ecm /= ecm0) then

      dphi=2.d0*pi

      s=ecm**2

c Initial momenta for m1=m2:

      m12=m1**2
      m22=m12

      p1(0)=ecm/2.d0
      p2(0)=p1(0)

      dbe2=(m1/p1(0))**2
      dbe=dbe2/2.d0
      betae=1.d0-dbe

      if (icuts == 1) then
       rbe=(1.d0+ctgb)/(1.d0-ctgb)
      else
       rbe=(1.d0+betae)**2/dbe2
      end if

      nct7=betae/log(rbe)

      pp=betae*p1(0)

      p1(1:)=(/0.d0,0.d0,pp/)
      p2(1:)=(/0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(2.d0*pi)**11

c Flux factor:

      flux=4.d0*ecm*pp

      ecm0=ecm

      end if ! ecm0

      if (iv1 == 1) then
       gvmv1=gamv1*mv1
       mv12=mv1**2
      end if

      if (iv2 == 1) then
       gvmv2=gamv2*mv2
       mv22=mv2**2
      end if

      msum=m3+m4+m5+m6

      if (ipr < 3) then

c energy and angles of a photon in the c.m.s.:

      re7=(ecm-msum)*(ecm+msum)/(2.d0*ecm*ecut)

      p7(0)=ecut*re7**x(1)
      de7=p7(0)*log(re7)

      if (ipr == 1) then

       if (icuts == 1) then
        ct7=(1.d0-(1.d0+ctgb*betae)/rbe**x(2))/betae
       else
        ct7=(1.d0-(1.d0+betae)/rbe**x(2))/betae
       end if

       if (ct7 >= 1.d0) then
        ct7=1.d0
        dct7=dbe/nct7
        st7=0.d0
       else
        dct7=(1.d0-ct7+dbe*ct7)/nct7
        st7=sqrt(1.d0-ct7**2)
       end if

      else if (ipr == 2) then

       if (icuts == 1) then
        ct7=((1.d0-ctgb*betae)*rbe**x(2)-1.d0)/betae
       else
        ct7=(dbe*rbe**x(2)-1.d0)/betae
        end if
       if (ct7 <= -1.d0) then
        ct7=-1.d0
        dct7=dbe/nct7
        st7=0.d0
       else
        dct7=(1.d0+ct7-dbe*ct7)/nct7
        st7=sqrt(1.d0-ct7**2)
       end if

      end if ! ipr = 1/2

c momentum of the photon in the c.m.s.:

       p7(1:)=p7(0)*(/st7,0.d0,ct7/)
       ps7=(/p7(0),ct7,st7,1.d0,0.d0/)

c sp=(p3+p4+p5+p6)^2:

      sp=ecm*(ecm-2.d0*p7(0))
      sqsp=sqrt(sp)

c s456=(p4+p5+p6)^2:

      s456min=(m4+m5+m6)**2
      s456max=(sqsp-m3)**2

      if (iv1 == 0 .or. (iv1 == 1 .and. gamv1 == 0.d0)) then
       ds456=s456max-s456min
       s456=ds456*x(3)+s456min
      else if (iv1 == 1) then
       ats456=atan((s456min-mv12)/gvmv1)
       ns456=gvmv1/(atan((s456max-mv12)/gvmv1)-ats456)
       s456=gvmv1*tan(gvmv1/ns456*x(3)+ats456)+mv12
       if (s456 < s456min) then
        dlips=zero
        return
       end if
       ds456=((s456-mv12)**2+gvmv1**2)/ns456
      else if (iv1 == 2) then
       rs456=s456max/s456min
       ns456=1.d0/log(rs456)
       s456=s456min*rs456**x(3)
       ds456=s456/ns456
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs456=sqrt(s456)

c p3 in the c.m.s.:

      dct3=2.d0
      ct3=dct3*x(4)-1.d0
      st3=sqrt(1.d0-ct3**2)
      phi=dphi*x(5)

      m32=m3**2
      p3(0)=(sp-s456+m32)/(2.d0*sqsp)

      pp3=sqrt(p3(0)**2-m32)
      p3(1:)=pp3*(/st3*cos(phi),st3*sin(phi),ct3/)

c s56=(p5+p6)^2:

      s56min=(m5+m6)**2
      s56max=(sqs456-m4)**2

      if (iv2 == 0 .or. (iv2 == 1 .and. gamv2 == 0.d0)) then
       ds56=s56max-s56min
       s56=ds56*x(6)+s56min
      else if (iv2 == 1) then
       ats56=atan((s56min-mv22)/gvmv2)
       ns56=gvmv2/(atan((s56max-mv22)/gvmv2)-ats56)
       s56=gvmv2*tan(gvmv2/ns56*x(6)+ats56)+mv22
       if (s56 < s56min) then
        dlips=zero
        return
       end if
       ds56=((s56-mv22)**2+gvmv2**2)/ns56
      else if (iv2 == 2) then
       rs56=s56max/s56min
       ns56=1.d0/log(rs56)
       s56=s56min*rs56**x(6)
       ds56=s56/ns56
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)


c p4 and p56=p5+p6 in the c.m.s. of 4, 5 and 6

      call kinff(sqs456,sqs56,m4,x(7:8),p56,p4,pp56,dlips456)

c p5 and p6 in the c.m.s. of 5 and 6

      call kinff(sqs56,m5,m6,x(9:10),p5,p6,pp5,dlips56)

c The phase space normalization:

      dlips=cps*(pi*p7(0)*de7*dct7)*(pi*pp3/sqsp)*ds456*ds56
     &     *dlips456*dlips56

      if (dlips > zero) then

c Velocity of the c.m.s. of particles 5 and 6 in the c.m.s. of particles
c 4, 5 and 6:

c Boost p5 and p6 back to the c.m.s. of particles 4, 5 and 6:

      e=sqrt(s56+pp56**2)
      v=p56(1:)/e
      g=e/sqs56

      call boost(g,v,p5)
      call boost(g,v,p6)

c Boost p4, p5 and p6 back to the c.m.s. of particles 3, 4, 5 and 6:

      e=sqrt(s456+pp3**2)
      v=-p3(1:)/e
      g=e/sqs456

      call boost(g,v,p4)
      call boost(g,v,p5)
      call boost(g,v,p6)

c Boost p3, p4, p5 and p6 back to the c.m.s.:

      e=ecm-p7(0)
      v=-p7(1:)/e
      g=e/sqsp

      call boost(g,v,p3)
      call boost(g,v,p4)
      call boost(g,v,p5)
      call boost(g,v,p6)

      end if !ipr
      end if ! dlips

      if (dlips > zero) then

c Spherical components of the momenta p3,...,p6:

      call spheric(p3,ps3)
      call spheric(p4,ps4)
      call spheric(p5,ps5)
      call spheric(p6,ps6)

ckk>
c      write(*,*)'p1',sqrt(abs(srr(p1,p1))),m1
c      write(*,*)'p2',sqrt(abs(srr(p2,p2))),m2
c      write(*,*)'p3',sqrt(abs(srr(p3,p3))),m3
c      write(*,*)'p4',sqrt(abs(srr(p4,p4))),m4
c      write(*,*)'p5',sqrt(abs(srr(p5,p5))),m5
c      write(*,*)'p6',sqrt(abs(srr(p6,p6))),m6
c      write(*,*)'p7',sqrt(abs(srr(p7,p7)))
ckk<


      end if ! dlips

      end


