      subroutine kinff(ecm,m3,m4,x,p3,p4,pp,dlips)
c----------------------------------------------------------------------c
c Returns cartezian components of the final state four momenta and
c the phase space normalization for a 2 --> 2 process in the c.m.s. 
c The factor (2*pi)^-2 is not included in dlips.
c
c Input: ecm - the c.m.s. energy,
c        m3,m4 - particle masses,
c        x(i) - random variables, 0 < x(i) < 1, i=1,2,
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 24 February, 1999; modified 5 July, 2001
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      real(dp) ecm,m3,m4,x(2),p3(0:3),p4(0:3),pp,dlips

      real(dp), parameter :: pi=3.141592653589793238d0, zero=0.d0

c Local variables

      integer :: ifirst=1

      real(dp) :: dphi,dct,cps,lamsq,ct,st,phi

      save ifirst,dphi,dct,cps

      if (ifirst == 1 ) then
       ifirst=0
       dphi=2.d0*pi
       dct=2.d0
       cps=dct*dphi/4.d0
      end if

      if (ecm < m3+m4) then
       dlips=zero
      else

       pp=sqrt(lamsq(ecm,m3,m4))/(2.d0*ecm)

       if (pp < zero) then
        dlips=zero
        return
       end if

       ct=dct*x(1)-1.d0
       st=sqrt(1.d0-ct**2)
       phi=dphi*x(2)

       p3=(/sqrt(m3**2+pp**2),pp*st*cos(phi),pp*st*sin(phi),pp*ct/)
       p4=(/sqrt(m4**2+pp**2),-p3(1),-p3(2),-p3(3)/)

       dlips=cps*pp/ecm

      end if

      end


