      subroutine kinffg(ie3,ict3,ecm,m3,m4,ecut,m22,e2,x,p3,p4,p7,dlips)
c----------------------------------------------------------------------c
c Returns the final state four momentum p3 and the final state momenta 
c p4,p7 and the phase space c normalization for a 2 --> 3 process in 
c the c.m.s. with a photon in the final state. 
c The factor (2*pi)^-5 is not included in dlips.
c
c Input: ict3 = 0/1 if 1/t pole related to the eeg-vertex is absent/present
c               in the 3 body decay channel,
c        ie3 = 0 if energy of particle 3 is uniformly generated,
c              1 if it is generated according to 1/(p4.p7),
c        ecm - the c.m.s. energy,
c        m3,m4 - particle masses,
c        ecut - lower cut on the energy of the photon,
c        m22 - the squared mass of particle 2 (relevant only if ict3=1),
c        e2 - the energy of particle 2 (relevant only if ict3=1),
c        x(i) - random variables, 0 < x(i) < 1, i=1,...,5
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 22 April, 1999; modified on 1 June, 2000
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      real(dp) ecm,e7min,m3,m4,ecut,m22,e2,x(5),p3(0:3),p4(0:3),p7(0:3),
     &        dlips

c Local variables

      integer :: ifirst=1,ie3,ict3

      real(dp) s,e7max,re7,ne7,de7,ecm3,e3max,de3,ce3,dele3,re3,pp32,
     &        pp3,
     &        pp3max,e7num,lamsq,ct37,st37,ct3,phi3,phi7,phi37,st3,
     &        cf3,sf3,e4max,px,py,pz

      real(dp) m32,m42,dm34,twopi,dphi3,dphi37,ct3x,dct3,da3,a3,ra3,cps

      save ifirst,dphi3,dphi37,cps

      if (ifirst == 1 ) then
       ifirst=0
       twopi=8.d0*atan(1.d0)
       dphi3=twopi
       dphi37=twopi
       cps=dphi3*dphi37/8.d0
      end if

      s=ecm**2
      m32=m3**2
      m42=m4**2
      dm34=m32-m42

c The energy of particle 3:

      if (ie3 == 0) then
       pp3max=sqrt(lamsq(ecm,m3,m4))/(2.d0*ecm)
       e3max=sqrt(m32+pp3max**2)
       de3=e3max-m3
       p3(0)=de3*x(1)+m3
      else if (ie3 == 1) then
       e4max=sqrt(m42+lamsq(ecm,m3,m4)/(2.d0*ecm)**2)
       ce3=(ecm+dm34/ecm)/2.d0
c       re3=(ce3-e3max)/(ce3-m3)
       re3=m42/(2.d0*ecm)*(ecut/e4max)/(ce3-m3)
       dele3=(ce3-m3)*re3**x(1)
       p3(0)=ce3-dele3
c       de3=-(ce3-p3(0))*log(re3)
       de3=-dele3*log(re3)
      else if (ie3 == 2) then
       e4max=sqrt(m42+lamsq(ecm,m3,m4)/(2.d0*ecm)**2)
       e3max=sqrt(m32
     &      +lamsq(ecm,m3,m4*sqrt(1.d0+ecut/e4max))/(2.d0*ecm)**2)
       ce3=(ecm+dm34/ecm)/2.d0
       re3=(e3max-m3)/(m42/(2.d0*ecm)*(ecut/e4max))
       p3(0)=(ce3*re3*x(1)+m3)/(1.d0+re3*x(1))
       de3=(ce3-p3(0))**2/((ce3-m3)/re3)
      else
       write(*,*)'ie3 =',ie3,' IS NOT ALLOWED'
       stop
      end if

      if (p3(0) < m3) then
       dlips=0.d0
       return
      end if

      pp32=p3(0)**2-m32
      pp3=sqrt(pp32)

c The photon energy:

      ecm3=ecm-p3(0)
      e7num=ecm3**2-pp32-m42

c      e7min=(ecm*(ecm3-p3(0))+dm34)/(2.d0*(ecm3+pp3))

      e7min=max(ecut,e7num/(2.d0*(ecm3+pp3)))

      e7max=e7num/(2.d0*(ecm3-pp3))

      if(e7max <= e7min) then
       dlips=0.d0
       return
      end if

      re7=e7max/e7min

      ne7=1.d0/log(re7)
      p7(0)=e7min*re7**x(2)
      de7=p7(0)/ne7

c      de7=e7max-e7min
c      p7(0)=de7*x(2)+e7min
c       ne7=e7min*e7max/(e7max-e7min)
c       p7(0)=ne7*e7min/(ne7-e7min*x(2))
c       de7=p7(0)**2/ne7

c energy of particle 4:

       p4(0)=ecm3-p7(0)
     
c cos of the angle between p3 and p7:

      if (pp3 == 0.d0) then
       ct37=1.d0
      else
       ct37=(p4(0)**2-pp32-p7(0)**2-m42)/(2.d0*pp3*p7(0))
      end if

c Angles of particle 3:

      if (ict3 == 0) then

      dct3=2.d0
      ct3=dct3*x(3)-1.d0

      else if (ict3 == 1) then

c      da3=m22*(e2-p3(0))**2/(2.d0*(e2*p3(0))**2)
c      da3=m22*(e2-p4(0))**2/(2.d0*(e2*p4(0))**2)
      da3=m22/(e2*p3(0))

      a3=1.d0+da3

      ra3=(a3+1.d0)/da3
      ct3x=(a3+1.d0)/ra3**x(3)
      ct3=a3-ct3x
      dct3=log(ra3)*ct3x

      else ! ict3

      write(*,*)'ict3 =',ict3,' is not allowed'
      stop

      end if ! ict3

c Phase space normalization:

      if (abs(ct37)  > 1.d0) then
       dlips=0.d0
       return
      else
       dlips=cps*de7*de3*dct3
      end if

      if (abs(ct3) < 1.d0) then
       st3=sqrt(1.d0-ct3**2)
      else if (ct3 < -1.d0) then
       ct3=-1.d0
       st3=0.d0
      else if (ct3 > 1.d0) then
       ct3=1.d0
       st3=0.d0
      end if

      phi3=dphi3*x(4)
      cf3=cos(phi3)
      sf3=sin(phi3)  

c The azimuthal angle of particle 7 with respect to particle 3, phi7-phi3:

      phi37=dphi37*x(5)

c Momenta of particles 3, 7 and 4 in the frame where p3(3)=Oz:

      p3(1:)=(/0.d0,0.d0,pp3/)
      st37=sqrt(1.d0-ct37**2)
      p7(1:)=p7(0)*(/st37*cos(phi37),st37*sin(phi37),ct37/)

c Rotation of the frame by angle theta_3 in the xOz plane and by angle
c -phi_3 in the xOy plane:

       p3(1)=pp3*st3*cf3
       p3(2)=pp3*st3*sf3
       p3(3)=pp3*ct3

       px=p7(1)
       py=p7(2)
       pz=p7(3)
       p7(1)=(px*ct3+pz*st3)*cf3-py*sf3
       p7(2)=(px*ct3+pz*st3)*sf3+py*cf3
       p7(3)=-px*st3+pz*ct3

      p4(1:)=-(p3(1:)+p7(1:))

      end


