      subroutine mateeee(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &                   p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &                   p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,mat)
c----------------------------------------------------------------------c
c Returns the squared matrix element of the t-channel diagrams of
c
c      e+(p1) + e-(p2) --> e-(p3) + e+(p4) + e-(p5) + e+(p6)
c
c for the neutral current processes not containing virtual gluons 
c
c Input: pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 27 September, 2000;  modified on 1 November, 2001
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use genpv12

      implicit none
      integer, parameter :: dp=kind(1.d0)

      real(dp) t14,t52,srr
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      integer i1,i2,i3,i4,i5,i6,i

      complex(dp), dimension(2,2,n3,n4,n5,n6) :: mat

      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,xp5,xp6,
     &                              yp1,yp2,yp3,yp4,yp5,yp6

      complex(dp), dimension(4,2) :: u1
      complex(dp), dimension(4,2) :: u2
      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(0:3,n1,2) :: ea14,ez14
      complex(dp), dimension(2,2,n1,2) :: xa14,ya14,xz14,yz14
      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,
     &                                    ya36,yz36
      complex(dp), dimension(0:3,n5,2) :: ea52,ez52
      complex(dp), dimension(2,2,n5,2) :: xa52,ya52,xz52,yz52

      complex(dp), dimension(n3,n5,2,2,n4,n6) :: amp23
      complex(dp), dimension(n3,2,n4,n5,2,n6) :: amp24
      complex(dp), dimension(n5,n3,n6,2,n4,2) :: amp25
      complex(dp), dimension(n5,2,n4,n3,n6,2) :: amp30
      complex(dp), dimension(2,n3,n6,n5,2,n4) :: amp31
      complex(dp), dimension(2,n5,2,n3,n6,n4) :: amp33


c Building blocks of the diagrams:

      call fvf(u3,n3,p3,m3,ga3m,ga3p,zero,ma2,u6,n6,p6,m6,
     &                                                 ea36,xa36,ya36)
      call fvf(u3,n3,p3,m3,gz3m,gz3p,zero,mz2,u6,n6,p6,m6,
     &                                                 ez36,xz36,yz36)

       t14=-2.d0*ps1(0)*ps4(0)*(1.d0-ps4(1)
     &    +(m2*(p1(0)-p4(0))/(p1(0)*p4(0)))**2/2.d0)
       call fvf(u1,2,-p1,m1,ga1m,ga1p,t14,ma2,u4,n4,p4,m4,
     &                                                 ea14,xa14,ya14)
      call fvf(u1,2,-p1,m1,gz1m,gz1p,t14,mz2,u4,n4,p4,m4,
     &                                                 ez14,xz14,yz14)
       t52=-2.d0*ps2(0)*ps5(0)*(1.d0+ps5(1)
     &    +(m2*(p2(0)-p5(0))/(p2(0)*p5(0)))**2/2.d0)
       call fvf(u5,n5,p5,m5,ga5m,ga5p,t52,ma2,u2,2,-p2,m2,
     &                                                 ea52,xa52,ya52)
       call fvf(u5,n5,p5,m5,gz5m,gz5p,t52,mz2,u2,2,-p2,m2,
     &                                                 ez52,xz52,yz52)


      call fefef(u3,p3,
     &           ga3p*xa52+gz3p*xz52,ga3m*ya52+gz3m*yz52,onec,onec,
     &           xp1-xp4-xp6,yp1-yp4-yp6,cm3,ocom,
     &           ga6p*xa14+gz6p*xz14,ga6m*ya14+gz6m*yz14,onec,onec,
     &           u6,p6,amp23,n3,n5*2,2*n4,n6)

      call fefef(u3,p3,
     &           ga3p*xa14+gz3p*xz14,ga3m*ya14+gz3m*yz14,onec,onec,
     &           xp2-xp5-xp6,yp2-yp5-yp6,cm3,ocom,
     &           ga6p*xa52+gz6p*xz52,ga6m*ya52+gz6m*yz52,onec,onec,
     &           u6,p6,amp24,n3,2*n4,n5*2,n6)

      call fefef(u5,p5,
     &           ga5p*xa36+gz5p*xz36,ga5m*ya36+gz5m*yz36,onec,onec,
     &           xp3+xp5+xp6,yp3+yp5+yp6,cm5,ocom,
     &           ga2p*xa14+gz2p*xz14,ga2m*ya14+gz2m*yz14,onec,onec,
     &                                u2,-p2,amp25,n5,n3*n6,2*n4,2)

      call fefef(u5,p5,ga5p*xa14+gz5p*xz14,
     &                                ga5m*ya14+gz5m*yz14,onec,onec,
     &           xp2-xp3-xp6,yp2-yp3-yp6,cm5,ocom,
     &           ga2p*xa36+gz2p*xz36,ga2m*ya36+gz2m*yz36,onec,onec,
     &           u2,-p2,amp30,n5,2*n4,n3*n6,2)

      call fefef(u1,-p1,
     &           ga1p*xa36+gz1p*xz36,ga1m*ya36+gz1m*yz36,onec,onec,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &           ga4p*xa52+gz4p*xz52,ga4m*ya52+gz4m*yz52,onec,onec,
     &           u4,p4,amp31,2,n3*n6,n5*2,n4)

      call fefef(u1,-p1,ga1p*xa52+gz1p*xz52,
     &                                ga1m*ya52+gz1m*yz52,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm1,ocom,
     &           ga4p*xa36+gz4p*xz36,ga4m*ya36+gz4m*yz36,onec,onec,
     &           u4,p4,amp33,2,n5*2,n3*n6,n4)


c The spin averaged squared matrix element:

      do i1=1,2
      do i2=1,2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

c A sum of the diagrams:

      mat(i1,i2,i3,i4,i5,i6)=
     &   +amp23(i3,i5,i2,i1,i4,i6)
     &   +amp24(i3,i1,i4,i5,i2,i6)
     &   +amp25(i5,i3,i6,i1,i4,i2)
     &   +amp30(i5,i1,i4,i3,i6,i2)
     &   +amp31(i1,i3,i6,i5,i2,i4)
     &   +amp33(i1,i5,i2,i3,i6,i4)

      end do
      end do
      end do
      end do
      end do
      end do

      end 

