      subroutine matllll(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &                   p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &                   p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,mat)
c----------------------------------------------------------------------c
c Returns the matrix element of the s-channel diagrams of
c
c      e+(p1) + e-(p2) --> l(p3) + l~(p4) + l(p5) + l~(p6)
c
c for the neutral current processes containing identical leptons
c
c Input: pi(0:3), i=3,...,6 - external particle momenta,
c        psi(0:4) - spherical components of the external particle momenta,
c        xpi,ypi - spinorial components of the external particle momenta,
c        ui - spinors of the external particls.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 27 September, 2000;  modified on 1 November, 2001
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use genpv12

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer i1,i2,i3,i4,i5,i6,i
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      complex(dp) :: scc
      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,xp5,xp6,
     &                              yp1,yp2,yp3,yp4,yp5,yp6

      complex(dp), dimension(2,2,n3,n4,n5,n6) :: mat

      complex(dp), dimension(4,2) :: u1
      complex(dp), dimension(4,2) :: u2
      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(0:3,n3,n6) :: ea36,ez36
      complex(dp), dimension(2,2,n3,n6) :: xa36,xz36,ya36,yz36
      complex(dp), dimension(0:3,n5,n4) :: ea54,ez54
      complex(dp), dimension(2,2,n5,n4) :: xa54,xz54,ya54,yz54
      complex(dp), dimension(n3,n6) :: h36
      complex(dp), dimension(n5,n4) :: h54

      complex(dp), dimension(2,n5,n4,n3,n6,2) :: amp7
      complex(dp), dimension(2,n3,n6,n5,n4,2) :: amp8
      complex(dp), dimension(n3,2,2,n5,n4,n6) :: amp9
      complex(dp), dimension(n3,n5,n4,2,2,n6) :: amp10
      complex(dp), dimension(n5,2,2,n3,n6,n4) :: amp11
      complex(dp), dimension(n5,n3,n6,2,2,n4) :: amp12


c Building blocks of the diagrams:

      if (ga3m == ocom) then
        ea36=ocom
        xa36=ocom
        ya36=ocom
      else
        call fvf(u3,n3,p3,m3,ga3m,ga3p,zero,ma2,u6,n6,p6,m6,
     &                                                 ea36,xa36,ya36)
      end if ! ga3m

      call fvf(u3,n3,p3,m3,gz3m,gz3p,zero,mz2,u6,n6,p6,m6,
     &                                                 ez36,xz36,yz36)

      if (ga5m /= ocom) then
       call fvf(u5,n5,p5,m5,ga5m,ga5p,zero,ma2,u4,n4,p4,m4,
     &                                                 ea54,xa54,ya54)
      else
        ea54=ocom
        xa54=ocom
        ya54=ocom
      end if
      call fvf(u5,n5,p5,m5,gz5m,gz5p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez54,xz54,yz54)

      call fefef(u1,-p1,ga1p*xa54+gz1p*xz54,
     &                                ga1m*ya54+gz1m*yz54,onec,onec,
     &          -xp1+xp5+xp4,-yp1+yp5+yp4,cm1,ocom,
     &           ga2p*xa36+gz2p*xz36,ga2m*ya36+gz2m*yz36,onec,onec,
     &                                u2,-p2,amp7,2,n5*n4,n3*n6,2)

      call fefef(u1,-p1,ga1p*xa36+gz1p*xz36,
     &                                ga1m*ya36+gz1m*yz36,onec,onec,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &           ga2p*xa54+gz2p*xz54,ga2m*ya54+gz2m*yz54,onec,onec,
     &                                u2,-p2,amp8,2,n3*n6,n5*n4,2)

      if (ihiggs == 1) then

      call fhf(u5,n5,p5,hff(5),zero,mh2,u4,n4,p4,h54)
      call fhf(u3,n3,p3,hff(3),zero,mh2,u6,n6,p6,h36)

      call fefefh(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,onec,onec,
     &                 u6,p6,hff(6)*h54,2,n54,amp9,n3,n12,n54,n6)

      call fefefh(u3,p3,ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,
     &            onec,onec,xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                 u6,p6,hff(3)*h54,1,n54,amp10,n3,n54,n12,n6)

      call fefefh(u5,p5,ga5p*xa12+gz5p*xz12,
     &                                ga5m*ya12+gz5m*yz12,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm5,ocom,
     &           ga4p*xa36+gz4p*xz36,ga4m*ya36+gz4m*yz36,onec,onec,
     &                 u4,p4,hff(4)*h36,2,n36,amp11,n5,n12,n36,n4)

      call fefefh(u5,p5,ga5p*xa36+gz5p*xz36,
     &                                ga5m*ya36+gz5m*yz36,onec,onec,
     &           xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &           ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                 u4,p4,hff(5)*h36,1,n36,amp12,n5,n36,n12,n4)

      else ! ihiggs

      call fefef(u3,p3,ga3p*xa12+gz3p*xz12,
     &                                ga3m*ya12+gz3m*yz12,onec,onec,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,cm3,ocom,
     &           ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,onec,onec,
     &                                u6,p6,amp9,n3,4,n5*n4,n6)

      call fefef(u3,p3,ga3p*xa54+gz3p*xz54,ga3m*ya54+gz3m*yz54,
     &           onec,onec,xp3+xp5+xp4,yp3+yp5+yp4,cm3,ocom,
     &           ga6p*xa12+gz6p*xz12,ga6m*ya12+gz6m*yz12,onec,onec,
     &                                u6,p6,amp10,n3,n5*n4,4,n6)

      call fefef(u5,p5,ga5p*xa12+gz5p*xz12,
     &                                ga5m*ya12+gz5m*yz12,onec,onec,
     &          -xp1-xp2+xp5,-yp1-yp2+yp5,cm5,ocom,
     &           ga4p*xa36+gz4p*xz36,ga4m*ya36+gz4m*yz36,onec,onec,
     &                                u4,p4,amp11,n5,4,n3*n6,n4)

      call fefef(u5,p5,ga5p*xa36+gz5p*xz36,
     &                                ga5m*ya36+gz5m*yz36,onec,onec,
     &           xp5+xp3+xp6,yp5+yp3+yp6,cm5,ocom,
     &           ga4p*xa12+gz4p*xz12,ga4m*ya12+gz4m*yz12,onec,onec,
     &                                u4,p4,amp12,n5,n3*n6,4,n4)

      end if ! ihiggs

c A sum of the diagrams:

      do i1=1,2
      do i2=1,2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

      mat(i1,i2,i3,i4,i5,i6)=
     &    -amp7(i1,i5,i4,i3,i6,i2)
     &    -amp8(i1,i3,i6,i5,i4,i2)
     &    -amp9(i3,i1,i2,i5,i4,i6)
     &    -amp10(i3,i5,i4,i1,i2,i6)
     &    -amp11(i5,i1,i2,i3,i6,i4)
     &    -amp12(i5,i3,i6,i1,i2,i4)

      if (ihiggs == 1) then
       mat(i1,i2,i3,i4,i5,i6)=mat(i1,i2,i3,i4,i5,i6)
     &          -hzz*(h54(i5,i4)*scc(ez12(:,i1,i2),ez36(:,i3,i6))
     &          +h36(i3,i6)*scc(ez12(:,i1,i2),ez54(:,i5,i4)))
      end if


      end do
      end do
      end do
      end do
      end do
      end do

      end 

