      subroutine matnnnn(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &                   p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &                   p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,mat)
c----------------------------------------------------------------------c
c Returns the squared matrix element of the t-channel diagrams of
c
c      e+(p1) + e-(p2) --> ve(p3) + ve~(p4) + ve(p5) + ve~(p6)
c
c for the neutral current processes not containing virtual gluons 
c
c Input: pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 21 March, 2001; modified 24 April, 2001; 1 November, 2001
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use genpv12

      implicit none
      integer, parameter :: dp=kind(1.d0)

      real(dp) t14,t52,srr
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      integer i1,i2,i3,i4,i5,i6,i

      complex(dp), dimension(2,2,n3,n4,n5,n6) :: mat

      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,xp5,xp6,
     &                              yp1,yp2,yp3,yp4,yp5,yp6

      complex(dp), dimension(4,2) :: u1
      complex(dp), dimension(4,2) :: u2
      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(0:3,n1,n6) :: ew16
      complex(dp), dimension(2,2,n1,n6) :: xw16,yw16
      complex(dp), dimension(0:3,n5,n2) :: ew52
      complex(dp), dimension(2,2,n5,n2) :: xw52,yw52
      complex(dp), dimension(0:3,n3,n2) :: ew32
      complex(dp), dimension(2,2,n3,n2) :: xw32,yw32
      complex(dp), dimension(0:3,n5,n4) :: ez54
      complex(dp), dimension(2,2,n5,n4) :: xz54,yz54

      complex(dp), dimension(n3,n1,n6,n5,n2,n4) :: amp20
      complex(dp), dimension(n1,n6,n3,n2,n5,n4) :: amp26
      complex(dp), dimension(n1,n5,n4,n3,n2,n6) :: amp27
      complex(dp), dimension(n1,n3,n2,n5,n4,n6) :: amp28
      complex(dp), dimension(n3,n1,n6,n5,n4,n2) :: amp32
      complex(dp), dimension(n3,n5,n4,n1,n6,n2) :: amp34


c Building blocks of the diagrams:

       call fvf(u1,n1,-p1,m1,wff,ocom,zero,mw2,u6,n6,p6,m6,
     &                                                 ew16,xw16,yw16)

       call fvf(u3,n3,p3,m3,wff,ocom,zero,mw2,u2,n2,-p2,m2,
     &                                                 ew32,xw32,yw32)

      call fvf(u5,n5,p5,m5,wff,ocom,zero,mw2,u2,n2,-p2,m2,
     &                                                 ew52,xw52,yw52)

       call fvf(u5,n5,p5,m5,gz5m,gz5p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez54,xz54,yz54)

c Amplitudes:

      call fefef(u3,p3,xw16,yw16,wff,ocom,
     &           xp3+xp6-xp1,yp3+yp6-yp1,cm4,ocom,
     &           xw52,yw52,wff,ocom,u4,p4,amp20,n3,n1*n6,n5*n2,n4)

      call eee(-p1+p6,ew16,p2-p3,ew32,-p4-p5,
     &                       zww*ez54,amp26,n1*n6,n3*n2,n5*n4)

      call fefef(u1,-p1,xz54,yz54,gz1m,gz1p,
     &          -xp1+xp4+xp5,-yp1+yp4+yp5,cm1,ocom,
     &           xw32,yw32,wff,ocom,u6,p6,amp27,n1,n5*n4,n3*n2,n6)

      call fefef(u1,-p1,xw32,yw32,wff,ocom,
     &          -xp4-xp5-xp6,-yp4-yp5-yp6,ocom,ocom,
     &           xz54,yz54,gz6m,gz6p,u6,p6,amp28,n1,n3*n2,n5*n4,n6)

      call fefef(u3,p3,xw16,yw16,wff,ocom,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm2,ocom,
     &           xz54,yz54,gz2m,gz2p,u2,-p2,amp32,n3,n1*n6,n5*n4,n2)

      call fefef(u3,p3,xz54,yz54,gz3m,gz3p,
     &           xp3+xp4+xp5,yp3+yp4+yp5,ocom,ocom,
     &           xw16,yw16,wff,ocom,u2,-p2,amp34,n3,n5*n4,n1*n6,n2)


c The spin averaged squared matrix element:

      do i1=1,2
      do i2=1,2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6

c A sum of the diagrams:

      mat(i1,i2,i3,i4,i5,i6)=
     &     -amp20(i3,i1,i6,i5,i2,i4)
     &     +amp26(i1,i6,i3,i2,i5,i4)
     &     +amp27(i1,i5,i4,i3,i2,i6)
     &     +amp28(i1,i3,i2,i5,i4,i6)
     &     +amp32(i3,i1,i6,i5,i4,i2)
     &     +amp34(i3,i5,i4,i1,i6,i2)

      end do
      end do
      end do
      end do
      end do
      end do

      end 

