      subroutine matnnnng(p1,ps1,xp1,yp1,u1,p2,ps2,xp2,yp2,u2,
     &              p3,ps3,xp3,yp3,u3,p4,ps4,xp4,yp4,u4,
     &              p5,ps5,xp5,yp5,u5,p6,ps6,xp6,yp6,u6,
     &              p7,ps7,xp7,yp7,e7,xe7,ye7,matg)
c----------------------------------------------------------------------c
c Returns the matrix element of the t-channel diagrams of
c
c   e+(p1) + e-(p2) --> e-(p3) + e+(p4) + e-(p5) + e+(p6) + gamma(p7)
c
c for the neutral current processes not containing virtual gluons
c
c Input: pi, i=1,...,7 - external particle momenta,
c        psi, i=1,...,7 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 21 March, 2001; modified on 2 November, 2001
c----------------------------------------------------------------------c

      use mathprms 
      use parsm
      use helproc
      use parproc
      use genpv12
      use genpv12g

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer i1,i2,i3,i4,i5,i6,i7,i

      real(dp), dimension(0:3) ::  p1,p2,p3,p4,p5,p6,p7
      real(dp), dimension(0:4) ::  ps1,ps2,ps3,ps4,ps5,ps6,ps7
      real(dp) srr,e7(0:3,2)

      complex(dp) d37,d47,d57,d67

      complex(dp), dimension(2,2) :: xp1,xp2,xp3,xp4,xp5,xp6,xp7,
     &                              yp1,yp2,yp3,yp4,yp5,yp6,yp7

      complex(dp), dimension(2,2,n3,n4,n5,n6,2) :: matg

      complex(dp), dimension(4,2) :: u1
      complex(dp), dimension(4,2) :: u2
      complex(dp), dimension(4,n3) :: u3
      complex(dp), dimension(4,n4) :: u4
      complex(dp), dimension(4,n5) :: u5
      complex(dp), dimension(4,n6) :: u6

      complex(dp), dimension(2,2,2) :: xe7,ye7

      complex(dp), dimension(0:3,n3,n2) :: ew32
      complex(dp), dimension(0:3,n3,n2,n7) :: ew32g
      complex(dp), dimension(2,2,n3,n2) :: xw32,yw32
      complex(dp), dimension(2,2,n3,n2,n7) :: xw32g,yw32g
      complex(dp), dimension(0:3,n5,n4) :: ez54
      complex(dp), dimension(0:3,n5,n4,n7) :: ez54g
      complex(dp), dimension(2,2,n5,n4) :: xz54,
     &                                    yz54
      complex(dp), dimension(2,2,n5,n4,n7) :: xz54g,
     &                                       yz54g
      complex(dp), dimension(0:3,n1,n6) :: ew16
      complex(dp), dimension(2,2,n1,n6) :: xw16,yw16
      complex(dp), dimension(0:3,n1,n6,n7) :: ew16g
      complex(dp), dimension(2,2,n1,n6,n7) :: xw16g,yw16g
      complex(dp), dimension(0:3,n5,n2) :: ew52
      complex(dp), dimension(2,2,n5,n2) :: xw52,yw52
      complex(dp), dimension(0:3,n5,n2,n7) :: ew52g
      complex(dp), dimension(2,2,n5,n2,n7) :: xw52g,yw52g
      complex(dp), dimension(2,2,n7) :: h00g

      complex(dp) :: mat

      complex(dp), dimension(n3,n1,n6,n7,n5,n2,n4) :: m20_2
      complex(dp), dimension(n3,n1,n6,n7,n5,n2,n4) :: m20_3
      complex(dp), dimension(n3,n1,n6,n5,n2,n7,n4) :: m20_4
      complex(dp), dimension(n1,n6,n7,n3,n2,n5,n4) :: m26_1
      complex(dp), dimension(n1,n6,n3,n2,n7,n5,n4) :: m26_2
      complex(dp), dimension(n1,n6,n3,n2,n5,n4,n7) :: m26_3
      complex(dp), dimension(n3,n2,n1,n6,n5,n4,n7) :: m26_4
      complex(dp), dimension(n1,n7,n5,n4,n3,n2,n6) :: m27_1
      complex(dp), dimension(n1,n5,n4,n7,n3,n2,n6) :: m27_2
      complex(dp), dimension(n1,n5,n4,n7,n3,n2,n6) :: m27_3
      complex(dp), dimension(n1,n5,n4,n3,n2,n7,n6) :: m27_4
      complex(dp), dimension(n1,n7,n3,n2,n5,n4,n6) :: m28_1
      complex(dp), dimension(n1,n3,n2,n7,n5,n4,n6) :: m28_2
      complex(dp), dimension(n1,n3,n2,n5,n4,n7,n6) :: m28_3
      complex(dp), dimension(n3,n1,n6,n7,n5,n4,n2) :: m32_1
      complex(dp), dimension(n3,n1,n6,n5,n4,n7,n2) :: m32_2
      complex(dp), dimension(n3,n1,n6,n7,n5,n4,n2) :: m32_3
      complex(dp), dimension(n3,n1,n6,n5,n4,n7,n2) :: m32_4
      complex(dp), dimension(n3,n5,n4,n1,n6,n7,n2) :: m34_1
      complex(dp), dimension(n3,n5,n4,n7,n1,n6,n2) :: m34_2
      complex(dp), dimension(n3,n5,n4,n1,n6,n7,n2) :: m34_3


c Fermion denominators:

      d37=cmplx(srr(p3+p7,p3+p7),zero,dp)-cm3**2
      d47=cmplx(srr(p4+p7,p4+p7),zero,dp)-cm4**2
      d57=cmplx(srr(p5+p7,p5+p7),zero,dp)-cm5**2
      d67=cmplx(srr(p6+p7,p6+p7),zero,dp)-cm6**2

      call fvf(u1,n1,-p1,m1,wff,ocom,zero,mw2,u6,n6,p6,m6,
     &                                                 ew16,xw16,yw16)
      call ffvgnew(u1,n1,-p1,-xp1,-yp1,cm1,ga1m,d17,
     &           u6,n6,p6,xp6,yp6,cm6,ga6m,d67,wff,ocom,mw2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,-gww,ew16,ew16g,xw16g,yw16g,
     &           0,hff(6),mh2,h00g)
      call fvf(u3,n3,p3,m3,wff,ocom,zero,mw2,u2,n2,-p2,m2,
     &                                                 ew32,xw32,yw32)
      call ffvgnew(u3,n3,p3,xp3,yp3,cm3,ga3m,d37,
     &           u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,wff,ocom,mw2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,gww,ew32,ew32g,xw32g,yw32g,
     &           0,hff(3),mh2,h00g)
      call fvf(u5,n5,p5,m5,wff,ocom,zero,mw2,u2,n2,-p2,m2,
     &                                                 ew52,xw52,yw52)
      call ffvgnew(u5,n5,p5,xp5,yp5,cm5,ga5m,d57,
     &           u2,n2,-p2,-xp2,-yp2,cm2,ga2m,d27,wff,ocom,mw2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,gww,ew52,ew52g,xw52g,yw52g,
     &           0,hff(3),mh2,h00g)

      call fvf(u5,n5,p5,m5,gz5m,gz5p,zero,mz2,u4,n4,p4,m4,
     &                                                 ez54,xz54,yz54)
      call ffvgnew(u5,n5,p5,xp5,yp5,cm5,ga5m,d57,
     &           u4,n4,p4,xp4,yp4,cm4,ga4m,d47,gz5m,gz5p,mz2,
     &           e7,xe7,ye7,n7,p7,xp7,yp7,ocom,ez54,ez54g,xz54g,yz54g,
     &           0,hff(5),mh2,h00g)

c Amplitudes:

      call fefefef(u3,p3,xw16,yw16,wff,ocom,
     &             xp3+xp6-xp1,yp3+yp6-yp1,cm1,ocom,
     &             xe7,ye7,ga1m,ga1p,
     &            -xp4+xp2-xp5,-yp4+yp2-yp5,cm2,ocom,
     &             xw52,yw52,wff,ocom,u4,p4,
     &             m20_2,n3,n1*n6,n7,n5*n2,n4)

      call fefef(u3,p3,xw16g,yw16g,wff,ocom,
     &          -xp4+xp2-xp5,-yp4+yp2-yp5,cm1,ocom,
     &           xw52,yw52,wff,ocom,u4,p4,m20_3,n3,n1*n6*n7,n5*n2,n4)

      call fefef(u3,p3,xw16,yw16,wff,ocom,
     &           xp3+xp6-xp1,yp3+yp6-yp1,cm1,ocom,
     &           xw52g,yw52g,wff,ocom,u4,p4,m20_4,n3,n1*n6,n5*n2*n7,n4)

      call eee(-p1+p6+p7,ew16g,p2-p3,ew32,-p4-p5,
     &                   zww*ez54,m26_1,n1*n6*n7,n3*n2,n5*n4)

      call eee(-p1+p6,ew16,p2-p3-p7,ew32g,-p4-p5,
     &                   zww*ez54,m26_2,n1*n6,n3*n2*n7,n5*n4)

      call eee(-p1+p6,ew16,p2-p3,ew32,-p4-p5-p7,
     &              zww*ez54g,m26_3,n1*n6,n3*n2,n5*n4*n7)

      call eeee(ew32,ew16,gzww*ez54,cmplx(e7,zero,dp),
     &                                    m26_4,n3*n2,n1*n6,n5*n4,n7)

      call fefefef(u1,-p1,xe7,ye7,ga1m,ga1p,xp7,yp7,cm1,d17,
     &             xz54,yz54,gz1m,gz1p,
     &             xp2-xp3-xp6,yp2-yp3-yp6,cm1,ocom,
     &             xw32,yw32,wff,ocom,u6,p6,m27_1,n1,n7,n5*n4,n3*n2,n6)

      call fefefef(u1,-p1,xz54,yz54,gz1m,gz1p,
     &            -xp1+xp4+xp5,-yp1+yp4+yp5,cm1,ocom,
     &             xe7,ye7,ga1m,ga1p,
     &             xp2-xp3-xp6,yp2-yp3-yp6,cm1,ocom,
     &             xw32,yw32,wff,ocom,u6,p6,m27_2,n1,n5*n4,n7,n3*n2,n6)

      call fefef(u1,-p1,xz54g,yz54g,gz1m,gz1p,
     &           xp2-xp3-xp6,yp2-yp3-yp6,cm1,ocom,
     &           xw32,yw32,wff,ocom,u6,p6,m27_3,n1,n5*n4*n7,n3*n2,n6)

      call fefef(u1,-p1,xz54,yz54,gz1m,gz1p,
     &          -xp1+xp4+xp5,-yp1+yp4+yp5,cm1,ocom,
     &           xw32g,yw32g,wff,ocom,u6,p6,m27_4,n1,n5*n4,n3*n2*n7,n6)


      call fefefef(u1,-p1,xe7,ye7,ga1m,ga1p,xp7,yp7,cm1,d17,
     &             xw32,yw32,wff,ocom,
     &            -xp4-xp5-xp6,-yp4-yp5-yp6,ocom,ocom,
     &             xz54,yz54,gz6m,gz6p,u6,p6,m28_1,n1,n7,n3*n2,n5*n4,n6)

      call fefef(u1,-p1,xw32g,yw32g,wff,ocom,
     &          -xp4-xp5-xp6,-yp4-yp5-yp6,ocom,ocom,
     &           xz54,yz54,gz6m,gz6p,u6,p6,m28_2,n1,n3*n2*n7,n5*n4,n6)

      call fefef(u1,-p1,xw32,yw32,wff,ocom,
     &          -xp1-xp2+xp3,-yp1-yp2+yp3,ocom,ocom,
     &           xz54g,yz54g,gz6m,gz6p,u6,p6,m28_3,n1,n3*n2,n5*n4*n7,n6)

      call fefefef(u3,p3,xw16,yw16,wff,ocom,
     &            -xp1+xp3+xp6,-yp1+yp3+yp6,cm2,ocom,
     &             xe7,ye7,ga2m,ga2p,
     &             xp2-xp4-xp5,yp2-yp4-yp5,cm2,ocom,
     &             xz54,yz54,gz2m,gz2p,
     &                               u2,-p2,m32_1,n3,n1*n6,n7,n5*n4,n2)

      call fefefef(u3,p3,xw16,yw16,wff,ocom,
     &            -xp1+xp3+xp6,-yp1+yp3+yp6,cm2,ocom,
     &             xz54,yz54,gz2m,gz2p,
     &             xp7,yp7,cm2,d27,xe7,ye7,ga2m,ga2p,
     &                               u2,-p2,m32_2,n3,n1*n6,n5*n4,n7,n2)

      call fefef(u3,p3,xw16g,yw16g,wff,ocom,
     &           xp2-xp4-xp5,yp2-yp4-yp5,cm2,ocom,
     &           xz54,yz54,gz2m,gz2p,u2,-p2,m32_3,n3,n1*n6*n7,n5*n4,n2)

      call fefef(u3,p3,xw16,yw16,wff,ocom,
     &          -xp1+xp3+xp6,-yp1+yp3+yp6,cm2,ocom,
     &           xz54g,yz54g,gz2m,gz2p,
     &                               u2,-p2,m32_4,n3,n1*n6,n5*n4*n7,n2)

      call fefefef(u3,p3,xz54,yz54,gz3m,gz3p,
     &             xp3+xp4+xp5,yp3+yp4+yp5,ocom,ocom,
     &             xw16,yw16,wff,ocom,
     &             xp7,yp7,cm2,d27,xe7,ye7,ga2m,ga2p,
     &             u2,-p2,m34_1,n3,n5*n4,n1*n6,n7,n2)

      call fefef(u3,p3,xz54g,yz54g,gz3m,gz3p,
     &           xp1+xp2-xp6,yp1+yp2-yp6,ocom,ocom,
     &           xw16,yw16,wff,ocom,u2,-p2,m34_2,n3,n5*n4*n7,n1*n6,n2)

      call fefef(u3,p3,xz54,yz54,gz3m,gz3p,
     &           xp3+xp4+xp5,yp3+yp4+yp5,ocom,ocom,
     &           xw16g,yw16g,wff,ocom,u2,-p2,m34_3,n3,n5*n4,n1*n6*n7,n2)


c The spin averaged squared matrix element:


      do i1=1,2
      do i2=1,2
      do i3=1,n3
      do i4=1,n4
      do i5=1,n5
      do i6=1,n6
      do i7=1,2

      mat=
     &   -m20_2(i3,i1,i6,i7,i5,i2,i4)
     &   -m20_3(i3,i1,i6,i7,i5,i2,i4)
     &   -m20_4(i3,i1,i6,i5,i2,i7,i4)
     &   +m26_1(i1,i6,i7,i3,i2,i5,i4)
     &   +m26_2(i1,i6,i3,i2,i7,i5,i4)
     &   +m26_3(i1,i6,i3,i2,i5,i4,i7)
     &   -m26_4(i3,i2,i1,i6,i5,i4,i7)
     &   +m27_1(i1,i7,i5,i4,i3,i2,i6)
     &   +m27_2(i1,i5,i4,i7,i3,i2,i6)
     &   +m27_3(i1,i5,i4,i7,i3,i2,i6)
     &   +m27_4(i1,i5,i4,i3,i2,i7,i6)
      mat=mat
     &   +m28_1(i1,i7,i3,i2,i5,i4,i6)
     &   +m28_2(i1,i3,i2,i7,i5,i4,i6)
     &   +m28_3(i1,i3,i2,i5,i4,i7,i6)
     &   +m32_1(i3,i1,i6,i7,i5,i4,i2)
     &   +m32_2(i3,i1,i6,i5,i4,i7,i2)
     &   +m32_3(i3,i1,i6,i7,i5,i4,i2)
     &   +m32_4(i3,i1,i6,i5,i4,i7,i2)
     &   +m34_1(i3,i5,i4,i1,i6,i7,i2)
     &   +m34_2(i3,i5,i4,i7,i1,i6,i2)
     &   +m34_3(i3,i5,i4,i1,i6,i7,i2)

      matg(i1,i2,i3,i4,i5,i6,i7)=mat


      end do
      end do
      end do
      end do
      end do
      end do
      end do

      end 

