      subroutine psneeffff(ite,itw,iw34,iw56,ecm,mv1,gamv1,mv2,gamv2,
     &                     p1,m1,p2,m2,p3,m3,p4,m4,p5,m5,p6,m6,dlips)
c----------------------------------------------------------------------c
c Returns the phase space normalization, dlips for a 2 --> 4  process 
c the c.m.s. 
c
c                   p1 + p2 --> p3 + p4 + p5 + p6
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c
c Input: ite = 0/1(2) if 1/t pole related to the eeg-vertex is 
c                     absent/present in the electron (positron) channel,
c        ite = 3 if 1/t pole related to the eeg-vertex is 
c                     absent,
c        itw = 0 if the angle of a f3+f4 pair is uniformly generated,
c              1 if a f3+f4 pair is parallel to the initial e+,
c              2 if a f3+f4 pair is parallel to the initial e-,
c        iw34 = 0 if the W --> f3 f4 is off the resonance,
c               1 0 the W --> f3 f4 is in the resonance,
c        iw56 = 0 if the W --> f5 f6 is off the resonance,
c               1 0 the W --> f5 f6 is in the resonance,
c        ecm - the c.m.s. energy,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 26 April, 2000; modified on 29 May 2000, 9 October 2000; 5 May, 2001;
c                                 29 January, 2004
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer ite,itw,iw34,iw56

      real(dp) :: ecm,mv1,gamv1,mv2,gamv2,m1,m2,m3,m4,m5,m6,
     &           s34,s56,dlips

      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6

      real(dp) :: lamsq,s34min,s34max,ds34,s56min,s56max,ds56,pi,dphi,
     &           ct34,sqs34,sqs56,pp34,cps,cj,a34,ob34,rb34,dct34,
     &           gvmv1,mv12,gvmv2,mv22,srr,
     &           s346,s346min,s346max,ds346,sqs346,m52,ct5,dct5,pp5,
     &           a5,da5,pp,m12,m22

      real(dp) :: ecm0=0.d0

      save pi,dphi,m12,m22,cps,cj,ecm0


      if (ecm.ne.ecm0) then

c Integration limits:

      pi=4.d0*atan(1.d0)

      dphi=2.d0*pi

      m12=m1**2
      m22=m2**2

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(dphi)**7/4.d0*pi**2

      ecm0=ecm

      end if


      gvmv1=gamv1*mv1
      mv12=mv1**2
      gvmv2=gamv2*mv2
      mv22=mv2**2

      if (ite == 0) then

      s34min=(m3+m4)**2

      s34max=(ecm-m5-m6)**2

      s34=srr(p3+p4,p3+p4)
      if (s34 < s34min) then
       s34=s34min
      end if

      if (iw34 == 0) then
       ds34=s34max-s34min
      else if (iw34 == 1) then
       ds34=((s34-mv12)**2+gvmv1**2)
     & /(gvmv1/(atan((s34max-mv12)/gvmv1)-atan((s34min-mv12)/gvmv1)))
      else if (iw34 == 2) then
       ds34=s34*log(s34max/s34min)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

      s56min=(m5+m6)**2
      s56max=(ecm-sqs34)**2
ckk>
       if(s56max < s56min) then
        write(*,*)'alarm1',s56max,s56min
       end if
ckk<

      s56=srr(p5+p6,p5+p6)

      if (s56 < s56min) then
       s56=s56min
      else if (s56 > s56max) then
       s56=s56max
      end if

      if (iw56 == 0) then
       ds56=s56max-s56min
      else if (iw56 == 1) then
       ds56=((s56-mv22)**2+gvmv2**2)
     & /(gvmv2/(atan((s56max-mv22)/gvmv2)-atan((s56min-mv22)/gvmv2)))
      else if (iw56 == 2) then
       ds56=s56*log(s56max/s56min)
      else
       write(*,*)'iw56 =',iw56,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)

      pp34=sqrt(lamsq(ecm,sqs34,sqs56))/(2.d0*ecm)

      ct34=(p3(3)+p4(3))/pp34

      if (ct34 >= 1.d0) then
       ct34=1.d0
      else if (ct34 <= -1.d0) then
       ct34=-1.d0
      end if

      if (itw == 0) then
       dct34=2.d0
      else
       a34=(ecm*sqrt(s34+pp34**2)-s34)/(ecm*pp34)
       ob34=a34-1.d0
       if (abs(ob34) < 1.d-9) then
        ob34=2.d0*s34*s56/(ecm**2-s34-s56)**2
        rb34=ob34/(a34+1.d0)
       else
        rb34=ob34/(a34+1.d0)
       end if

       if (itw == 1) then
        dct34=-(a34-ct34)*log(rb34)
       else if (itw == 2) then
        dct34=-(a34+ct34)*log(rb34)
       else
        write(*,*)'psneeffff.f: itw =',itw,' IS NOT ALLOWED'
        stop
       end if

      end if

c Phase space normalization and Jakobian:

      dlips=cps*pp34/ecm
     &     *sqrt(lamsq(sqs34,m3,m4))/(2.d0*s34)
     &     *sqrt(lamsq(sqs56,m5,m6))/(2.d0*s56)
     &     *ds34*ds56*dct34

      else if (ite > 0) then

c s346=(p3+p4+p6)^2

      s346min=(m3+m4+m6)**2
      s346max=(ecm-m5)**2
      ds346=s346max-s346min

      s346=srr(p3+p4+p6,p3+p4+p6)
      sqs346=sqrt(s346)

c p5(0) in the c.m.s.:

      m52=m5**2
      p5(0)=(ecm**2-s346+m52)/(2.d0*ecm)

      pp5=sqrt(p5(0)**2-m52)

c ct5 in the c.m.s.:

      ct5=p5(3)/pp5

      if (ct5 < -1.d0) then
       ct5=-1.d0
      else if (ct5 > 1.d0) then
       ct5=1.d0
      end if

      if (ite < 3) then

      if (ite == 1) then
       da5=(p2(0)-p5(0))*(m52*p2(0)-m22*p5(0))/(2.d0*(p2(0)*p5(0))**2)
      else if (ite == 2) then
       da5=(p1(0)-p5(0))*(m52*p1(0)-m12*p5(0))/(2.d0*(p1(0)*p5(0))**2)
      end if

      if (da5 < 1.d-14) then
       da5=1.d-14
      end if

      if (da5 < 1.d-9) then
       a5=1.d0+da5
      else
       if (ite == 1) then
        pp=abs(p2(3))
        a5=(p2(0)*p5(0)-(m22+m52)/2.d0)/(pp*pp5)
       else if (ite == 2) then
        pp=abs(p1(3))
        a5=(p1(0)*p5(0)-(m12+m52)/2.d0)/(pp*pp5)
       end if
       da5=a5-1.d0
      end if

      if (ite == 1) then
       dct5=log((a5+1.d0)/da5)*(a5+ct5)
      else if (ite == 2) then
       dct5=log((a5+1.d0)/da5)*(a5-ct5)
      end if

      else ! ite >= 3

      dct5=2.d0

      end if ! ite

c s34=(p3+p4)^2

      s34=srr(p3+p4,p3+p4)
      sqs34=sqrt(s34)

      s34min=(m3+m4)**2
      s34max=(sqs346-m6)**2

      if (iw34 == 0) then
       ds34=s34max-s34min
      else if (iw34 == 1) then
       if (s34 < s34min) then
        s34=s34min
       end if
       ds34=((s34-mv12)**2+gvmv1**2)
     & /(gvmv1/(atan((s34max-mv12)/gvmv1)-atan((s34min-mv12)/gvmv1)))
      else if (iw34 == 2) then
       ds34=s34*log(s34max/s34min)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

c The phase space normalization:

      dlips=cps*pp5/ecm*ds346*dct5*ds34
     &     *sqrt(lamsq(sqs346,sqs34,m6))/(2.d0*s346)
     &     *sqrt(lamsq(sqs34,m3,m4))/(2.d0*s34)

      else ! ite

      write(*,*)'ite =',ite,' is not allowed'
      stop

      end if ! ite

      end


