      subroutine psneeffffg(ibr,ict,ite,itw,iw34,iw56,ie3,ict3,ecm,ecut,
     &      mv1,gamv1,mv2,gamv2,p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,p4,ps4,m4,
     &            p5,ps5,m5,p6,ps6,m6,p7,ps7,dlips)
c----------------------------------------------------------------------c
c Returns the phase space normalization, dlips for a 2 --> 5  process 
c the c.m.s. 
c                p1 + p2 --> p3 + p4 + p5 + p6 + p7,
c
c where p7 is considered massless.
c The phase space integral for massless particles is given by
c          Lips=(2.d0/3.d0)**2/(8.d0*pi)**7*(ecm**2)**3.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p). 
c
c        p1,p2,p3,p4,p5,p6,p7 - the usual four momentum components of 
c                               a particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: ibr = 1, for the initial state bremssrahlung,
c              2, for the initial state bremssrahlung,
c        ict = 0 if the initial state photon angle is uniformly 
c                distributed,
c              1 if the initial state photon angle is distributed as
c                1/(1-betae*ct),
c              2 if the initial state photon angle is distributed as
c                1/(1+betae*ct),
c        ite = 0/1 if 1/t pole related to the eeg-vertex is absent/present,
c        itw = 0 if the angle of a f3+f4 pair is uniformly generated,
c              1 if a f3+f4 pair is parallel to the initial e+,
c              2 if a f3+f4 pair is parallel to the initial e-,
c        iw34 = 0 if the W --> f3 f4 is off the resonance,
c               1 0 the W --> f3 f4 is in the resonance,
c        iw56 = 0 if the W --> f5 f6 is off the resonance,
c               1 0 the W --> f5 f6 is in the resonance,
c        ie3 = 0 if energy of particle 3 is uniformly generated,
c              1 if it is generated according to 1/(p4.p7),
c        ict3 = 0/1 if 1/t pole related to the eeg-vertex is absent/present
c               in the 3 body decay channel,
c        ecm - the c.m.s. energy,
c        ecut - lower cut on the energy of the photon,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        me - electron mass,
c        m3,...,m6 - final particle masses.
c
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 9 May, 1999; modified on 30 May 2000; 5 May, 2001; 2 February, 2004
c----------------------------------------------------------------------c

      use cutvalues

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer imom,ibr,iw34,iw56,ie3,ite,itw,ict,ict3

      real(dp) :: ecm,ecut,mv1,gamv1,mv2,gamv2,m1,m2,m3,m4,m5,m6,dlips
      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p34
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7

c Local variables:

      integer :: i,nsph

      real(dp) :: ecm0=0.d0,s,m12,m22,m32,m42,m52,m34,m346,m56,msum,
     &           lamsq,
     &           e7max,e7,de7,s34min,s34max,s56min,s56max,pi,pi2,dphi,
     &           cps10,cps11,cps20,cps21,cps347,dm34,e4,e4max,e7num

      real(dp) s34,s56,dct34,ct34,ct3,ct5,dct5,phi3,phi5

      real(dp) sp,sqsp,cf7,sf7,ds34,sqs34,ds56,sqs56,
     &        st34,st3,cf3,sf3,st5,cf5,sf5,pst,cf34,sf34

      real(dp) pp34,e56,pp3,pp32,pp2,pp5,p0,px,py,pz,ob34,rb34,g34,
     &        a34,b56,g56,gamma,beta,e3456,b347,a347,g347,
     &        p34x,p34y,p34z,p347(0:3),p56(0:3),p3x,p3y,p3z

      real(dp) :: betae,dbe2,dbe,rbe,nct7,ct7,dct7
      real(dp) :: gvmv1,mv12,gvmv2,mv22

      real(dp) ds347,s347,s347min,s347max,sqs347,pp347,e347,dct347,
     &        ct347,st347,cf347,sf347,dlips56,dlips347,ecutp,ob347,
     &        rb347,s3467,sqs3467,e3467,b3467,g3467,v3467(1:3),
     &        pv3467,dlips3467,p2p(1:3),p3p(1:3),p5p(1:3),p7p(1:3)

      real(dp) ecm3,e3max,e3,de3,ce3,re3,e7min,srr,dct3,da3,a3

      real(dp) v56(1:3),pv56,v347(1:3),pv347,v3456(1:3),pv,
     &        s346min,s346max,ds346,s346,
     &        sqs346,g3456,pv3456,e2,p2z,e5,de5,p5z,da5,a5,db5,beta5,
     &        dlips346,dlips34

      save s,m12,m22,dbe2,dbe,betae,rbe,pi,pi2,dphi,
     &     cps10,cps11,cps20,cps21,cps347,ecm0

      if (ecm.ne.ecm0) then

      s=ecm**2

c Initial momenta for m1=m2:

      m12=m1**2
      m22=m12

      dbe2=4.d0*m12/s
      dbe=dbe2/2.d0
      betae=sqrt(1.d0-dbe2)

      if (icuts == 1) then
       rbe=(1.d0+ctgb)/(1.d0-ctgb)
      else
       rbe=(1.d0+betae)**2/dbe2
      end if

      pi=4.d0*atan(1.d0)
      pi2=pi**2
      dphi=2.d0*pi

c Constant parts of the phase space and Jakobian:

      cps10=1.d0/(dphi)**10/2.d0/4.d0**3*(2.d0)**2*(dphi)**3
      cps11=cps10/2.d0**2
      cps20=1.d0/dphi**10/4.d0
      cps21=cps20*2.d0
      cps347=dphi**2/8.d0

      ecm0=ecm

      end if ! ecm0

      gvmv1=gamv1*mv1
      mv12=mv1**2
      gvmv2=gamv2*mv2
      mv22=mv2**2

      m32=m3**2
      m42=m4**2

      m34=m3+m4
      m56=m5+m6
      msum=m34+m56

c Kinematic variables:

      if (ibr == 1) then

c energy and angles of particle 7 in the CMS:

      e7max=(ecm-msum)*(ecm+msum)/(2.d0*ecm)

      de7=p7(0)*log(e7max/ecut)

c      dct7=(1.d0-ps7(1)**2+ps7(1)**2*dbe2)/(betae/log(rbe))

      if (ict == 0) then
        dct7=2.d0
      else
       nct7=betae/log(rbe)
       if (ict == 1) then
        ct7=ps7(1)
        if (ct7 >= 1.d0) then
         dct7=dbe/nct7
        else if (ct7 <= -1.d0) then
         ct7=-1.d0
         dct7=(2.d0-dbe)/nct7
        else
c         dct7=(1.d0-betae*ct7)/nct7
         dct7=(1.d0-ct7+dbe*ct7)/nct7
        end if
       else if (ict == 2) then
        ct7=ps7(1)
        if (ct7 <= -1.d0) then
         dct7=dbe/nct7
        else if (ct7 >= 1.d0) then
         ct7=1.d0
         dct7=(2.d0-dbe)/nct7
        else
c         dct7=(1.d0+betae*ct7)/nct7
         dct7=(1.d0+ct7-dbe*ct7)/nct7
        end if
       else
        write(*,*)'ict=',ict,' IS NOT ALLOWED'
       end if
      end if


c sp=(p3+p4+p5+p6)^2:

      sp=ecm*(ecm-2.d0*p7(0))
      sqsp=sqrt(sp)

      if (ite == 0) then

c s34=(p3+p4)^2:

      s34min=(m34)**2
      s34max=(sqsp-m56)**2

ckk>
      if (s34max < s34min) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: s34max < s34min',s34max,s34min 
       return
      end if
ckk<
      s34=srr(p3+p4,p3+p4)

ckk>
      if (s34 > s34max) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: s34 > s34max',s34,s34max 
       return
      else if (s34 < s34min) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: s34 < s34min',s34,s34min 
       return
      end if
ckk<

      if (iw34 == 0) then
       ds34=s34max-s34min
      else if (iw34 == 1) then
       ds34=((s34-mv12)**2+gvmv1**2)
     &  /(gvmv1/(atan((s34max-mv12)/gvmv1)-atan((s34min-mv12)/gvmv1)))
      else if (iw34 == 2) then
       ds34=s34*log(s34max/s34min)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)
      pp3=sqrt(lamsq(sqs34,m3,m4))/(2.d0*sqs34)

ckk>
      if (pp3 <= 0.d0) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: pp3 < 0',pp3
       return
      end if
ckk<


c s56=(p5+p6)^2:

ckk>
      if (sqs34 > sqsp) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: sqs34 > sqsp',sqs34,sqsp 
       return
      end if
ckk<
      s56min=m56**2
      s56max=(sqsp-sqs34)**2

ckk>
      if (s56max < s56min) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: s56max < s56min',s56max,s56min 
       return
      end if
ckk<

      p56=p5+p6
      s56=srr(p56,p56)

ckk>
      if (s56 > s56max) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: s56 > s56max',s56,s56max 
       return
      else if (s56 < s56min) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: s56 < s56min',s56,s56min 
       return
      end if
ckk<
      if (iw56 == 0) then
       ds56=s56max-s56min
      else if (iw56 == 1) then
       ds56=((s56-mv22)**2+gvmv2**2)
     & /(gvmv2/(atan((s56max-mv22)/gvmv2)-atan((s56min-mv22)/gvmv2)))
      else if (iw56 == 2) then
       ds56=s56*log(s56max/s56min)
      else
       write(*,*)'iw56 =',iw56,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)
      pp5=sqrt(lamsq(sqs56,m5,m6))/(2.d0*sqs56)

ckk>
      if (pp5 <= 0.d0) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: pp5 < 0',pp5
       return
      end if
ckk<

c the length of p3+p4 in the frame where p3+p4+p5+p6=0:
 
      pp34=sqrt(lamsq(sqsp,sqs34,sqs56))/(2.d0*sqsp)

ckk>
      if (pp34 <= 0.d0) then
       dlips=0.d0
       write(*,*)'psneeffffg.f: pp34 < 0',pp34
       return
      end if
ckk<

c polar angle of p3+p4 in the cms of particles 3,4,5 and 6, p3+p4+p5+p6=0,
c azimuthal angle is assumed to be zero, ie. the three momentum p3+p4 is 
c generated in the xOz plane in this frame:


      if (itw == 0) then

       dct34=2.d0

      else

       a34=(sqsp*sqrt(s34+pp34**2)-s34)/(sqsp*pp34)
       ob34=a34-1.d0

       if (abs(ob34) < 1.d-9) then
        ob34=2.d0*s34*s56/(sp-s34-s56)**2
        rb34=ob34/(a34+1.d0)
       else
        rb34=ob34/(a34+1.d0)
       end if

c Boost to the the frame where p3+p4+p5+p6=0:

        e3456=ecm-p7(0)
        gamma=e3456/sqsp
        beta=-p7(0)/e3456

        v3456=beta*(/ps7(2)*ps7(3),ps7(2)*ps7(4),ps7(1)/)

        pv=dot_product(v3456,p3(1:)+p4(1:))
        p0=p3(0)+p4(0)
        p34z=p3(3)+p4(3)+gamma*(gamma*pv/(gamma+1.d0)-p0)*v3456(3)

        ct34=p34z/pp34

       if (itw == 1) then
        dct34=-(a34-ct34)*log(rb34)
       else if (itw == 2) then
        dct34=-(a34+ct34)*log(rb34)
       else
        write(*,*)'psneeffffg.f: itw =',itw,' IS NOT ALLOWED'
        stop
       end if

      end if ! itw


c Phase space normalization including jakobian
c                          (=de7*dct7*ds34*ds56*(2.d0)**3*(dphi)**3)

      dlips=cps10*p7(0)*pp34/sqsp*pp3/sqs34*pp5/sqs56
     &                                      *de7*dct7*dct34*ds34*ds56


      else if (ite == 1) then

c s346=(p3+p4+p6)^2

      s346min=(m34+m6)**2
      s346max=(sqsp-m5)**2
      ds346=s346max-s346min

      p34=p3+p4

      s346=srr(p34+p6,p34+p6)
      sqs346=sqrt(s346)

c s34=(p3+p4)^2

      s34min=(m34)**2
      s34max=(sqs346-m6)**2

      s34=srr(p34,p34)
      sqs34=sqrt(s34)

      if (iw34 == 0) then
       ds34=s34max-s34min
      else if (iw34 == 1) then
       if (s34 < s34min) then
        s34=s34min
       end if
       ds34=((s34-mv12)**2+gvmv1**2)
     & /(gvmv1/(atan((s34max-mv12)/gvmv1)-atan((s34min-mv12)/gvmv1)))
      else if (iw34 == 2) then
       ds34=s34*log(s34max/s34min)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

c Velocity of the c.m.s. of particles 3, 4, 5 and 6 in the c.m.s.:

      e3456=sqrt(sp+p7(0)**2)
      v3456=-p7(1:)/e3456
      g3456=e3456/sqsp

c Boost p2 and p5 to the c.m.s. of particles 3, 4, 5 and 6:

      if (ict < 2) then
c       pv3456=dot_product(v3456,p2(1:))
       pv3456=v3456(3)*p2(3)
       p0=p2(0)
       e2=g3456*(p0-pv3456)
       p2p=p2(1:)+g3456*(g3456*pv3456/(g3456+1.d0)-p0)*v3456(1:)
      else 
       p2p=p2(1:)-p7(1:)
       pv3456=dot_product(v3456,p2p)
       p0=p2(0)-p7(0)
       e2=g3456*(p0-pv3456)
       p2p=p2p+g3456*(g3456*pv3456/(g3456+1.d0)-p0)*v3456
      end if

      pv3456=dot_product(v3456,p5(1:))
      p0=p5(0)
      e5=g3456*(p0-pv3456)
      p5p=p5(1:)+g3456*(g3456*pv3456/(g3456+1.d0)-p0)*v3456(1:)

c Momentum of particle 5 in the c.m.s. of particles 3, 4, 5 and 6:

c      pp5=sqrt(lamsq(sqsp,sqs346,m5))/(2.d0*sqsp)
      m52=m5**2
      pp5=sqrt(e5**2-m52)

      pp2=sqrt(p2p(1)**2+p2p(2)**2+p2p(3)**2)

      if (ict < 2) then

       da5=(e2-e5)*(m52*e2-m22*e5)/(2.d0*(e2*e5)**2)

       if (da5 < 1.d-9) then
        a5=1.d0+da5
       else
        a5=(e2*e5-(m22+m52)/2.d0)/(pp2*pp5)
        da5=a5-1.d0
       end if
      else
       a5=(e2*e5-(e2**2-pp2**2+m52)/2.d0)/(pp2*pp5)
       da5=a5-1.d0
       if (da5 < 1.d-14) then
        da5=1.d-14
        a5=1.d0+da5
       end if
      end if

      ct5=dot_product(p5p,p2p)/(pp5*pp2)

      if (ct5 < -1.d0) then
       ct5=-1.d0
      else if (ct5 > 1.d0) then
       ct5=1.d0
      end if

      dct5=log((a5+1.d0)/da5)*(a5-ct5)

c The phase space normalization including jakobian

      dlips=cps11*p7(0)*pp5/sqsp*de7*dct7*ds346*dct5*ds34
     &     *sqrt(lamsq(sqs346,sqs34,m6)*lamsq(sqs34,m3,m4))/(s346*s34)


      else ! ite

      write(*,*)'ite =',ite,' is not allowed'
      stop

      end if ! ite


      else if (ibr == 2) then

      if (ite == 0) then

c s347=(p3+p4+p7)^2:

      s347min=m34*(m34+2.d0*ecut)
      s347max=(ecm-m56)**2

      p347=p3+p4+p7
      s347=srr(p347,p347)

      if (iw34 == 0) then
       ds347=s347max-s347min
      else if (iw34 == 1) then
       ds347=((s347-mv12)**2+gvmv1**2)
     & /(gvmv1/(atan((s347max-mv12)/gvmv1)-atan((s347min-mv12)/gvmv1)))
      else if (iw34 == 2) then
       ds347=s347*log(s347max/s347min)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      sqs347=sqrt(s347)
      
c The length of p3+p4+p7=-p5-p6 in the CMS:
c s56=(p5+p6)^2:

      s56min=m56**2
      s56max=(ecm-sqs347)**2

      p56=p5+p6
      s56=srr(p56,p56)

      if (iw56 == 0) then
       ds56=s56max-s56min
      else if (iw56 == 1) then
       ds56=((s56-mv22)**2+gvmv2**2)
     & /(gvmv2/(atan((s56max-mv22)/gvmv2)-atan((s56min-mv22)/gvmv2)))
      else if (iw56 == 2) then
       ds56=s56*log(s56max/s56min)
      else
       write(*,*)'iw56 =',iw56,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)

      pp347=sqrt(lamsq(ecm,sqs347,sqs56))/(2.d0*ecm)


      if (pp347 <= 0.d0) then

       dlips=0.d0
       return

      else

c The spherical angles of p3+p4+p7 in the CMS:

      ct347=p347(3)/pp347

      if (ct347 >= 1.d0) then
       ct347=1.d0
       st347=0.d0
       cf347=1.d0
       sf347=0.d0
      else if (ct347 <= -1.d0) then
       ct347=-1.d0
       st347=0.d0
       cf347=-1.d0
       sf347=0.d0
      else
       st347=sqrt(1.d0-ct347**2)
       cf347=p347(1)/(pp347*st347)
       sf347=p347(2)/(pp347*st347)
      end if

c Necessary boost parameters:

      e347=sqrt(s347+pp347**2)
      b347=pp347/e347
      g347=e347/sqs347

      e56=sqrt(s56+pp347**2)
      b56=-pp347/e56
      g56=e56/sqs56

c Velocity of the c.m.s. of particles 3, 4 and 7 in the c.m.s.:

      v347=b347*(/st347*cf347,st347*sf347,ct347/)

      dlips56=pi*sqrt(lamsq(sqs56,m5,m6))/(2.d0*s56)

c Boost to the c.m.s. of particles 3, 4 and 7:

      pv347=dot_product(v347,p3(1:))
      e3=g347*(p3(0)-pv347)
      p3z=p3(3)+g347*(g347*pv347/(g347+1.d0)-p3(0))*v347(3)
      e7=g347*(p7(0)-dot_product(v347,p7(1:)))


      if (itw == 0) then

       dct347=2.d0

      else

       a347=(ecm*e347-s347)/(ecm*pp347)
       ob347=a347-1.d0

       if (abs(ob347) < 1.d-9) then
        ob347=2.d0*s347*s56/(s-s347-s56)**2
       end if

       rb347=ob347/(a347+1.d0)
 
       if (itw == 1) then
        dct347=-(a347-ct347)*log(rb347)
       else if (itw == 2) then
        dct347=-(a347+ct347)*log(rb347)
       else
        write(*,*)'psneeffffg.f: itw =',itw,' IS NOT ALLOWED'
        stop
       end if

      end if ! itw
      end if ! pp347

c The minimum value of ecut in the c.m.s. of particle 3, 4 and 7:

      ecutp=ecut*sqrt((1.d0-abs(b347))/(1.d0+abs(b347)))

      dm34=m32-m42

      if (ie3 == 0) then
       de3=sqrt(m32+lamsq(sqs347,m3,m4)/(4.d0*s347))-m3
      else if (ie3 == 1) then
       e4max=sqrt(m42+lamsq(sqs347,m3,m4)/(4.d0*s347))
       ce3=(sqs347+dm34/sqs347)/2.d0
       re3=m42/(2.d0*sqs347)*(ecutp/e4max)/(ce3-m3)
       de3=-(ce3-e3)*log(re3)
      else if (ie3 == 2) then
       e4max=sqrt(m42+lamsq(sqs347,m3,m4)/(4.d0*s347))
       e3max=sqrt(m32
     & +lamsq(sqs347,m3,m4*sqrt(1.d0+ecutp/e4max))/(4.d0*s347))
       ce3=(sqs347+dm34/sqs347)/2.d0
       re3=(e3max-m3)/(m42/(2.d0*sqs347)*(ecutp/e4max))
       de3=(ce3-e3)**2/((ce3-m3)/re3)
      else
       write(*,*)'ie3 =',ie3,' IS NOT ALLOWED'
       stop
      end if

      pp32=e3**2-m32

      pp3=sqrt(pp32)

c The photon energy:

      ecm3=sqs347-e3
      e7num=ecm3**2-pp32-m42

      e7min=max(ecutp,e7num/(2.d0*(ecm3+pp3)))

      e7max=e7num/(2.d0*(ecm3-pp3))

      if (e7max < e7min) then
       dlips=0.d0
       return
      end if

      if (e7 < e7min) then
       dlips=0.d0
       return
      else if (e7 > e7max) then
       dlips=0.d0
       return
      end if

      de7=e7*log(e7max/e7min)

      if (ict3 == 0) then

      dct3=2.d0

      else if (ict3 == 1) then

c      e2=g347*(p2(0)-dot_product(v347,p2(1:)))
      e2=g347*(p2(0)-v347(3)*p2(3))

      da3=m22/(e2*e3)

      a3=1.d0+da3

      ct3=p3z/sqrt(e3**2-m32)
      dct3=log((a3+1.d0)/da3)*(a3-ct3)

      else ! ict3

      write(*,*)'ict3 =',ict3,' is not allowed'
      stop

      end if ! ict3

      dlips347=cps347*de7*de3*dct3

c The phase space normalization:

      dlips=cps20*pp347/ecm*ds347*ds56*dct347*dlips347*dlips56


      else if (ite > 0) then

c p5 in the c.m.s.:

      m346=m34+m6
      m52=m5**2

      de5=((ecm-m346)*(ecm+m346)+m52)/(2.d0*ecm)-m5

      da5=(p2(0)-p5(0))*(m52*p2(0)-m22*p5(0))/(2.d0*(p2(0)*p5(0))**2)

      if (da5 < 1.d-9) then
       a5=1.d0+da5
      else
       a5=(p2(0)*p5(0)-(m22+m52)/2.d0)/(ps2(0)*ps5(0))
       da5=a5-1.d0
      end if

      dct5=log((a5+1.d0)/da5)*(a5+ps5(1))

c s3467=(p3+p4+p6+p7)^2:

      s3467=ecm*(ecm-2.d0*p5(0))+m52
      sqs3467=sqrt(s3467)

c Velocity of the c.m.s. of particles 3, 4, 6 and 7 in the c.m.s.:

      e3467=sqrt(s3467+ps5(0)**2)
      b3467=-ps5(0)/e3467
      v3467=b3467*(/ps5(2)*ps5(3),ps5(2)*ps5(4),ps5(1)/)
      g3467=e3467/sqs3467

      if (ite == 1) then

c s347=(p3+p4+p7)^2:

      s347min=m34**2
      s347max=(sqs3467-m6)**2
      p347=p3+p4+p7
      s347=srr(p347,p347)
      sqs347=sqrt(s347)

c Boost p3, p7 and p347 to the c.m.s. of particles 3, 4, 6 and 7:

      pv3467=dot_product(v3467,p3(1:))
      p0=p3(0)
      e3=g3467*(p0-pv3467)
      p3p=p3(1:)+g3467*(g3467*pv3467/(g3467+1.d0)-p0)*v3467

      pv3467=dot_product(v3467,p7(1:))
      p0=p7(0)
      e7=g3467*(p0-pv3467)
      p7p=p7(1:)+g3467*(g3467*pv3467/(g3467+1.d0)-p0)*v3467

      pv3467=dot_product(v3467,p347(1:))
      p0=p347(0)
      e347=g3467*(p0-pv3467)
      p347(1:)=p347(1:)+g3467*(g3467*pv3467/(g3467+1.d0)-p0)*v3467


      if (iw34 == 0) then
       ds347=s347max-s347min
      else if (iw34 == 1) then
       ds347=((s347-mv12)**2+gvmv1**2)
     & /(gvmv1/(atan((s347max-mv12)/gvmv1)-atan((s347min-mv12)/gvmv1)))
      else if (iw34 == 2) then
       ds347=s347*log(s347max/s347min)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

c p3+p4+p7 and p6 in the the c.m.s. frame of particles 3, 4, 6 and 7:

      pp347=sqrt(lamsq(sqs3467,sqs347,m6))/(2.d0*sqs3467)

      if (pp347 < 0.d0) then
       pp347=0.d0
      end if

c The minimum value of ecut in the c.m.s. of particle 3, 4 and 7:

      ecutp=ecut*sqrt((1.d0-abs(b3467))/(1.d0+abs(b3467)))

c Velocity of the c.m.s. of particles 3, 4 and 7 in the c.m.s. of particles
c 3, 4, 6 and 7:

      e347=sqrt(s347+pp347**2)
      b347=pp347/e347
      v347=p347(1:)/e347
      g347=e347/sqs347

      ecutp=ecutp*sqrt((1.d0-abs(b347))/(1.d0+abs(b347)))

c Boost p3 and p7 to the c.m.s. of particles 3, 4 and 7:

      e3=g347*(e3-dot_product(v347,p3p))

      e7=g347*(e7-dot_product(v347,p7p))

      dm34=m32-m42

      if (ie3 == 0) then
       de3=sqrt(m32+lamsq(sqs347,m3,m4)/(4.d0*s347))-m3
      else if (ie3 == 1) then
       e4max=sqrt(m42+lamsq(sqs347,m3,m4)/(4.d0*s347))
       ce3=(sqs347+dm34/sqs347)/2.d0
       re3=m42/(2.d0*sqs347)*(ecutp/e4max)/(ce3-m3)
       de3=-(ce3-e3)*log(re3)
      else if (ie3 == 2) then
       e4max=sqrt(m42+lamsq(sqs347,m3,m4)/(4.d0*s347))
       e3max=sqrt(m32
     & +lamsq(sqs347,m3,m4*sqrt(1.d0+ecutp/e4max))/(4.d0*s347))
       ce3=(sqs347+dm34/sqs347)/2.d0
       re3=(e3max-m3)/(m42/(2.d0*sqs347)*(ecutp/e4max))
       de3=(ce3-e3)**2/((ce3-m3)/re3)
      else
       write(*,*)'ie3 =',ie3,' IS NOT ALLOWED'
       stop
      end if

      pp32=e3**2-m32
      pp3=sqrt(pp32)

c The photon energy:

      ecm3=sqs347-e3
      e7num=ecm3**2-pp32-m42

      e7min=max(ecutp,e7num/(2.d0*(ecm3+pp3)))

      e7max=e7num/(2.d0*(ecm3-pp3))

      if (e7max < e7min) then
       dlips=0.d0
       return
      end if

      if (e7 < e7min) then
       dlips=0.d0
       return
      else if (e7 > e7max) then
       dlips=0.d0
       return
      end if

      de7=e7*log(e7max/e7min)

      dlips347=pi2*de7*de3

      dlips3467=pi*sqrt(lamsq(sqs3467,sqs347,m6))/(2.d0*s3467)

c The phase space normalization:

      dlips=cps21*ps5(0)*de5*dct5*ds347*dlips3467*dlips347


      else if (ite == 2) then

c s346=(p3+p4+p6)^2:

      s346min=m346**2
      ds346=s3467-s346min
      p34=p3+p4
      s346=srr(p34+p6,p34+p6)
      sqs346=sqrt(s346)

c Boost p5 and p7 to the c.m.s. of particles 3, 4, 6 and 7:

      pv3467=dot_product(v3467,p7(1:))
      p0=p7(0)
      e7=g3467*(p0-pv3467)
      ecutp=g3467*(ecut-pv3467)
      p7p=p7(1:)+g3467*(g3467*pv3467/(g3467+1.d0)-p0)*v3467


      if (e7 <= ecutp) then
       write(*,*)'1 e7 < 0',e7,ecutp
       dlips=0.d0
       return
      end if

c      pv3467=dot_product(v3467,p5(1:))
      pv3467=-dot_product(p5(1:),p5(1:))/e3467

      p0=p5(0)
      e5=g3467*(p0-pv3467)
      p5p=p5(1:)+g3467*(g3467*pv3467/(g3467+1.d0)-p0)*v3467

c p3+p4+p6 and p7 in the the c.m.s. frame of particles 3, 4, 6 and 7:

      db5=(m5/e5)**2
      beta5=sqrt(1.d0-db5)
      nct7=beta5/log((1.d0+beta5)**2/db5)

      ct7=dot_product(p5p,p7p)/(sqrt(e5**2-m52)*e7)

      if (ct7 >= 1.d0) then
       ct7=1.d0
       dct7=db5/(2.d0*nct7)
      else if (ct7 <= -1.d0) then
       ct7=-1.d0
       dct7=(4.d0-db5)/(2.d0*nct7)
      else
       dct7=(1.d0-ct7+db5*ct7/2.d0)/nct7
      end if

      dlips3467=pi/2.d0*e7/sqs3467*dct7

c Generate s34=(p3+p4)^2

      s34min=(m34)**2
      s34max=(sqs346-m6)**2

      s34=srr(p34,p34)
      sqs34=sqrt(s34)

      if (iw34 == 0) then
       ds34=s34max-s34min
      else if (iw34 == 1) then
       if (s34 < s34min) then
        s34=s34min
       end if
       ds34=((s34-mv12)**2+gvmv1**2)
     & /(gvmv1/(atan((s34max-mv12)/gvmv1)-atan((s34min-mv12)/gvmv1)))
      else if (iw34 == 2) then
       ds34=s34*log(s34max/s34min)
      else
       write(*,*)'iw34 =',iw34,' IS NOT ALLOWED'
       stop
      end if

      dlips346=pi*sqrt(lamsq(sqs346,sqs34,m6))/(2.d0*s346)

      dlips34=pi*sqrt(lamsq(sqs34,m3,m4))/(2.d0*s34)

c The phase space normalization:

      dlips=cps21*ps5(0)*de5*dct5*ds346*dlips3467*ds34*dlips346*dlips34


      else ! ite=1/ite=2

      write(*,*)'ite =',ite,' is not allowed'
      stop

      end if ! ite=1/ite=2

      end if ! ite

      else ! ibr

      write(*,*)'ibr =',ibr,' IS NOT ALLOWED'
      stop

      end if ! ibr


      end subroutine


