      subroutine psneetbffg(ipr,iv1,iv2,ecm,ecut,mv1,gamv1,mv2,gamv2,
     &                      p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,p4,ps4,m4,
     &                      p5,ps5,m5,p6,ps6,m6,p7,ps7,dlips)
c----------------------------------------------------------------------c
c Returns phase space normalization, dlips, of
c a bremsstrahlung to 2 --> 4 process in the c.m.s. 
c
c                   p1 + p2 --> p3 + (p4 + p5 + p6) + p7
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: ipr = 1,..,6 -specifies which particle radiates a photon,
c        iv1 = 0/1 or 2 if the intermediate boson V1 is off/in resonance
c                                                            or massless,
c        iv2 = 0/1 or 2 if the intermediate boson V2 is off/in resonance
c                                                            or massless
c        ecm - the c.m.s. energy,
c        ecut - lower cut on the energy of the photon,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 13 July, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 
      use cutvalues

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: ipr,iv1,iv2

      real(dp) :: ecm,ecut,mv1,gamv1,mv2,gamv2,m1,m2,m3,m4,m5,m6,
     &           msum,s456,s56,dlips

      real(dp), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p56
      real(dp), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7
      real(dp), dimension(1:3) :: v

      real(dp) :: pp,lamsq,s456min,s456max,ds456,s56min,s56max,
     &           ds56,dlips56,gvmv1,mv12,gvmv2,mv22,
     &           pp3,m32,sqs456,sqs56,
     &           srr,sp,sqsp,de7,e7,nct7,dct7

      real(dp) :: ecm0=0.d0,s,m12,m22,dbe2,dbe,betae,rbe,dphi,cps

      save s,m12,m22,dbe2,dbe,betae,rbe,dphi,cps,ecm0,nct7

      if (ecm /= ecm0) then

      dphi=2.d0*pi

      s=ecm**2

c Initial momenta for m1=m2:

      m12=m1**2
      m22=m12

      dbe2=(m1/(ecm/2.d0))**2
      dbe=dbe2/2.d0
      betae=1.d0-dbe

      if (icuts == 1) then
       rbe=(1.d0+ctgb)/(1.d0-ctgb)
      else
       rbe=(1.d0+betae)**2/dbe2
      end if

      nct7=betae/log(rbe)

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(2.d0*pi)**11

      ecm0=ecm

      end if ! ecm0

      msum=m3+m4+m5+m6

      if (ipr < 3) then

c energy and angles of a photon in the c.m.s.:

      de7=p7(0)*log((ecm-msum)*(ecm+msum)/(2.d0*ecm*ecut))

      if (ipr == 1) then
       dct7=(1.d0-ps7(1)+dbe*ps7(1))/nct7
      else if (ipr == 2) then
       dct7=(1.d0+ps7(1)-dbe*ps7(1))/nct7
      end if ! ipr = 1/2

c sp=(p3+p4+p5+p6)^2:

      sp=ecm*(ecm-2.d0*p7(0))
      sqsp=sqrt(sp)

c s456=(p4+p5+p6)^2:

      s456=srr(p4+p5+p6,p4+p5+p6)
      s456min=(m4+m5+m6)**2
      s456max=(sqsp-m3)**2
      if (s456 < s456min) then
       dlips=zero
       return
      end if

      if (iv1 == 0 .or. (iv1 == 1 .and. gamv1 == 0.d0)) then
       ds456=s456max-s456min
      else if (iv1 == 1) then
       gvmv1=gamv1*mv1
       mv12=mv1**2
       ds456=((s456-mv12)**2+gvmv1**2)/
     &  (gvmv1/(atan((s456max-mv12)/gvmv1)-atan((s456min-mv12)/gvmv1)))
      else if (iv1 == 2) then
       ds456=s456*log(s456max/s456min)
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs456=sqrt(s456)

c p3 in the c.m.s.:

      m32=m3**2
      pp3=sqrt(((sp-s456+m32)/(2.d0*sqsp))**2-m32)

c s56=(p5+p6)^2:

      s56=srr(p5+p6,p5+p6)
      s56min=(m5+m6)**2
      s56max=(sqs456-m4)**2
      if (s56 < s56min) then
       dlips=zero
       return
      end if

      if (iv2 == 0 .or. (iv2 == 1 .and. gamv2 == 0.d0)) then
       ds56=s56max-s56min
      else if (iv2 == 1) then
       gvmv2=gamv2*mv2
       mv22=mv2**2
       ds56=((s56-mv22)**2+gvmv2**2)
     & /(gvmv2/(atan((s56max-mv22)/gvmv2)-atan((s56min-mv22)/gvmv2)))
      else if (iv2 == 2) then
       ds56=s56*log(s56max/s56min)
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)

c The phase space normalization:

      dlips=cps*(pi*p7(0)*de7*dct7)*(pi*pp3/sqsp)*ds456*ds56
     &     *pi*sqrt(lamsq(sqs456,sqs56,m4))/(2.d0*s456)
     &     *pi*sqrt(lamsq(sqs56,m5,m6))/(2.d0*s56)

      end if !ipr

      end


