      subroutine usp(u,s,gp,gm,pp,pm,m,den,ugen,n,ns)
c----------------------------------------------------------------------c
c Returns a generalized spinor:
c
c           ugen(i)=u(i)*(gp*P+ + gm*P-)*(p+m)/den
c
c Input: u - a (bar) spinor,
c        s - a generalized scalar,
c        gp,gm - complex chirality couplings of the scalar boson to 
c                the fermion, 
c        pp,pm - intermidiate fermion momentum in the spinorial 
c                representation,
c        m - an intermidiate fermion mass including the width, relevant 
c            only if the corresponding denominator den=0,
c        den - the denominator of the fermion propagator,
c        n,ns - numbers of polarization states.
c
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 13 November, 2000
c----------------------------------------------------------------------c

      implicit none
      integer, parameter :: dp=kind(1.d0)

      integer :: n,ns
      complex(dp) :: m,den,gp,gm
      complex(dp), dimension(2,2) :: pp,pm
      complex(dp) :: u(4,n),s(ns),ugen(4,n,ns)

      integer i,is
      complex(dp), parameter :: ocom=(0.d0,0.d0)
      complex(dp) :: cden,cgp,cgm,mgp,mgm,ug(4)
      real(dp) p2

      if (den == ocom) then
c       p2=dble(pp(1,1)*pp(2,2)-pp(1,2)*pp(2,1))
c       cden=cmplx( p2-m**2,m*gam,8)
       cden=pp(1,1)*pp(2,2)-pp(1,2)*pp(2,1)-m**2
      else
       cden=den
      end if

      cgp=gp/cden
      cgm=gm/cden
      mgp=m*cgp
      mgm=m*cgm

      do i=1,n
       ug(:2)=mgm*u(:2,i)+cgp*matmul(u(3:,i),pm)
       ug(3:)=mgp*u(3:,i)+cgm*matmul(u(:2,i),pp)
       do is=1,ns
       ugen(:,i,is)=ug*s(is)
       end do
      end do

      end subroutine
